/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Base/PIAlloc.cpp,v $
 * $Date: 2001/04/07 13:30:17 $
 *
 Description:
	Link in override to built-in memory management.
\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>

#include <assert.h>
#include "PIAlloc.h"

/*
** Implemement default handler functions for new, delete, PIBeforeNew
** and PIBeforeDelete
*/
/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void *PIDummyNew( size_t tSize )
{
	assert( tSize );
	void *pV = malloc( tSize );
	if ( !pV )
		{
		assert( 0 );
		abort();
		};
	return pV;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIDummyDelete( void *pVoid )
{
	if ( pVoid ) free( pVoid );
}

/*
** Declare hook functions for new, and delete.
** this allows users of this library to use thier own implementations
** of these functions
*/
void *(* hook_new)( size_t tSize )=PIDummyNew;
void (* hook_delete)( void * )=PIDummyDelete;

/*
** A value of 0 here removes all custom memory management.
*/
#if 1

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void *operator new( size_t tS )
{
	return (hook_new)( tS );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void operator delete( void *pData )
{
	(hook_delete)( pData );
}

#endif

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIDummyBeforeNew( int, const char * )
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIDummyBeforeDelete( int, const char * )
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIDummyMark( int, const char * )
{
}

void (* hook_PIBeforeNew)(int, const char*)=PIDummyBeforeNew;
void (* hook_PIBeforeDelete)(int, const char*)=PIDummyBeforeDelete;
void (* hook_PIMark)(int, const char*)=PIDummyMark;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIBeforeNew( int iLine, const char *pFileName )
{
	(hook_PIBeforeNew)( iLine, pFileName );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIBeforeDelete( int iLine, const char *pFileName )
{
	(hook_PIBeforeDelete)( iLine, pFileName );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void PIMark( int iLine, const char *pFileName )
{
	(hook_PIMark)( iLine, pFileName );
}

