/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Base/StrToken.cpp,v $
 * $Date: 2001/04/07 13:30:17 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "DeQuote.h"
#include "StrToken.h"
#include "PIAlloc.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
StringTokenizer::StringTokenizer()
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
StringTokenizer::StringTokenizer( const StringTokenizer &tDiv )
:	theString( tDiv.GetString() )
{
	int iSize = tDiv.NumTokens();
	for(int i=0; i<iSize; i++)
		{
		lStrings.Append( (DblList::type)PI_NEW(PIString(tDiv.GetToken(i)) ) );
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
StringTokenizer::StringTokenizer(const char *pDividend, const char *pDivisor,
		int iFlags )
:	theString( pDividend )
{
	Divide( pDividend, pDivisor, iFlags );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
StringTokenizer::StringTokenizer(const char *pDividend, char cDivisor,
		int iFlags )
:	theString( pDividend )
{
	char szDivisor[2];
	szDivisor[0]=cDivisor;
	szDivisor[0]='\0';
	Divide( pDividend, szDivisor, iFlags );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void StringTokenizer::Divide(const char *pDividend, const char *pDivisor,
		int iFlags )
{
	assert( pDividend && pDivisor );
	if ( !pDividend || !pDivisor )
		{ return; };
	int iSubLen = strlen( pDivisor );
	int iLen = strlen(pDividend);
	char *pS = PI_NEW( char[iLen+1] );
	strcpy( pS, pDividend );

	int iInQuotes = 0;
	const char *pTmp = pS;
	for( int i=0; i<iLen; )
		{
		switch( iInQuotes )
			{
			case 0:
				if ( pS[i]=='"' && !(iFlags & IGNORE_QUOTES) )
					{ iInQuotes = 1; }
				else if ( !strncmp( &(pS[i]), pDivisor, iSubLen ) )
					{
					pS[i]='\0';
					i+=iSubLen;
					if ( !(iFlags&IGNORE_QUOTES) )
						{ lStrings.Append( (DblList::type)
							PI_NEW(PIString(DeQuote(pTmp))) ); }
					else
						{ lStrings.Append( (DblList::type)
							PI_NEW(PIString(pTmp)) ); };
					pTmp = &(pS[i]);
					continue;
					};
				break;

			case 1:
				if ( pS[i]=='"' )
					{ iInQuotes = 0; };
				break;

			default:
				assert( 0 );
			};	
		i++;
		};
	if ( !(iFlags&IGNORE_QUOTES) )
		{ lStrings.Append( (DblList::type)PI_NEW(PIString(DeQuote(pTmp))) ); }
	else
		{ lStrings.Append( (DblList::type)PI_NEW(PIString(pTmp)) ); };

	PI_DELETE( pS );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
StringTokenizer::~StringTokenizer()
{
	for(DblListIterator i(lStrings); 
			!i.BadIndex(); 
			i++)
		{ PI_DELETE( (PIString *)i.Current() ); };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
const PIString &StringTokenizer::GetToken(int i) const
{
	if ( i<NumTokens() )
		{
		return *((PIString *)lStrings[i]);
		}
	else
		{
		assert( 0 );
		return PIString::Empty();
		}
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
const int StringTokenizer::NumTokens() const
{
	return lStrings.Size();
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void StringTokenizer::AppendToken( const PIString &s)
{
	lStrings.Append( (DblList::type)PI_NEW(PIString(s)) );
}

