/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/DB/PIConfDB.h,v $
 * $Date: 2001/04/07 13:30:18 $
 *
 Description:
\*____________________________________________________________________________*/
/* $HeaderTop:$ */

#ifndef PICONFDB_H_
#define PICONFDB_H_

#include "_PIBase.h"
#include "_PIDBKey.h"
#include "PIString.h"

class ConfigDBParser;
/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class PIConfigDBString
{
private:
	const char *pSourceFile;
	long lX;
	long lY;
	PIString sString;

public:
	PIConfigDBString( const char *pC, long lTheX, long lTheY,
		const char *pTheSource )
		:	pSourceFile( pTheSource ), lX( lTheX ), lY( lTheY ), sString( pC )
		{};
	inline const PIString &GetString() const		{ return sString; };
	inline const char *GetFileName() const			{ return pSourceFile; };
	inline long GetX() const						{ return lX; };
	inline long GetY() const						{ return lY; };
	void WriteInfo( ostream &os ) const
		{
			os << sString << '\t' << "/* " << pSourceFile << ": "
				<< lY << ", " << lX << " */";
		};
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class PIConfigDB : public _PIBaseDB
{
public:
private:
	/* --- typeframes for the built-in types --- */
/*	static TypeFrame aTypeFrames[ NUM_TYPES ]; */
 
    bool LoadConfigurationFile( const char *pFileName );

protected:
	virtual ConfigDBParser *CreateParser( const char *pFileName );

public:
	PIConfigDB( _PIDB *pParent, const char *pName=PIDBKEY_CONFIGURATION );
	virtual ~PIConfigDB()		{};

	virtual bool Load( const char *pFileName );

	/* --- implement API functions --- */
	static int LoadConfigurationFile( _PIDB *pDB, const char *pFileName,
		PIConfigDB **ppConfigDB );
	static PIConfigDB *FindConfiguration( _PIDB *pDB );
	static PIConfigDB *LookupTree( PIConfigDB *pDB, const char *pKey,
		char *pInfoBuffer, int iLen );
	static const char *LookupValue( PIConfigDB *pDB, const char *pKey,
		char *pInfoBuffer, int iLen );
	static PIConfigDB *LookupObjectWithName( PIDB *pDB, PIConfigDB *pConfig,
		const char *pName );

	/* --- public static data --- */
	static TypeFrame tConfigLineTypeFrame;
};

#endif	/* PICONFDB_H_ */

