/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/DB/_pidb.cpp,v $
 * $Date: 2001/04/07 13:30:18 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include "_PIDB.h"
#include "AutoDel.h"

#include <iostream.h>
#define D { cerr << __FILE__ << ": " << __LINE__ << endl; };

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIDB::_PIDB( _PIDB *pTheParent )
:	pParent( pTheParent )
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
	Implements the API function. Checks to make sure this tree is the
	root node of the tree hierarchy before deleting it. If not it
	returns PIAPI_EINVAL.
\*____________________________________________________________________________*/
int _PIDB::Delete()
{
	if ( GetParent() )
		{ return PIAPI_ERROR; };
	PI_DELETE( this );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
_PIDB *_PIDB::GetRoot()
{
	_PIDB *pDB = this;
	while( pDB->GetParent() )
		{
		pDB = pDB->GetParent();
		assert( pDB );
		};
	return pDB;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
bool _PIDB::Detach()
{
	if ( !GetParent() )
		{ return false; };

	/* --- make an iterator and iterate to find the node --- */
	_PIDBIterator *pIter = 0;
	AutoDelete<_PIDBIterator> tA( pIter = GetParent()->GetIterator(
		PIDBTYPE_TREE, 0, 0 ) );
	(void)tA;

	if ( !pIter )
		{ return false; };

	for( ; pIter->AtValidElement(); pIter->Next() )
		{
		if ( pIter->Current( 0 ) == this )
			{
			PIMUSTBETRUE( pIter->DetachCurrent() == this );
			pParent = 0;
			return true;
			};
		};
	return false;
}

