/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/ISAPI/HandBase.cpp,v $
 * $Date: 2001/04/07 13:30:20 $
 *
 Description:
	Base class for all C++ handlers, provides a more comfortable interface
	above the raw Pi3 HTTP interface.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <ctype.h>

#include "HandBase.h"
#include "PIString.h"
#include "DeQuote.h"
#include "PICompat.h"
#include "HTTPCore.h"
#include "PIHTTP.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
HandlerBaseISAPI::HandlerBaseISAPI( PIObject *pTheObject )
:
	pObject( pTheObject ),
	iOK( 1 )
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void HandlerBaseISAPI::ReadParameters( int iArgc, const char *ppArgv[] )
{
	if ( !PIObject_readParameters( pObject, iArgc, ppArgv, ParameterFn,
		this ) )
		{
		iOK = 0;
		}
	else
		{
		iOK = 1;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
HandlerBaseISAPI::~HandlerBaseISAPI()
{
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int HandlerBaseISAPI::Execute( int iArgc, const char *ppArgv[] )
{
	PI_MARK;

	if ( iArgc!=1 )
		{ return PIAPI_ERROR; };	

	PIHTTP *pPIHTTP = (PIHTTP *)ppArgv[0];	
	if ( !pPIHTTP )
		{ return PIAPI_ERROR; };
	int iPhase = (int)pPIHTTP->ciPhase;

	int iRet = Handle(
		iPhase,
		*pPIHTTP,
		*( pPIHTTP->pBuffer )
		);
	PI_MARK;
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseISAPI_execute( PIObject *pObj, int iArgc,
		const char *ppArgv[] )
{
	if ( !pObj ) return PIAPI_ERROR;
#if WIN32
	return ((HandlerBaseISAPI *)PIObject_getUserData(pObj))->
		Execute( iArgc, ppArgv );
#elif POSIX
	PIHTTP *pPIHTTP = (PIHTTP *)*ppArgv;
#define MSG "\
<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"/icons/Pi3Tile.gif\">\
<H2>Not supported</H2>\
The required handler is not supported on this operating system.</BODY>"

	HTTPCore_sendGeneralHeaders( pPIHTTP );
	HTTPCore_sendEntityHeaders( pPIHTTP, pPIHTTP->pResponseDB );
	PIIOBuffer_write( pPIHTTP->pBuffer, MSG, sizeof(MSG)-1, PIIOBUF_NONE );
	return PIAPI_ERROR;
#endif
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseISAPI_onClassLoad( PIClass_LoadAction eLoad, void * )
{
	switch( eLoad )
		{
		case STARTUP:
		default:;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int HandlerBaseISAPI_constructor( PIObject *pObj, HandlerBaseISAPI *pHandler )
{
#if POSIX
	return PIAPI_COMPLETED;
#endif
	if (!( pHandler && pHandler->IsOK() ))
		{
		delete pHandler;
		return PIAPI_ERROR;
		};
	if ( PIObject_setUserData( pObj, pHandler ) )
		{
		delete pHandler;
		CONFIG_ERR( pObj, "HandlerBaseISAPI_constructor: PIObject_setUserData() failed" );
		return PIAPI_ERROR;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseISAPI_copyConstructor( PIObject *, int, const char *[] )
{
	/* --- forbid copy constructor --- */
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseISAPI_destructor( PIObject *pObj, int,
	const char *[] )
{
#if WIN32
	delete (HandlerBaseISAPI *)PIObject_getUserData( pObj );
#endif
	return PIAPI_COMPLETED;
}

