/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/ISAPI/ISAPI20.cpp,v $
 * $Date: 2002/04/07 16:16:43 $
 *
 Description:
	Execute an ISAPI plugin.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>

#if WIN32
#include <HTTPExt.h>
#endif


#include "HandBase.h"
#include "HTTPCore.h"
#include "HTTPUtil.h"
#include "PIStrStr.h"
#include "DeQuote.h"
#include "PiAPI.h"
#include "StrToken.h"

#if WIN32

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_EXTRAHDRPREFIX					"ExtraHeadersPrefix"
#define KEY_CONF_EXTRAHEADERSIGNORE				"ExtraHeadersIgnore"
#define KEY_CONF_EXTRAHEADERS					"ExtraHeaders"
#define VALUE_NO								"No"
#define VALUE_YES								"Yes"
#define KEY_CONF_VARIABLE						"Variable"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	ISAPI20

Description:
	Load and execute an ISAPI (Internet Server API) extension. You can find more
	informations about ISAPI filters in the <A HREF="http://msdn.microsoft.com/library/psdk/iisref/isgu0q0n.htm">
	MSDN library</A>.

Options:

<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>Variable
<TD>-
<TD>&lt;Pi3Expression&gt;
<TD>A variable definition
<TD>Variable "GATEWAY_INTERFACE=CGI/1.1"

<TR>
<TD>ExtraHeaders
<TD>Yes
<TD>Yes|No
<TD>Indicates if extra headers are considered
<TD>ExtraHeaders Yes

<TR>
<TD>ExtraHeadersPrefix
<TD>-
<TD>&lt;A string&gt;
<TD>Used as prefix of each extra header
<TD>ExtraHeadersPrefix &quot;HTTP_&quot;

<TR>
<TD>ExtraHeadersIgnore
<TD>-
<TD>&lt;Space delimited Strings&gt;
<TD>List of unconsidered extra headers
<TD>ExtraHeadersIgnore &quot;Content-Type Content-Length&quot;

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>Variable</H5>
Specifies a server variable expression to be set into the buffer
of ISAPI extension calls to GetServerVariable. Could be set multiple
times in one configuration object.

<H5>ExtraHeaders</H5>
This configuration key defines, if extra headers are considered by
the processing of this handler.

<H5>ExtraHeadersPrefix</H5>
This configuration value will be the trailing string of each extra
request header.

<H5>ExtraHeadersIgnore</H5>
The headers in this list are not treated as extra headers.

Phase:
	HANDLE

Returns:
	PIAPI_COMPLETED, PIAPI_CONTINUE, PIAPI_ERROR, or INT_REDIRECT according
	to the status returned by the extension.

Note:
<H5>Parameters</H5>
The Pi3Expressions in the configuration variables may contain besides
the standard shortcuts used in Pi3Expressions the following, context
specific parameters.

<CENTER>
<TABLE BORDER=1>
<TH>Parameter<TH>Evaluates to<TR>
<TD>%a<TD>All extra headers, as required for header ALL_HTTP<TR>
<TD>%l<TD>Content-Length as DWORD, as required for ISAPI<TR>
</TABLE>
</CENTER>

Example:
	<PRE>
	&lt;Object&gt;
		Name ISAPI20
		Class ISAPI20Class
	&lt;/Object&gt;

	&lt;Object&gt;
		...
		Handle Condition="&cmp(&dblookup(response,string,ObjectMap,ISAPI),ISAPI)" \
			ISAPI20
		...
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Prototypes for callback functions
	Class for ISAPI context
\*____________________________________________________________________________*/
BOOL WINAPI fnGetServerVariable( HCONN	   hConn,
					LPSTR	   lpszVariableName,
					LPVOID	  lpvBuffer,
					LPDWORD	 lpdwSize );

BOOL WINAPI fnWriteClient( HCONN	  ConnID,
					LPVOID	 Buffer,
					LPDWORD	lpdwBytes,
					DWORD	  dwReserved );

BOOL WINAPI fnReadClient( HCONN	  hConn,
					LPVOID	 lpvBuffer,
					LPDWORD	lpdwSize );

BOOL WINAPI fnServerSupportFunction( HCONN	  hConn,
					DWORD	  dwHSERRequest,
					LPVOID	 lpvBuffer,
					LPDWORD	lpdwSize,
					LPDWORD	lpdwDataType );


/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Class for ISAPI context
\*____________________________________________________________________________*/
class ISAPI20;
class ISAPIContext : public _EXTENSION_CONTROL_BLOCK
{
public:
	ISAPI20 *pISAPI20;
	PIHTTP *pPIHTTP;
	int iRc;
	inline int Finish()
		{
		return PISync_unlock( sema );
		};
	inline int isFinished()
		{
		return PISync_lock( sema );
		};

private:
	PISync *sema;

	inline char *DBL( PIDB *pDB, int iType, const char *pKey )
		{
		char *pTmp = (char *)PIDB_lookup( pDB, iType, pKey, 0 );
		return pTmp ? pTmp : "";
		};

public:
	ISAPIContext( ISAPI20 *pTheISAPI20, PIHTTP *pThePIHTTP )
	:	pISAPI20( pTheISAPI20 ), pPIHTTP( pThePIHTTP ), iRc( PIAPI_COMPLETED ), sema( PIPlatform_allocLocalSemaphore(0,1) )
		{

		/* --- locals --- */
		PIDB *pQ = pPIHTTP->pRequestDB;
		PIDB *pR = pPIHTTP->pResponseDB;

		/* --- initialize member data --- */
		cbSize = sizeof( _EXTENSION_CONTROL_BLOCK );
		dwVersion = MAKELONG( HSE_VERSION_MINOR, HSE_VERSION_MAJOR );
		ConnID = (HCONN)this;
		dwHttpStatusCode = 0;
		*lpszLogData = '\0';

		/* --- input variables --- */
		lpszMethod = DBL( pQ, PIDBTYPE_STRING, KEY_HTTP_METHOD );
		lpszQueryString = DBL( pQ, PIDBTYPE_STRING, KEY_HTTP_QUERYSTRING );
		lpszPathInfo = DBL( pR, PIDBTYPE_STRING, KEY_INT_PATHINFO );
		lpszPathTranslated = DBL(pR, PIDBTYPE_STRING, KEY_INT_PATHTRANSLATED);
		lpszContentType = DBL( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE );

		/* --- check for stdin --- */
		cbAvailable = 0;
		lpbData = NULL;
		const char *pContentLength = DBL( pQ, PIDBTYPE_RFC822,
			KEY_HTTP_CONTENTLENGTH );
		cbTotalBytes = pContentLength ? atol( pContentLength ) : 0;

		/* --- callback functions --- */
		GetServerVariable = fnGetServerVariable;
		WriteClient = fnWriteClient;
		ReadClient = fnReadClient;
		ServerSupportFunction = fnServerSupportFunction;
		};

	virtual ~ISAPIContext()	{
		PISync_unlock( sema );
		PISync_delete( sema );
	};

};


/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class ISAPI20 : public HandlerBaseISAPI
{
private:
	/* ---
	Configuration data
	--- */
	PIDB *pTypes;					/* DB for variable types */
   	PIString sExtraHeadersPrefix;	/* Extra Header for environment values, */
									/* by default "HTTP_" */
	int iExtraHeaders;				/* whether or not to send extra headers */
	DblList lIgnoreHeaders;			/* extra headers to ignore */
	FnPi3Write aFunctions[256];		/* callback functions */
	DWORD dwPlatform;
	DWORD initVersion()
		{
		//
		//  Find out whether we are on NT or Win95/98
		//
		OSVERSIONINFO VerInfo;
		VerInfo.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
		return GetVersionEx( &VerInfo ) ? VerInfo.dwPlatformId : 0;
		};

	inline int checkIgnoreList(const char *pKey)
		{
		/* --- exclude the variables in lIgnoreHeaders --- */
		const char *pExclude;
		for( DblListIterator l( lIgnoreHeaders ); !l.BadIndex(); l++ )
			{
			pExclude = (const char *)l.Current();
			if (!PIUtil_stricmp(pKey, pExclude))
				{
				return 1;
				}
			};
		return 0;
		}

protected:
	/*___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ *

				Load an expression or condition

	 *___ +++++++++++++++++++++++++++++++++++++++++++++++++ ___ */
	int LoadExpression( const char *pName, Pi3Expression **ppExpression,
		FnPi3Write *pFunctions, const char *pValue, PIOStrStream &os )	
		{
		assert( ppExpression );
		Pi3Expression *pExpression = *ppExpression;
		if ( pExpression )
			{
			os << "'" << pName << "' may only be specified once." << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};
		Pi3String *pError = Pi3String_new( 0 );
		pExpression = Pi3Expression_new( pValue, pFunctions, pError );
		if ( !pExpression )
			{
			os << "Error parsing expression: " <<
				Pi3String_getPtr( pError ) << ends;
			CONFIG_ERR( Object(), os.str() );
			Pi3Expression_delete( pExpression );
			Pi3String_delete( pError );
			return 0;
			};
		Pi3String_delete( pError );
		*ppExpression = pExpression;
		return 1;
		};

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );
		PIOStrStream os;
		os << pWhere << "ISAPI: ";
		if ( !PIUtil_stricmp( KEY_CONF_EXTRAHDRPREFIX, pVariable ) )
			{
				sExtraHeadersPrefix = (const char *)DeQuote( pValue );
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTRAHEADERSIGNORE, pVariable ) )
			{
			StringTokenizer tTokens( (const char *)DeQuote(pValue), " " );
			for( int i=0; i<tTokens.NumTokens(); i++)
				{
				lIgnoreHeaders.Append( (DblList::type)
					PI_NEW( PIString( tTokens.GetToken( i ) ) ) );
				};
			}
		else if ( !PIUtil_stricmp( KEY_CONF_EXTRAHEADERS, pVariable ) )
			{
			if ( !PIUtil_stricmp( VALUE_NO, pValue ) )
				{ iExtraHeaders = 0; };
			}
		else if ( !PIUtil_stricmp( KEY_CONF_VARIABLE, pVariable ) )
			{
			const char *pK;
			const char *pV;
			StringTokenizer tTokens( (const char *)DeQuote(pValue), "=" );
			if (tTokens.NumTokens() < 2)
				{
				os << "Invalid Variable '" << pValue <<
					"'" << ends;
				CONFIG_ERR( Object(), os.str() );
				return 0;
				}

			pK = tTokens.GetToken( 0 );
			pV = tTokens.GetToken( 1 );

			Pi3String *pError = Pi3String_new( 0 );
			Pi3Expression *pExpr = Pi3Expression_new( pV, aFunctions,
				pError );
			if ( !pExpr )
				{
				os << Pi3String_getPtr( pError ) << ends;
				Pi3String_delete( pError );
				CONFIG_ERR( Object(), os.str() ); 
				return 0;
				};
			Pi3String_delete( pError );
			PIDB_add( pTypes, PIDBTYPE_OPAQUE, pK, (void *)pExpr, 0 );
			}
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

	/* ---
	Callback function for Pi3Expression parameter %a
	--- */
	static int f_a( PIHTTP *pPIHTTP, void *pData, char *pszBuffer, int iLength )
	{
		(void)pPIHTTP;
		ISAPIContext &tData = *( (ISAPIContext *)pData );
	    assert( pszBuffer );

		PIDB *pQ = pPIHTTP->pRequestDB;											
		int i = 0;																
	
		/* --- loop over all RFC822 headers in request block --- */				
		PIDBIterator *pIter = PIDB_getIterator( pQ, PIDBTYPE_RFC822, 0, 0 );	
		if ( !pIter ) { return 0; }

		const char *pKey;													
		const char *pValue;
		for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) )
			{																
			pValue = (const char *)PIDBIterator_current( pIter,	&pKey );
			assert( pKey && pValue );											

			if ( !pKey || !pValue ) { /* sanity */ continue; };										

			if (tData.pISAPI20->checkIgnoreList(pKey)) continue;

			void *fnEnv = PIDB_lookup( tData.pISAPI20->pTypes, PIDBTYPE_OPAQUE, pKey, 0 );
			if ( fnEnv ) { continue; };

			int iVarLen = strlen( pKey );										
			int iVarPos = tData.pISAPI20->sExtraHeadersPrefix.Len();								
			int iValSize = strlen( pValue );									

			// keep space for ': \r\n\0'
			if ( (i+iVarPos+iVarLen+iValSize+5) < iLength )							
				{																
				memcpy( &(pszBuffer[i]), tData.pISAPI20->sExtraHeadersPrefix, iVarPos );
				i += iVarPos;
				for( int j=0; j<iVarLen; j++, i++ )							
					{																
					if ( isalnum( pKey[j] ) )										
						{ pszBuffer[i]=toupper(pKey[j]); }						
					else															
						{ pszBuffer[i]='_'; };									
					};
				pszBuffer[i++] = ':';
				pszBuffer[i++] = ' ';
				memcpy( &(pszBuffer[i]), pValue, iValSize ); i+=iValSize;		
				pszBuffer[i++] = '\r';
				pszBuffer[i++] = '\n';											
				}																
			else																
				{																
				PIDBIterator_delete( pIter );
				::SetLastError( ERROR_INSUFFICIENT_BUFFER );				
				return 1024;													
				};																
			};
		PIDBIterator_delete( pIter );
		pszBuffer[i] = 0;											
		return i;															
	};

	/* ---
	Callback function for Pi3Expression parameter %l
	--- */
	static int f_l( PIHTTP *pPIHTTP, void *pData, char *pszBuffer, int iLength )
	{
	(void)pPIHTTP;
	ISAPIContext &tData = *( (ISAPIContext *)pData );
    assert( pszBuffer );

	*((DWORD *)pszBuffer) = tData.cbTotalBytes;
	pszBuffer[sizeof(DWORD)] = 0;
	return sizeof(DWORD);
	}


public:
	ISAPI20( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	HandlerBaseISAPI( pObject ),
		pTypes( PIDB_new( 0, "Types" ) ),
		iExtraHeaders( 1 ),
		dwPlatform( initVersion() )
		{
		/* --- clear out callback functions --- */
		memset( aFunctions, 0, sizeof( FnPi3Write ) * 256 );
		aFunctions['a'] = f_a;
		aFunctions['l'] = f_l;
		ReadParameters( iArgc, ppArgv );
		};

	virtual ~ISAPI20()
		{
		PIDB_delete( pTypes );
		for( DblListIterator i( lIgnoreHeaders ); !i.BadIndex(); i++ )
			{ PI_DELETE( (PIString *)i.Current() ); };
		};

	/* ---
	The real stuff
	--- */
	int RunISAPI( PIHTTP *pPIHTTP, PFN_HTTPEXTENSIONPROC fnHttpExtensionProc,
		const char *pPath )
		{
		assert( pPIHTTP );

		/* ---
		Create new ISAPI context
		--- */
		ISAPIContext tContext( this, pPIHTTP );

		/* ---
		Read in stdin (if any)
		--- */
		if ( tContext.cbTotalBytes )
			{
			/* ---
			See how much we can read on the first pass and leave the	
			extension for call ReadClient() for the rest		
			--- */
			int iRead = 0;
			const char *pBuffer = PIIOBuffer_read( pPIHTTP->pBuffer,
				&iRead );
			if ( !pBuffer || !iRead )
				{
				/* ---
				This is an error
				--- */
				HTTPCore_logError( pPIHTTP, "ISAPI20: Could not read client \
data (%ld bytes) for ISAPI DLL '%s'.", (long)tContext.cbTotalBytes, pPath );
				return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
				};
			
			/* --- 
			Allocate a buffer and put data in it, buffer size will be
			no more that internal buffer in PIIOBuffer object.
			(usually between 4 and 16 K).
			--- */
			tContext.lpbData = (LPBYTE)PIHTTP_allocMem( pPIHTTP, iRead );
			memcpy( tContext.lpbData, pBuffer, iRead );
			tContext.cbAvailable = iRead;
			};

		/* ---
		Change impersonation level (for ASP on WinNT)
		--- */
		if (dwPlatform == VER_PLATFORM_WIN32_NT) ImpersonateSelf(SecurityImpersonation);

		/* ---
		Fire off extension
		--- */
		int iRet = (fnHttpExtensionProc)( &tContext );

		/* ---
		Check for error message
		--- */
		if ( tContext.lpszLogData && *(tContext.lpszLogData) )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI dll with path '%s' \
logged an error message: ________________________________________________________________________________\n%s\n________________________________________________________________________________", pPath, tContext.lpszLogData );
			};
		
		pPIHTTP->iStatus = tContext.dwHttpStatusCode;

		switch( iRet )
			{
			case HSE_STATUS_SUCCESS:
				/*
				** Assume Content-Length could not be set, disable
				** keep open
				*/
				PIDB_replace( pPIHTTP->pConnectionDB, PIDBTYPE_OPAQUE,
					KEY_INT_KEEPOPEN, 0, 0 );
				break;

			case HSE_STATUS_SUCCESS_AND_KEEP_CONN:
				/*
				** OK
				*/
				break;

			case HSE_STATUS_PENDING:
				/*
				** We must wait until this asynchronous extension
				** has finished
				*/
				if ((tContext.isFinished() == PIAPI_ERROR) && HTTPCore_debugEnabled())
					{
					/*
					** The asynchronous extension was timed out
					*/
					HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension \
with path '%s' will be unloaded, wait for semaphore failed with error code %d",
						pPath, PIPlatform_getLastError());
					tContext.iRc = PIAPI_ERROR;
					break;
					}
				else
					{
					switch( tContext.iRc )
						{
						case 0:
						case HSE_STATUS_SUCCESS:
							/*
							** Assume Content-Length could not be set, disable
							** keep open
							*/
							PIDB_replace( pPIHTTP->pConnectionDB, PIDBTYPE_OPAQUE,
								KEY_INT_KEEPOPEN, 0, 0 );
							tContext.iRc = PIAPI_COMPLETED;
							break;
	
						case HSE_STATUS_SUCCESS_AND_KEEP_CONN:
							/*
							** OK
							*/
							tContext.iRc = PIAPI_COMPLETED;
							break;

						case HSE_STATUS_PENDING:
							/*
							** This is a heavy error, because even an asynchronous
							** extension must be finished here
							*/
							if ( HTTPCore_debugEnabled() )
								{
								HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension \
with path '%s' returned illegal status code HSE_STATUS_PENDING.", pPath );
								};

							tContext.iRc = PIAPI_ERROR;						
							break;

						case HSE_STATUS_ERROR:
							if ( HTTPCore_debugEnabled() )
								{
								HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension with path \
'%s' returned HSE_STATUS_ERROR.", pPath );
								};
							tContext.iRc = PIAPI_COMPLETED;	
							break;

						default:
							if ( HTTPCore_debugEnabled() )
								{
								HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension \
with path '%s' returned HSE_STATUS_ERROR or an unknown return code of %d.",
									pPath, tContext.iRc );
								};
							tContext.iRc = PIAPI_COMPLETED;
						};
					};
				break;

			case HSE_STATUS_ERROR:
				if ( HTTPCore_debugEnabled() )
					{
					HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension with path \
'%s' returned HSE_STATUS_ERROR.", pPath );
					};
				break;

			default:
				if ( HTTPCore_debugEnabled() )
					{
					HTTPCore_logError( pPIHTTP, "ISAPI20: ISAPI extension with path \
'%s' returned an unknown return code of %d.", pPath, iRet );
					};
			};

		return tContext.iRc;
		};

	/* ---
	The real stuff
	--- */
	int DoISAPI( PIDLL *pDLL, const char *pPath, PIHTTP *pPIHTTP )
		{
		assert( pDLL );

		PFN_GETEXTENSIONVERSION fnGetVersion = (PFN_GETEXTENSIONVERSION)
			PIDLL_getAddress( pDLL, "GetExtensionVersion" );
		if ( !fnGetVersion )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Could not load function \
'GetExtensionVersion' from ISAPI DLL '%s'.", pPath );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

		PFN_HTTPEXTENSIONPROC fnHttpExtensionProc = (PFN_HTTPEXTENSIONPROC)
			PIDLL_getAddress( pDLL, "HttpExtensionProc" );
		if ( !fnHttpExtensionProc )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Could not load function \
'HttpExtensionProc' from ISAPI DLL '%s'.", pPath );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

		PFN_TERMINATEEXTENSION fnTerminateExtension = (PFN_TERMINATEEXTENSION)
			PIDLL_getAddress( pDLL, "TerminateExtension" );

		/* ---
		No check, fnTerminateExtension is optional
		--- */

		/* ---
		Do GetExtensionVersion
		--- */
		HSE_VERSION_INFO tInfo;
		if ( !( (fnGetVersion)( &tInfo ) ) )
			{
			HTTPCore_logError( pPIHTTP, "ISAPI20: Call to \
'GetExtensionVersion' failed for ISAPI DLL '%s'. Win32 Error code is %d.",
				pPath , ::GetLastError() );
			return HTTPUtil_doHTTPError( pPIHTTP, ST_INTERNALERROR );
			};

		/* ---
		OK do the DLL processing
		--- */
		int iRet = RunISAPI( pPIHTTP, fnHttpExtensionProc, pPath );
	
		/* ---
		Unload DLL
		--- */
		if ( fnTerminateExtension )
			{
			(fnTerminateExtension)(HSE_TERM_MUST_UNLOAD);
			};
		
		return iRet;
		};

	int Handle( int iPhase, PIHTTP &tPIHTTP, PIIOBuffer &/* tBuffer */ )
		{
		if ( iPhase!=PH_HANDLE )
			{ return PIAPI_ERROR; };

		PIDB *pR = tPIHTTP.pResponseDB;
		PIDB *pQ = tPIHTTP.pRequestDB;

		/* ---
		Get file object
		--- */
		const char *pPath = (const char *)PIDB_lookup( pR, PIDBTYPE_STRING,
			KEY_INT_PATH, 0 );
		PIFInfo *pFInfo = HTTPCore_getCachedFile( pPath ); 
		if ( !pFInfo )
			{ return PIAPI_ERROR; };

		/* ---	
		This file must exist and be a regular file 
		--- */
		if ( !PIFInfo_exists( pFInfo ) ||
			!PIFInfo_isRegular( pFInfo ) )
			{
			HTTPCore_logError( &tPIHTTP, "ISAPI20: File with \
path '%s' does not \
exist or is not a regular file.", pPath );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		PIDLL *pDLL = PIDLL_new( pPath );
		if ( !PIDLL_isLoaded( pDLL ) )
			{
			const char *pDesc = PIDLL_getErrorDescription( pDLL );
			HTTPCore_logError( &tPIHTTP, "ISAPI20: ISAPI DLL with \
path '%s' could not be loaded, error description is \
'%s'.", pPath, pDesc );
			HTTPCore_releaseCachedFile( pFInfo );	
			return HTTPUtil_doHTTPError( &tPIHTTP, ST_INTERNALERROR );
			};

		int iRet = DoISAPI( pDLL, pPath, &tPIHTTP );
		PIDLL_delete( pDLL );

		HTTPCore_releaseCachedFile( pFInfo );

		return iRet;
		};

	BOOL GetServerVariable( PIHTTP *pPIHTTP, LPSTR lpszVariableName,
		LPVOID lpvBuffer, LPDWORD lpdwSize, ISAPIContext *pContext )
		{
		if ( !lpszVariableName || !lpdwSize )
			{
			::SetLastError( ERROR_INVALID_PARAMETER );
			return FALSE;
			};

		/* ---
		Convert variable to upper case
		--- */
		PIString sTmp( lpszVariableName );
		sTmp.ConvertToUpperCase();

		Pi3Expression *pExpr = (Pi3Expression *)PIDB_lookup(pTypes, PIDBTYPE_OPAQUE,
			(const char *)sTmp, 0);

		if (pExpr)
			{
			int iWritten = Pi3Expression_write( pExpr, pPIHTTP, pContext, (char *)lpvBuffer,
				*lpdwSize );
			if ( iWritten==-1 )
				{
				*lpdwSize = 0;
				::SetLastError( ERROR_INVALID_INDEX );
				return 0;
				}

			/*
			** Skip empty variable fields, this allows an environment
			** variable to be added conditionally
			*/
			if ( iWritten>=0 )
				{
				if ( iWritten>=*lpdwSize )
					{
					*lpdwSize = iWritten;
					::SetLastError( ERROR_INSUFFICIENT_BUFFER );
					return 0;
					};
				((char *)lpvBuffer)[iWritten] = '\0';	
				*lpdwSize = iWritten;
				return 1;
				};
			}

		/* --- no variable found, try to find an extra header --- */
		if ( iExtraHeaders ) 
			{
			PIDB *pQ = pPIHTTP->pRequestDB;											
	
			/* --- loop over all RFC822 headers in request block --- */				
			PIDBIterator *pIter = PIDB_getIterator( pQ, PIDBTYPE_RFC822, 0, 0 );	
			if ( !pIter ) { return 0; }

			const char *pKey;													
			const char *pValue;
			for(; PIDBIterator_atValidElement( pIter ); PIDBIterator_next( pIter ) )
				{
				pValue = (const char *)PIDBIterator_current( pIter,	&pKey );
				assert( pKey && pValue );											

				if ( !pKey || !pValue ) { /* sanity */ continue; };

				if (checkIgnoreList(pKey)) continue;

				PIString sTest(sExtraHeadersPrefix);
				sTest.Concatenate(pKey);
				char *pBuf = (char *)(const char *)sTest;
				for( int i=0; i<sTest.Len(); i++ )							
					{																
					if ( isalnum( pBuf[i] ) )										
						{ pBuf[i]=toupper(pBuf[i]); }						
					else															
						{ pBuf[i]='_'; };		
					}

				if (!PIUtil_stricmp((const char *)sTest, (const char *)sTmp))
					{
					strncpy((char *)lpvBuffer, pValue, strlen(pValue));
					*lpdwSize = strlen(pValue);
					((char *)lpvBuffer)[*lpdwSize] = '\0';	
					return 1;
					}
				}
			PIDBIterator_delete( pIter );

			*(char *)lpvBuffer = '\0';
			*lpdwSize = 0;
			::SetLastError( ERROR_INVALID_INDEX );
			return 0;
			}
		
		::SetLastError( ERROR_INVALID_INDEX );
		return 0;
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnGetServerVariable( HCONN hConn, LPSTR lpszVariableName,
	LPVOID lpvBuffer, LPDWORD lpdwSize )
{
	ISAPIContext *pContext = (ISAPIContext *)hConn;
	BOOL iRet = pContext->pISAPI20->GetServerVariable( pContext->pPIHTTP,
		lpszVariableName, lpvBuffer, lpdwSize, pContext );
	if ( HTTPCore_debugEnabled() )
		{
		HTTPCore_logDebug( DBG_MED, "ISAPI20: Return code<%d>, variable<%s>, \
value<%x>, size<%d>.",
			iRet, lpszVariableName ? lpszVariableName : "",
			(void *)lpvBuffer, lpdwSize ? *lpdwSize : 0 );
		};
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnWriteClient( HCONN ConnID, LPVOID Buffer, LPDWORD	lpdwBytes,
	DWORD dwReserved )
{
	if ( !ConnID || !lpdwBytes || !Buffer )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;
	int iWritten = PIIOBuffer_write( pContext->pPIHTTP->pBuffer,
		(char *)Buffer, *lpdwBytes, PIIOBUF_NONE );
	if ( iWritten==-1 )
		{
		/*
		** Socket must have been closed
		*/
		::SetLastError( ERROR_INVALID_HANDLE );
		return FALSE;
		};
	*lpdwBytes = iWritten;
	return TRUE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnReadClient( HCONN ConnID, LPVOID lpvBuffer, LPDWORD lpdwSize )
{
	if ( !ConnID || !lpdwSize )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;
	int iRet = PIIOBuffer_readToBuffer( pContext->pPIHTTP->pBuffer,
		(char *)lpvBuffer, *lpdwSize );
	if ( iRet>0 )
		{
		*lpdwSize = iRet;
		return TRUE;
		}
	else
		{
		return FALSE;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
BOOL WINAPI fnServerSupportFunction( HCONN ConnID, DWORD dwHSERRequest,
	LPVOID lpvBuffer, LPDWORD lpdwSize, LPDWORD	lpdwDataType )
{
	if ( !ConnID )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	ISAPIContext *pContext = (ISAPIContext *)ConnID;

	if ( !pContext )
		{
		::SetLastError( ERROR_INVALID_PARAMETER );
		return FALSE;
		};

	PIHTTP *pPIHTTP = pContext->pPIHTTP;

	switch( dwHSERRequest )
		{
		case HSE_REQ_SEND_URL_REDIRECT_RESP:
			if ( !lpvBuffer )
				{
				::SetLastError( ERROR_INVALID_PARAMETER );
				return FALSE;
				};
			PIDB_replace( pPIHTTP->pResponseDB, PIDBTYPE_RFC822,
				KEY_HTTP_LOCATION, lpvBuffer, 0 );
			pPIHTTP->iStatus = ST_PERMANENTREDIRECT;
			HTTPCore_sendGeneralHeaders( pPIHTTP );
			HTTPCore_sendEntityHeaders( pPIHTTP, pPIHTTP->pResponseDB );
			pContext->iRc = PIAPI_COMPLETED;
			return TRUE;

		case HSE_REQ_SEND_URL:
			if ( !lpvBuffer )
				{
				::SetLastError( ERROR_INVALID_PARAMETER );
				return FALSE;
				};
			pContext->iRc = HTTPUtil_doHTTPError( pPIHTTP, ST_PERMANENTREDIRECT);
			pPIHTTP->iStatus = ST_PERMANENTREDIRECT;
			PIDB_replace( pPIHTTP->pResponseDB, PIDBTYPE_STRING,
				KEY_INT_PATH, lpvBuffer, 0 );
			return TRUE;

		case HSE_REQ_SEND_RESPONSE_HEADER:
			{
			/*
			** Assume Content-Length could not be set, disable
			** keep open
			*/
			PIDB_replace( pPIHTTP->pConnectionDB, PIDBTYPE_OPAQUE,
				KEY_INT_KEEPOPEN, 0, 0 );
			int iStatus = lpvBuffer ? atoi( (char *)lpvBuffer ) : 200;	
			pPIHTTP->iStatus = iStatus;
			pContext->dwHttpStatusCode = iStatus;
			pContext->iRc = PIAPI_COMPLETED;

			/*
			** Send server specific headers
			*/
			HTTPCore_sendGeneralHeaders( pPIHTTP );
			if ( !PIHTTP_isChild( pPIHTTP ) && lpdwSize && *lpdwSize && lpdwDataType )
				{
				PIIOBuffer_write( pPIHTTP->pBuffer, (const char *)lpdwDataType,
					*lpdwSize, PIIOBUF_NONE );
				};
			};
			return TRUE;

		case HSE_REQ_DONE_WITH_SESSION:
			pContext->iRc = *(LPDWORD)lpvBuffer;
			if ((pContext->Finish() == PIAPI_ERROR) && HTTPCore_debugEnabled())
				{
				/*
				** The unlock of the semaphore failed somehow
				*/
				HTTPCore_logError( pPIHTTP, "ISAPI20: Unlock semaphore failed \
with error code %d", PIPlatform_getLastError());
				};
			return TRUE;

		default:
			/*
			** Unhandled/unknown function code
			*/
			if ( HTTPCore_debugEnabled() )
				{
				HTTPCore_logError( pPIHTTP, "ISAPI20: Server support function \
called with unknown function code %d", dwHSERRequest );
				};
		};
	return FALSE;
}

#endif /* WIN32 */

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int ISAPI20_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
#if WIN32
	return HandlerBaseISAPI_constructor( pObj, PI_NEW( ISAPI20( pObj,
		iArgc, ppArgv ) ) );
#elif POSIX
	(void)pObj;
	(void)iArgc;
	(void)ppArgv;
	return PIAPI_COMPLETED;
#endif
}


/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int ISAPI20_destructor( PIObject *pObj, int,
	const char *[] )
{
#if WIN32
	delete (ISAPI20 *)PIObject_getUserData( pObj );
#endif
	return PIAPI_COMPLETED;
}


#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name ISAPI20Class
		Type LogicExtension
		Library ISAPI
		OnClassLoad HandlerBaseISAPI_onClassLoad
		Constructor ISAPI20_constructor
		CopyConstructor HandlerBaseISAPI_copyConstructor
		Destructor ISAPI20_destructor
		Execute HandlerBaseISAPI_execute
	</Class>

	<Object>
		Name ISAPI20
		Class ISAPI20Class
	</Object>

/*___+++CNF_END+++___*/
#endif

