/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/Admin.cpp,v $
 * $Date: 2002/08/02 06:58:56 $
 *
 Description:

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <assert.h>
#include <fstream.h>
#include <ctype.h>
#include <stdio.h>

#include "QConfig.h"
#include "Pi3File.h"
#include "HTTPCore.h"
#include "PIStrStr.h"
#include "StrToken.h"
#include "DeQuote.h"
#include "Admin.h"
#include "PIStream.h"
#include "Base64.h"
#include "md5.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
class _Admin
{
private:
    int iRemote;

public:
    PIObject *pParentObject;
    ConnectionPool *pPool;
    PIString sFileName;
    PIString sHost;
    int iPort;
    PIString sUsername;
    PIString sPassword;
    PIString sRealm;
	cbAuthData *cbGetAuthData;
	void *pUserData;
private:
    int iOK;

public:
    _Admin(
        PIObject *pParent,
        const char *pHost,
        int iThePort,
        const char *pUsername,
        const char *pPassword,
		cbAuthData *pGetAuthData,
		void *pTheUserData )
    :
        iRemote( 1 ),
		pParentObject( pParent ),
        pPool( 0 ),
        sFileName(),
        sHost( pHost ),
        iPort( iThePort ),
        sUsername( pUsername ),
        sPassword( pPassword ),
		cbGetAuthData( pGetAuthData ),
		pUserData( pTheUserData ),
        iOK( 0 )
        {
        HTTPCore_initClient();

        PIObject *pIOObject = PIObject_loadFromLine(
            PIObject_getDB( pParent ),
            PIObject_getConfigurationDB( pParent ),
            "ClientIOObject" );

        if ( !pIOObject )
            { return; };

        pPool = ConnectionPool_new( pIOObject, 0 );

        iOK = 1;
        };

    _Admin( const char *pFileName )
    :
        iRemote( 0 ),
		pParentObject( 0 ),
        pPool( 0 ),
        sFileName( pFileName ),
        sHost(),
        iPort( -1 ),
        sUsername(),
        sPassword(),
		cbGetAuthData( 0 ),
		pUserData( 0 ),
        iOK( 1 )
        {};

    inline int IsOK() const         { return iOK; };
    inline int IsRemote() const     { return iRemote; };

    ~_Admin()
        {
        if ( pPool )
            { ConnectionPool_delete( pPool ); };

        if ( IsRemote() )
            { HTTPCore_cleanupClient(); };
        };
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static int Internal_loadConfigFromFile( QConfig *pConfig,
    const char *pFileName )
{
	assert( pFileName );

	fstream ifs( pFileName, ios::in | ios::nocreate );
	if ( !ifs || ifs.eof() )
		{ return -1; };
	
    int iRet = Pi3File_readConfig( pConfig, ifs );

    ifs.close();

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static int Internal_writeConfigToFile( QConfig *pConfig,
    const char *pFileName )
{
	assert( pFileName && pConfig );
	ofstream ofs( pFileName, ios::out );

	if ( ofs.bad() || ofs.eof() )
		{ return -1; };

    int iRet = Pi3File_writeConfig( pConfig, ofs );

	ofs.close();

	return iRet;	/* success */
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_BasicAuthentication( PIDB *pQ,
	const char *pUsername, const char *pPassword )
{
	PIString sTmp = pUsername;
	sTmp.Concatenate(':');
	sTmp.Concatenate(pPassword);

	PIString sResult = "Basic ";
	Base64Encode((const char *)sTmp, sResult);

	PIDB_replace( pQ, PIDBTYPE_RFC822, KEY_HTTP_AUTHORIZATION,
		(void *)(const char *)sResult, 0);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_DigestAuthentication( PIDB *pQ,
	const char *pAuthenticate, const char *pUsername, const char *pPassword)
{
	assert(pAuthenticate);

	/*
	** Check if Digest authentication applies
	*/
	if ( !strncmp( pAuthenticate, "Digest ", 7 ) )
		{
		PIString sNonce;
		PIString sRealm;

		const char *pURI = (const char *)PIDB_lookup( pQ,
			PIDBTYPE_STRING, KEY_HTTP_URI, 0);

		int iMethod = (int)PIDB_lookup( pQ,
			PIDBTYPE_OPAQUE, KEY_HTTP_METHOD, 0);

		/*
		** Get the challenge data from server string
		*/
		StringTokenizer tTokens( pAuthenticate + 7, "," );
		for(int i=0; i<tTokens.NumTokens(); i++)
			{
			const char *pToken = tTokens.GetToken(i);

			/* --- skip leading whitespace --- */
			for( ; *pToken && (isspace(*pToken)); pToken++ );

			if (strstr(pToken,KEY_NONCE))
				{
				sNonce = (const char *)DeQuote(
					strstr(pToken,KEY_NONCE)+sizeof(KEY_NONCE));
				}
			else if (strstr(pToken,KEY_REALM))
				{
				sRealm = (const char *)DeQuote(
					strstr(pToken,KEY_REALM)+sizeof(KEY_REALM));
				}
			}

		PIString sTmp = pUsername;
		PIString sHA1;
		if ( strstr(pPassword, "0x") != pPassword )
			{
			// A1
			sTmp.Concatenate(':');
			sTmp.Concatenate(sRealm);
			sTmp.Concatenate(':');
			sTmp.Concatenate(pPassword);

			// H(A1)
			MD5Encode((const char *)sTmp, sHA1);
			}
		else
			{
			sHA1 = pPassword+2;
			};

		// A2
		sTmp = pMethodMap[iMethod];
		sTmp.Concatenate(':');
		sTmp.Concatenate(pURI);

		// H(A2)
		PIString sHA2;
		MD5Encode((const char *)sTmp, sHA2);

		// H(A1) : nonce : H(A2)
		sTmp = sHA1;
		sTmp.Concatenate(':');
		sTmp.Concatenate(sNonce);
		sTmp.Concatenate(':');
		sTmp.Concatenate(sHA2);

		PIString sResult = "Digest username=\"";
		sResult.Concatenate(pUsername);
		sResult.Concatenate("\",realm=\"");
		sResult.Concatenate(sRealm);
		sResult.Concatenate("\",nonce=\"");
		sResult.Concatenate(sNonce);
		sResult.Concatenate("\",uri=\"");
		sResult.Concatenate(pURI);
		sResult.Concatenate("\",response=\"");

		MD5Encode((const char *)sTmp, sResult);
		sResult.Concatenate('"');

		PIDB_replace( pQ, PIDBTYPE_RFC822, KEY_HTTP_AUTHORIZATION,
		(void *)(const char *)sResult, 0);
		}
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static int Internal_GetAuthType( PIDB *pR )
{
	const char *pAuthenticate = (const char *)PIDB_lookup(
		pR, PIDBTYPE_RFC822, KEY_HTTP_WWW_AUTHENTICATE, 0 );

	int iAuthType = 0;
	if (pAuthenticate)
		{
		StringTokenizer tTokens( pAuthenticate, " " );
		for(int i=0; i<tTokens.NumTokens(); i++)
			{
			const char *pToken = tTokens.GetToken(i);

			/* --- skip leading whitespace --- */
			for( ; *pToken && (isspace(*pToken)); pToken++ );

			if (strstr(pToken,KEY_BASIC_AUTH))
				{
				iAuthType = ID_BASIC_AUTH;
				}
			else if (strstr(pToken,KEY_DGST_AUTH))
				{
				iAuthType = ID_DGST_AUTH;
				}
			else if (strstr(pToken,KEY_NTLM_AUTH))
				{
				iAuthType = ID_NTLM_AUTH;
				}
			}
		}
	return iAuthType;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static const char *Internal_GetRealm( const char *pAuthenticate )
{
	assert(pAuthenticate);

	StringTokenizer tTokens( pAuthenticate, " " );
	for(int i=0; i<tTokens.NumTokens(); i++)
		{
		const char *pToken = tTokens.GetToken(i);

		/* --- skip leading whitespace --- */
		for( ; *pToken && (isspace(*pToken)); pToken++ );

		if (strstr(pToken,KEY_REALM))
			{
			char *pRealm = strstr(pToken,KEY_REALM)+sizeof(KEY_REALM);
			if (pRealm[strlen(pRealm)-1] != '"')
				{ pRealm[strlen(pRealm)-1] = '\0'; }
			return (const char *)DeQuote(pRealm);
			}
		}
	return NULL;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
    Returns a PIAPI succes code or an HTTP request status, in any case
    0 on success, otherwise an error.
\*____________________________________________________________________________*/
static int Internal_makeHTTPRequest(
    PIIOBuffer *pBuffer,
    int iMethod,
    const char *pAdminCommand,
    const char *pHost,
    int iPort,
    const char *pContentType,
    const char *pContentLength,
    int (* fnPutData)( void *pParam ), void *pPutDataParam,
    int (* fnGetData)( void *pParam ), void *pGetDataParam,
    const char *pUsername,
    const char *pPassword,
	PIString &sAuthenticate,
	int *iAuthType,
    PIString &sError
    )
{
    assert( pBuffer );

    const char *pCLF;

    PIDB *pQ = PIDB_new( 0, "Q" );
    PIDB *pR = PIDB_new( 0, "R" );

    /*
    ** Add some headers
    */
    PIDB_add( pQ, PIDBTYPE_OPAQUE, KEY_HTTP_PROTOCOL, (void *)PR_HTTP10, 0 );
    PIDB_add( pQ, PIDBTYPE_OPAQUE, KEY_HTTP_CONNECTION, (void *)0, 0 );
    PIDB_add( pQ, PIDBTYPE_OPAQUE, KEY_HTTP_METHOD, (void *)iMethod, 0 );
    PIDB_add( pQ, PIDBTYPE_STRING, KEY_HTTP_URI, "/", 0 );
    PIDB_add( pQ, PIDBTYPE_RFC822, KEY_HTTP_X_ADMINCOMMAND,
        (void *)pAdminCommand, 0 );
	if (atoi(pContentLength))
		{
		PIDB_add( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTTYPE,
			(void *)pContentType, 0 );
		PIDB_add( pQ, PIDBTYPE_RFC822, KEY_HTTP_CONTENTLENGTH,
			(void *)pContentLength, 0 );
		}
    PIDB_add( pQ, PIDBTYPE_RFC822, "User-Agent",
        (void *)"Pi3Web-Client/1.0", 0);

    /* ---
    Make sure the 'host' header is sent
    --- */
    PIOStrStream os;
    os << pHost;
    if ( iPort != HTTP_DEFAULT_PORT )
        {
        os << ":" << iPort;
        };
    os << ends;
    PIDB_replace( pQ, PIDBTYPE_RFC822, KEY_HTTP_HOST,
        (void *)os.str(), 0 );

	/*
	** Compute the authentication headers
	*/
	switch (*iAuthType)
		{
		case ID_BASIC_AUTH:
			Internal_BasicAuthentication(
				pQ,
				pUsername,
				pPassword
			);
			break;

		case ID_DGST_AUTH:
			Internal_DigestAuthentication(
				pQ,
				(const char *)sAuthenticate,
				pUsername,
				pPassword
			);
			break;

		default:;
		}

    /*
    ** Make the request
    */
    int iRet = HTTPUtil_doHTTPRequest( pBuffer, pQ );
    if ( iRet != PIAPI_COMPLETED )
        {
        sError = "Failed to connect to remote server '";
        sError.Concatenate( pHost );
        sError.Concatenate( "'." );
        return iRet;
        };

    /* --- more data to send? --- */
    if ( fnPutData!=0 )
        {
        iRet = (fnPutData)( pPutDataParam );
        if ( iRet )
            {
            sError = "Error sending data to remote server '";
            sError.Concatenate( pHost );
            sError.Concatenate( "'." );
            return iRet;
            };
        };

    /* --- send the whole thing on its way --- */
    PIIOBuffer_flush( pBuffer );

    /*
    ** Read the response
    */
    iRet = HTTPCore_readHeaders( pBuffer, pR, RH_RESPONSE );
    if ( iRet )
        {
        sError = "Error reading response headers from remote server '";
        sError.Concatenate( pHost );
        sError.Concatenate( "'." );
        return iRet;
        };
    
    /* ---
    Check out status line from remote server
    --- */
    if ( ( pCLF = (const char *)
        PIDB_lookup( pR, PIDBTYPE_STRING, KEY_HTTP_CLF, 0 ) ) )
        {
        /* ++++ __________________________ +++ *
            Got status line
         * ++++ __________________________ +++ */
        /* --- grap the status code sent by the server --- */
        int i=0;
        for(; pCLF[i] && !(isspace(pCLF[i])); i++); /* advance to space */
        for(; pCLF[i] && (isspace(pCLF[i])); i++); /* advance past space */
        iRet = atoi( &(pCLF[i]) );

        /*
        ** Check out the status
        */
        if ( iRet>=200 && iRet<299 )
            {
            /* --- 2xx assume success under HTTP/1.0 --- */ 
            iRet = 0;
            };

        /*
	    ** Authentication handling
		*/
		if (( iRet == 401 )&&( *iAuthType == ID_NO_AUTH ))
			{
			sAuthenticate = (const char *)PIDB_lookup(
				pR, PIDBTYPE_RFC822, KEY_HTTP_WWW_AUTHENTICATE, 0 );
			*iAuthType = Internal_GetAuthType( pR );
			}

        if ( iRet )
            {
            /*
            ** Set the error message
            */
            sError = "Received error response from server:\n\n";
            sError.Concatenate( &(pCLF[i]) );
            sError.Concatenate( "\n\n" );

            /*
            ** Read the rest of the response
            */
            int iLen;
            const char *pErrorBuffer = PIIOBuffer_read( pBuffer, &iLen );
            while( pErrorBuffer )
                {
                PIString sTmp( pErrorBuffer, iLen );
                sError.Concatenate( sTmp );
                pErrorBuffer = PIIOBuffer_read( pBuffer, &iLen );
                };
            };
        };

    if ( !pCLF )
        {
        sError = "Error reading response from remote server '";
        sError.Concatenate( pHost );
        sError.Concatenate( "'." );
        iRet = 400;
        };

    if ( !iRet )
        {
        /* --- expect to get data? --- */
        if ( fnGetData!=0 )
            {
            iRet = (fnGetData)( pGetDataParam );
			if (iRet)
				{
				sError = "Response from remote server seems not to ";
				sError.Concatenate( "contain a valid Pi3 config file." );
				iRet = 400;
				};
            };
        };

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
    All about getting a configuration file from the server.
\*____________________________________________________________________________*/
/*
** Structure for passing information to get and put callback functions
*/
struct DataParam
{
    PIIOBuffer *pBuffer;
    QConfig *pConfig;
    int iLength;
    void *pData;
    PIString tMessage;
};
static int Internal_fnGetDataFunction( void *pParam )
{
    DataParam &tParam = *( (DataParam *)pParam );
    
    /* --- make PIIStream and load the configuration file --- */
    PIIStream is( tParam.pBuffer );
    return Pi3File_readConfig( tParam.pConfig, is );
}
static int Internal_loadConfigFromServer(
    ConnectionPool *pPool,
    QConfig *pConfig,
    const char *pHost,
    int iPort,
    const char *pUsername,
    const char *pPassword,
	PIString &sRealm,
	cbAuthData *pGetAuthData,
	void *pUserData,
    PIString &sError )
{
    const char *pCLF = 0;
	int iRetry = 2;
	int isInteractive = !(strlen(pUsername) && strlen(pPassword));
	PIString sAuthenticate;
	int iAuthType = ID_NO_AUTH;

again:

    /*
    ** Get a connnection to the remote host from the connection pool
    */
    PIIOBuffer *pBuffer = ConnectionPool_grapConnection( pPool,
        pHost, iPort );

    if ( !pBuffer )
        { 
        sError = "Could not open connection to server '";
        sError.Concatenate( pHost );
        sError.Concatenate( "' for configuration download." );		
        return -1;
        };

    DataParam tParam;
    tParam.pBuffer = pBuffer;
    tParam.pConfig = pConfig;
    int iRet = Internal_makeHTTPRequest(
        pBuffer,
        MD_GET,
        KEY_ADMIN_GETCONFIG,
        pHost,
        iPort,
        ADMIN_CONTENTTYPE,
        "0",
        0, 0,        /* no data to put */
        Internal_fnGetDataFunction, &tParam,
		pUsername,
		pPassword,
		sAuthenticate,
		&iAuthType,
        sError
        );

    /*
    ** Done with connection
    */
    ConnectionPool_releaseConnection( pPool, pBuffer, true );

	if (iRet == 401)
		{
		/*
		** Interactive login
		*/
		if ( isInteractive )
			{
			sRealm = Internal_GetRealm( sAuthenticate );
			if ( !pGetAuthData( pUserData )) { return iRet; };
			}
		else 
			{
			iRetry--;
			}

		if (iRetry) { goto again; };
		}

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
    All about putting a configuration file onto the server.
\*____________________________________________________________________________*/
static int Internal_fnPutDataFunction( void *pParam )
{
    DataParam &tParam = *( (DataParam *)pParam );
    
    /* --- make an PIOStream and write the configuration file --- */
    return !PIIOBuffer_write( tParam.pBuffer, (const char *)tParam.pData,
        tParam.iLength, PIIOBUF_NOBUFFER );
}
static int Internal_writeConfigToServer(
    ConnectionPool *pPool,
    QConfig *pConfig,
    const char *pHost,
    int iPort,
    const char *pUsername,
    const char *pPassword,
	PIString &sRealm,
	cbAuthData *pGetAuthData,
	void *pUserData,
    PIString &sError )
{
    const char *pCLF = 0;
	PIString sAuthenticate;
	int iAuthType = ID_NO_AUTH;
    int iRet = 0;
	int iRetry = 2;
	int isInteractive = !(strlen(pUsername) && strlen(pPassword));
	enum { BUF_SIZE=63 };
    char szFileSize[BUF_SIZE+1];
    PIFInfo *pFInfo = 0;
    int iLen;
    PIFMap *pFileMap = 0;
    const char *pMap = 0;
    DataParam tParam;
    ofstream ofs;
    int iTempFileWritten = 0;

again:

    /*
    ** Get a connnection to the remote host from the connection pool
    */
    PIIOBuffer *pBuffer = ConnectionPool_grapConnection( pPool,
        pHost, iPort );

    if ( !pBuffer )
        {
        sError = "Could not open connection to server '";
        sError.Concatenate( pHost );
        sError.Concatenate( "' for configuration upload." );
        return -1;
        };

    /*
    ** Open a temporary file and write the configuration into it
    */
    char szTmpFile[L_tmpnam];
    if ( !tmpnam( szTmpFile ) )
        {
        sError = "Could not create local temporary file for configuration \
upload.";
        iRet = -1;
        goto done;
        };
    ofs.open( szTmpFile );
    if ( !ofs.good() )
        {
        sError = "Could not open local temporary file '";
        sError.Concatenate( szTmpFile );
        sError.Concatenate( "' for configuration upload." );
        iRet = -1;
        goto done;
        };

    iTempFileWritten = 1;
    iRet = Pi3File_writeConfig( pConfig, ofs );
    ofs.close();
    if ( iRet )
        {
        sError = "Could write to local temporary file '";
        sError.Concatenate( szTmpFile );
        sError.Concatenate( "' for configuration upload." );
        goto done;
        };

    pFInfo = PIFInfo_new( szTmpFile );
    if ( !pFInfo )
        {
        sError = "Internal_writeConfigToServer: internal error.";
        iRet = -1;
        goto done;
        };

    sprintf( szFileSize, "%lu", PIFInfo_getSize( pFInfo ) );

    if ( PIFInfo_getSize( pFInfo )>0 )
        { pFileMap = PIFMap_new( pFInfo ); };
    if ( pFileMap )
        {
        pMap = (const char *)PIFMap_lock( pFileMap, &iLen );
        if ( pMap && iLen!=PIFInfo_getSize( pFInfo ) )
            {
            pMap = 0;
            };
        };

    /*
    ** If we got to this point without pMap, an error occurred someplace
    */
    if ( !pMap )
        {
        sError = "Could not memory map local temporary file '";
        sError.Concatenate( szTmpFile );
        sError.Concatenate( "' for configuration upload." );
        iRet = -1;
        goto done;
        };

    tParam.pBuffer = pBuffer;
    tParam.pConfig = pConfig;
    tParam.iLength = iLen;
    tParam.pData = (void *)pMap;
    iRet = Internal_makeHTTPRequest(
        pBuffer,
        MD_PUT,
        KEY_ADMIN_SAVECONFIG,
        pHost,
        iPort,
        ADMIN_CONTENTTYPE,
        szFileSize,
        Internal_fnPutDataFunction, &tParam,
        0, 0,        /* no data to get */
	    pUsername,
		pPassword,
		sAuthenticate,
		&iAuthType,
        sError
        );

    /*
    ** Done with connection
    */
done:
    if ( pFileMap ) { PIFMap_delete( pFileMap ); };
    if ( pFInfo ) { PIFInfo_delete( pFInfo ); };
    if ( iTempFileWritten )
        {
        ::remove( szTmpFile );
        };

    ConnectionPool_releaseConnection( pPool, pBuffer, true );

	if (iRet == 401)
		{
		/*
		** Interactive login
		*/
		if ( isInteractive )
			{
			sRealm = Internal_GetRealm( sAuthenticate );
			if ( !pGetAuthData( pUserData )) { return iRet; };
			}
		else 
			{
			iRetry--;
			}

		if (iRetry) { goto again; };
		}

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
Admin *Admin_newLocal( const char *pFileName )
{
    return PI_NEW( _Admin( pFileName ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
Admin *Admin_newRemote(
    PIObject *pParent,
    const char *pHost,
    int iPort,
    const char *pUsername,
    const char *pPassword,
	cbAuthData *pGetAuthData,
	void *pUserData
	)
{
    return PI_NEW( _Admin( pParent, pHost, iPort,
		pUsername, pPassword, pGetAuthData, pUserData ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Admin_delete( Admin *pAdmin )
{
    PI_DELETE( pAdmin );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Admin_loadConfig( Admin *pAdmin, QConfig *pConfig, PIString &sError )
{
    if ( !pAdmin )
        { return -1; };

    sError = PIString::Empty();

    if ( pAdmin->IsRemote() )
        {
        return Internal_loadConfigFromServer(
            pAdmin->pPool,
            pConfig,
            pAdmin->sHost,
            pAdmin->iPort,
			pAdmin->sUsername,
 			pAdmin->sPassword,
 			pAdmin->sRealm,
			pAdmin->cbGetAuthData,
			pAdmin->pUserData,
			sError );
        }
    else
        {
        return Internal_loadConfigFromFile( pConfig, pAdmin->sFileName );
        };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Admin_saveConfig( Admin *pAdmin, QConfig *pConfig, PIString &sError )
{
    if ( !pAdmin )
        { return -1; };

    if ( pAdmin->IsRemote() )
        {
        return Internal_writeConfigToServer(
            pAdmin->pPool,
            pConfig,
            pAdmin->sHost,
            pAdmin->iPort,
			pAdmin->sUsername,
 			pAdmin->sPassword,
 			pAdmin->sRealm,
			pAdmin->cbGetAuthData,
			pAdmin->pUserData,
            sError );
        }
    else
        {
        return Internal_writeConfigToFile( pConfig, pAdmin->sFileName );
        };
}

const char *Admin_GetRealm(Admin *pAdmin)
{
	return pAdmin->sRealm;
}

const char *Admin_GetUsername(Admin *pAdmin)
{
	return pAdmin->sUsername;
}

void Admin_SetUsername(Admin *pAdmin, const char *pUsername)
{
	pAdmin->sUsername = pUsername;
}

void Admin_SetPassword(Admin *pAdmin, const char *pPassword)
{
	pAdmin->sPassword = pPassword;
}
