/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/Http.cpp,v $
 * $Date: 2002/08/18 08:41:08 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"

#include "QConfig.h"
#include "Http.hrc"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define C_RESOURCENAME                    "HttpPage"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    HttpPage

Description:
    Creates the eponymous property page and is used by the PropSheet object.

Example:
	<PRE>
	&lt;Object&gt;
		Name HttpPage
		Class HttpPageClass
		PropertyPage HttpPage
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
static BOOL CALLBACK fnNewIndexFileDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static const char *__pIndexFiles[]=
{ 
"index.ssi", 
"index.html",
"index.htm",
"default.ssi",
"default.htm",
"default.html",

/* --- leave this last --- */
0 
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class HttpPage : public IFaceBase
{
private:
    QConfig *pConfig;
    PROPSHEETPAGE tPsp;
    Interface *pInterface;

    /*
    ** window private values
    */
	HWND hWndSrvStamp;
	HWND hWndList;
	HWND hWndGet;
	HWND hWndHead;
	HWND hWndPost;
	HWND hWndPut;
	HWND hWndDelete;
	HWND hWndOptions;
	HWND hWndTrace;
	HWND hWndDescription;
	HWND hWndAnnotation;
	HWND hWndUploadPath;
	HWND hWndLimit;
	HWND hWndDlgOK;
	HWND hWndDlgIndexFile;

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "HttpPage: ";

        if ( 0 )
            {
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	HttpPage( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pConfig( FindParentQConfig( PIObject_getDB( pObject ) ) ),
        pInterface( 0 ),
		hWndSrvStamp( 0 ),
		hWndList( 0 ),
		hWndGet( 0 ),
		hWndHead( 0 ),
		hWndPost( 0 ),
		hWndPut( 0 ),
		hWndDelete( 0 ),
		hWndOptions( 0 ),
		hWndTrace( 0 ),
		hWndDescription( 0 ),
		hWndAnnotation( 0 ),
		hWndUploadPath( 0 ),
		hWndLimit( 0 ),
		hWndDlgOK( 0 ),
		hWndDlgIndexFile( 0 )
		{
		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        /*
        ** Setup the PROPSHEETPAGE
        */
        memset( &tPsp, 0, sizeof( PROPSHEETPAGE ) );
        tPsp.dwSize = sizeof( PROPSHEETPAGE );
        tPsp.dwFlags = PSP_DEFAULT;
        tPsp.hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );

        /* Status page */
        tPsp.pszTemplate = C_RESOURCENAME;
        tPsp.pfnDlgProc = (DLGPROC)fnDialogProc;
        tPsp.lParam = (LPARAM)this;

        /*
        ** Set the data DB of this object
        */
        PIDB *pDB = PIObject_getDB( Object() );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_WINDOWTYPE,
            (void *)INTRFACE_PROPPAGE, 0 );
        assert(!PIDB_lookup( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE, 0));
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE,
            (void *)&tPsp, 0 );
        
        /*
        ** Done
        */
		};

	~HttpPage()
		{
		};

	void Internal_loadItems( HWND hDlg )
		{
		/*
		** Clear any existing items out of list
		*/
		::SendMessage( hWndList, LB_RESETCONTENT, 0, 0 );

		/*
		** Load all index files into list box
		*/
		int iTmp = 0;
		for(;; iTmp++ )
			{
			const char *pVal = M_LookupValueEx( pConfig, "HTTP", "IndexFile", iTmp );
			if ( !pVal ) { break; };
			::SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pVal );
			}

		::EnableWindow( ::GetDlgItem( hDlg, IDC_REMOVE ),
		::SendMessage( hWndList, LB_GETCOUNT, 0, 0 ));
		}

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                pInterface = &tWindow;
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return PIAPI_ABORT;

            case INTRFACE_MSG_LOAD:
                return PIAPI_ABORT;

            case INTRFACE_MSG_DEFAULTS:
				M_AddValue( pConfig, "HTTP", "ServerStamp", "Pi3Web/2.0.1" );
				// GET  HEAD POST PUT  DELE OPTI TRACE
				// 1    2    4    8    16   32   64
				M_AddValue( pConfig, "HTTP", "Methods", "3" );
				M_AddValue( pConfig, "HTTP", "Description", "On" );
				M_AddValue( pConfig, "HTTP", "Annotation", "On" );
				M_AddValue( pConfig, "HTTP", "UploadPath", "" );
				M_AddValue( pConfig, "HTTP", "Limit", "4194304" );

				/*
				** Index Files
				*/
				if ( !M_LookupValue( pConfig, "HTTP", "IndexFile" ) )
					{
					for( int i=0; __pIndexFiles[i]; i++ )
						{
						M_AddValue( pConfig, "HTTP", "IndexFile", __pIndexFiles[i] );
						};
					}
                break;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

	BOOL CALLBACK NewIdxFileDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
		{
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];

		switch (uMsg)
			{
			case WM_INITDIALOG:
				hWndDlgOK = GetDlgItem( hDlg, IDOK );
				hWndDlgIndexFile = GetDlgItem( hDlg, IDC_INDEX1 );
				assert( hWndDlgOK && hWndDlgIndexFile );
				SendMessage( hWndDlgIndexFile, EM_SETLIMITTEXT, 256, 0 );
				EnableWindow( hWndDlgOK, FALSE );
				break;

			case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDOK:
						{
						/*
						** Validate content of edit field
						*/
						*szBuf = '\0';
						SendMessage( hWndDlgIndexFile, WM_GETTEXT, BUF_SIZE, 
							(LPARAM)szBuf );
						PIString sIdxFile( szBuf );
						int i;
						int iCount = SendMessage( hWndList, CB_GETCOUNT, 0, 0 );
						for( i=0; i<iCount; i++ )
							{
							*szBuf = '\0';
							SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
							if ( sIdxFile == szBuf )
								{
								sprintf( szBuf, "IndexFile '%s' already exists.",
									(const char *)sIdxFile );
								MessageBox( hDlg, szBuf, "New Index File",
									MB_OK | MB_ICONEXCLAMATION );
								return (TRUE);
								};
							};

						::SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );
						};
						EndDialog( hDlg, 1 );
						break;

					case IDCANCEL:
						EndDialog( hDlg, 0 );
						break;

					case IDC_INDEX1:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
							{
							if ( SendMessage( GET_WM_COMMAND_HWND(wParam, lParam ), EM_LINELENGTH,
								0, 0 )>0 )
								{ EnableWindow( hWndDlgOK, TRUE ); }
							else
								{ EnableWindow( hWndDlgOK, FALSE ); }
							}
						else
							return (FALSE);
						break;

					default:;
					};

			default:
				return (FALSE);
			};

		return (TRUE);
		}

	BOOL DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
	    {
	    enum { BUF_SIZE=1023 };
	    char szBuf[BUF_SIZE+1];

		switch (uMsg)
			{
			case WM_INITDIALOG:
				{
				const char *pA = M_LookupValue( pConfig, "General", "Admin" );
				hWndSrvStamp = GetDlgItem( hDlg, IDC_SERVERSTAMP );
				hWndList = GetDlgItem( hDlg, IDC_LIST );
				hWndGet = GetDlgItem( hDlg, IDC_GET );
				hWndHead = GetDlgItem( hDlg, IDC_HEAD );
				hWndPost = GetDlgItem( hDlg, IDC_POST );
				hWndPut = GetDlgItem( hDlg, IDC_PUT );
				hWndDelete = GetDlgItem( hDlg, IDC_DELETE );
				hWndOptions = GetDlgItem( hDlg, IDC_OPTIONS );
				hWndTrace = GetDlgItem( hDlg, IDC_TRACE );
				hWndDescription = GetDlgItem( hDlg, IDC_DESCR );
				hWndAnnotation = GetDlgItem( hDlg, IDC_ANNOTATION );
				hWndUploadPath = GetDlgItem( hDlg, IDC_DOCUMENTROOT );
				hWndLimit = GetDlgItem( hDlg, IDC_LIMIT );
	
				assert( hWndSrvStamp && hWndList );
				assert( hWndGet && hWndHead && hWndPost && hWndPut );
				assert( hWndDelete && hWndOptions && hWndTrace );
				assert( hWndDescription && hWndAnnotation );
				assert( hWndUploadPath && hWndLimit );
				assert( pA );
	
				/* --- just for sanity --- */
				::SendMessage( hWndSrvStamp, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndUploadPath, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndLimit, EM_SETLIMITTEXT, 256, 0 );
	
				/* --- General --- */
				const char *pVal = M_LookupValue( pConfig, "HTTP", "ServerStamp" );
				::SendMessage( hWndSrvStamp, WM_SETTEXT, 0, (LPARAM)pVal );
				Internal_loadItems( hDlg );

				/* --- Methods --- */
				int iTmp = atoi(M_LookupValue( pConfig, "HTTP", "Methods" ));
				::SendMessage( hWndGet, BM_SETCHECK, (WPARAM)(iTmp & 1), 0 );
				::SendMessage( hWndHead, BM_SETCHECK, (WPARAM)(iTmp & 2), 0 );
				::SendMessage( hWndPost, BM_SETCHECK, (WPARAM)(iTmp & 4), 0 );
				::SendMessage( hWndPut, BM_SETCHECK, (WPARAM)(iTmp & 8), 0 );
				::SendMessage( hWndDelete, BM_SETCHECK, (WPARAM)(iTmp & 16), 0 );
				::SendMessage( hWndOptions, BM_SETCHECK, (WPARAM)(iTmp & 32), 0 );
				::SendMessage( hWndTrace, BM_SETCHECK, (WPARAM)(iTmp & 64), 0 );

				/* --- Upload Options --- */
				pVal = M_LookupValue( pConfig, "HTTP", "Description" );
				::SendMessage( hWndDescription, BM_SETCHECK, (WPARAM)(pVal[0]), 0 );
				pVal = M_LookupValue( pConfig, "HTTP", "Annotation" );
				::SendMessage( hWndAnnotation, BM_SETCHECK, (WPARAM)(pVal[0]), 0 );
				pVal = M_LookupValue( pConfig, "HTTP", "UploadPath" );
				::SendMessage( hWndUploadPath, WM_SETTEXT, 0, (LPARAM)pVal );
				pVal = M_LookupValue( pConfig, "HTTP", "Limit" );
				::SendMessage( hWndLimit, WM_SETTEXT, 0, (LPARAM)pVal );
			
				iTmp = ::SendMessage( hWndPost, BM_GETCHECK, 0, 0 );
				::EnableWindow( hWndDescription, iTmp );
				::EnableWindow( hWndAnnotation, iTmp );
				::EnableWindow( hWndUploadPath, iTmp );
				::EnableWindow( hWndLimit, iTmp );
	
				Unchanged( hDlg );
				};
				break;
	
			case WM_COMMAND:
				if ( GET_WM_COMMAND_CMD(wParam, lParam)==BN_CLICKED )
					{
					switch( GET_WM_COMMAND_ID( wParam, lParam) )
						{
						case IDC_POST:
							::EnableWindow( hWndDescription,
								::SendMessage( hWndPost, BM_GETCHECK, 0, 0 ));
							::EnableWindow( hWndAnnotation,
								::SendMessage( hWndPost, BM_GETCHECK, 0, 0 ));
							::EnableWindow( hWndUploadPath,
								::SendMessage( hWndPost, BM_GETCHECK, 0, 0 ));
							::EnableWindow( hWndLimit,
								::SendMessage( hWndPost, BM_GETCHECK, 0, 0 ));
							Changed( hDlg );
							break;

						case IDC_ADD:
							if ( DialogBoxParam(
		                        tPsp.hInstance,
								MAKEINTRESOURCE( IDD_NEWINDEXFILE ),
								hDlg,
								(DLGPROC)fnNewIndexFileDialogProc,
						        (LPARAM)this
								) )
								{
								::EnableWindow( ::GetDlgItem( hDlg, IDC_REMOVE ),
									::SendMessage( hWndList, LB_GETCOUNT, 0, 0 ));
								Changed( hDlg );
								};
							break;

						case IDC_REMOVE:
							{
							int iSel = ::SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
							if ( iSel==CB_ERR ) { break; };

							::SendMessage( hWndList, LB_DELETESTRING, iSel, 0 );
							int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0);
							if (iSel >= iCount) {
								SendMessage( hWndList, LB_SETCURSEL, iCount-1, 0 );
							} else {
								SendMessage( hWndList, LB_SETCURSEL, iSel, 0 );
							}

							::EnableWindow( ::GetDlgItem( hDlg, IDC_REMOVE ),
								::SendMessage( hWndList, LB_GETCOUNT, 0, 0 ));
							Changed( hDlg );
							}
						break;

						default:;
						};

					/* --- mark property sheet as modified --- */
					Changed( hDlg );
					}
			if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
				{
				/* --- mark property sheet as modified --- */
				Changed( hDlg );
				}
			if ( GET_WM_COMMAND_CMD(wParam, lParam)==BN_CLICKED )
				{
				/* --- mark property sheet as modified --- */
				Changed( hDlg );
				}
			break;
	
			case WM_DESTROY:
				break;
	
			case WM_NOTIFY:
				switch( ((NMHDR *)lParam)->code )
					{
					case PSN_QUERYCANCEL:
						return SHEET->Cancel(pInterface, hDlg);
					
					case PSN_KILLACTIVE:
						{
						/* --- General --- */
						*szBuf = '\0';
						::SendMessage( hWndSrvStamp, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "HTTP", "ServerStamp", szBuf ); 
	
						M_DeleteValue( pConfig, "HTTP", "IndexFile" ); 
						int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0);
						int iTmp = 0;
						for (;iTmp<iCount;iTmp++)
							{
							*szBuf = '\0';
							SendMessage( hWndList, LB_GETTEXT, iTmp, (LPARAM)szBuf );
							M_AddValue( pConfig, "HTTP", "IndexFile", szBuf ); 
							}

						/* --- Methods --- */
						iTmp = 0;
						if ( ::SendMessage( hWndGet, BM_GETCHECK, 0, 0 ))
							{ iTmp += 1; };
						if ( ::SendMessage( hWndHead, BM_GETCHECK, 0, 0 ))
							{ iTmp += 2; };
						if ( ::SendMessage( hWndPost, BM_GETCHECK, 0, 0 ))
							{ iTmp += 4; };
						if ( ::SendMessage( hWndPut, BM_GETCHECK, 0, 0 ))
							{ iTmp += 8; };
						if ( ::SendMessage( hWndDelete, BM_GETCHECK, 0, 0 ))
							{ iTmp += 16; };
						if ( ::SendMessage( hWndOptions, BM_GETCHECK, 0, 0 ))
							{ iTmp += 32; };
						if ( ::SendMessage( hWndTrace, BM_GETCHECK, 0, 0 ))
							{ iTmp += 64; };
						M_ReplaceValue( pConfig, "HTTP", "Methods",
							itoa(iTmp, szBuf, 10));

						/* --- Upload Options --- */
						if ( ::SendMessage( hWndDescription, BM_GETCHECK, 0, 0 )==BST_CHECKED )
							{ M_ReplaceValue( pConfig, "HTTP", "Description", "On" ); }
						else
							{ M_ReplaceValue( pConfig, "HTTP", "Description", "" ); };

						if ( ::SendMessage( hWndAnnotation, BM_GETCHECK, 0, 0 )==BST_CHECKED )
							{ M_ReplaceValue( pConfig, "HTTP", "Annotation", "On" ); }
						else
							{ M_ReplaceValue( pConfig, "HTTP", "Annotation", "" ); };

						*szBuf = '\0';
						::SendMessage( hWndUploadPath, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "HTTP", "UploadPath", szBuf ); 

						*szBuf = '\0';
						::SendMessage( hWndLimit, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "HTTP", "Limit", szBuf ); 
	
						SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
						return (TRUE);
						}
	
					case PSN_APPLY:
						return SHEET->SaveConfig(pInterface, hDlg);

					default:
						return (FALSE);
					};
				break;
	
			default:
					return (FALSE);
			};
	
		return (TRUE);
		}
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, ((LPPROPSHEETPAGE)lParam)->lParam );
        };

    HttpPage *pPage = (HttpPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->DialogProc( hDlg, uMsg, wParam, lParam ) : FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnNewIndexFileDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, lParam );
        };

    HttpPage *pPage = (HttpPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->NewIdxFileDialogProc( hDlg, uMsg, wParam, lParam ) :
        FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HttpPage_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( HttpPage( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name HttpPageClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor HttpPage_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name HttpPage
		Class HttpPageClass
	</Object>

/*___+++CNF_END+++___*/
#endif

