/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/Pi3File.cpp,v $
 * $Date: 2002/07/31 19:56:59 $
 *
 Description:

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <assert.h>
#include <fstream.h>
#include <ctype.h>

#include "PIString.h"
#include "Pi3File.h"
#include "MiscUtil.h"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define SECTION_START "+++---AD Section Start, Do not Modify:"
#define SECTION_END "+++---AD Section End---+++"
#define NOTMODIFIED_START "+++---AD Saved Section, modifications will be retained---+++"
#define NOTMODIFIED_END "+++---AD Saved section end---+++"
#define GENERATED_START "+++---AD Generated Section, modifications will be lost---+++"
#define GENERATED_END "+++---AD Generated section end---+++"

/*____________________________________________________________________________*\
 *
 Description:
	Directory index layout names
\*____________________________________________________________________________*/
static const char *__pLayoutNames[] =
{
"HTML table",
"HTML index with fixed font",
"Generic HTML index",
"DOS style HTML index",
"Plain text with Tabs",
"Customized index is used",

/* --- leave this last --- */
0
};

/*____________________________________________________________________________*\
 *
 Description:
	HTTP Default index files
\*____________________________________________________________________________*/
static const char *__pIndexFiles[]=
{ 
"index.ssi", 
"index.html",
"index.htm",
"default.ssi",
"default.htm",
"default.html",

/* --- leave this last --- */
0 
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_setupDefaults( QConfig *pConfig )
{
#define AD_DEFAULT(x,y,z) \
	{ \
	if ( !M_LookupValue( pConfig, x, y ) ) \
		{ M_ReplaceValue( pConfig, x, y, z ); }; \
	}

	/*
	** Internal values for allowing extensions.
	*/
	AD_DEFAULT( "Internal", "URLProtocol", "http://" );
	AD_DEFAULT( "Internal", "DefaultPort", "80" );
	AD_DEFAULT( "Internal", "ServerIOObjectLine", "\tIOObject ServerIOObject" );
	AD_DEFAULT( "Internal", "DefaultMIMEType", "application/octet-stream" );
	AD_DEFAULT( "Internal", "HandlerLine", DEFAULT_HANDLERS );

	/*
	** Identity defaults
	*/
	AD_DEFAULT( "Identity", "Use", "local" );
	AD_DEFAULT( "General", "Port", "80" );
	AD_DEFAULT( "General", "Host", "localhost" );

	/*
	** General and performance defaults
	*/
	AD_DEFAULT( "General", "Processes", "8" );
	AD_DEFAULT( "General", "Threads", "20" );
	AD_DEFAULT( "General", "ExitAfter", "1000" );
	AD_DEFAULT( "General", "Admin", "webmaster@yourhost.dom" );
	AD_DEFAULT( "General", "ExePath", "C:\\Pi3Web\\bin\\EnhPi3.exe" );
	AD_DEFAULT( "General", "BinDir", "C:\\Pi3Web\\bin" );
	AD_DEFAULT( "General", "InstallDir", "C:\\Pi3Web" );
	AD_DEFAULT( "General", "SendTimeout", "350" );
	AD_DEFAULT( "General", "RecvTimeout", "400" );
	AD_DEFAULT( "General", "DNSReverseLookup", "" );
	AD_DEFAULT( "General", "Debug", "" );
	AD_DEFAULT( "General", "ErrorLog", "On" );
	AD_DEFAULT( "General", "RunMode", "Desktop" );
	AD_DEFAULT( "General", "ServiceName", "Pi3Web" );
	AD_DEFAULT( "General", "KeepOpen", "On" );
	AD_DEFAULT( "General", "KeepOpenCount", "100" );
	AD_DEFAULT( "General", "KeepOpenTimeout", "5" );
	AD_DEFAULT( "General", "ErrorMessages", "Short" );

	/*
	** Directory indexing defaults
	*/
	AD_DEFAULT( "DirIndex", "Layout", "1" );
	// Type Name Alt  Size Date Dscr
	// 1    2    4    8    16   32  
	AD_DEFAULT( "DirIndex", "Columns", "59" );
	AD_DEFAULT( "DirIndex", "SortColumn", "Type" );
	// Link Case Desc
	// 1    2    4
	AD_DEFAULT( "DirIndex", "SortFlags", "1" );
	// Hdr  Ftr  Dscr
	// 1    2    4
	AD_DEFAULT( "DirIndex", "FileFlags", "7" );
	AD_DEFAULT( "DirIndex", "TagNew", "10" );
	AD_DEFAULT( "DirIndex", "Truncate", "20" );
	AD_DEFAULT( "DirIndex", "SizeFormat", "AbbrevSize" );

	/*
	** HTTP defaults
	*/
	AD_DEFAULT( "HTTP", "ServerStamp", "Pi3Web/2.0.1" );
	// GET  HEAD POST PUT  DELE OPTI TRACE
	// 1    2    4    8    16   32   64
	AD_DEFAULT( "HTTP", "Methods", "3" );
	AD_DEFAULT( "HTTP", "Description", "On" );
	AD_DEFAULT( "HTTP", "Annotation", "On" );
	AD_DEFAULT( "HTTP", "UploadPath", "" );
	AD_DEFAULT( "HTTP", "Limit", "4194304" );

	/*
	** Index Files
	*/
	if ( !M_LookupValue( pConfig, "HTTP", "IndexFile" ) )
		{
		for( int i=0; __pIndexFiles[i]; i++ )
			{
			M_AddValue( pConfig, "HTTP", "IndexFile", __pIndexFiles[i] );
			};
		}

	/*
	** SSL defaults
	*/
	AD_DEFAULT( "SSL", "Debug", "" );
	// SSL2 SSL3 TLS1 Debg Bugs Hack
	// 1    2    4    8    16   32
	AD_DEFAULT( "SSL", "Options", "51" );
	AD_DEFAULT( "SSL", "PKeyFile", "./key.pem" );
	AD_DEFAULT( "SSL", "CertFile", "./cert.pem" );
	AD_DEFAULT( "SSL", "Validation", "0" );
	AD_DEFAULT( "SSL", "Depth", "1" );
	AD_DEFAULT( "SSL", "CaCertPath", "./" );
	AD_DEFAULT( "SSL", "CaCertFile", "cacert.pem" );
	AD_DEFAULT( "SSL", "Ciphers", "DEFAULT" );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_writeDefaultParagraphs( ostream &os )
{
#define SECTION_P16 "\n\
# ---\n\
# \n\
# Scripts, WinScripts, FastCGIScripts\n\
#\n\
#	Handler sequences for resources which have mapped to these\n\
# 	respective types\n\
#\n\
# ---\n\
<Object>\n\
	Name Scripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),Scripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle StandardCGI\n\
</Object>\n\
\n\
<Object>\n\
	Name WinScripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),WinScripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle WindowsCGI\n\
</Object>\n\
"
	os << SECTION_P16;

#define SECTION_P17 "\n\
<Object>\n\
	Name FastCGIScripts\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),FastCGIScripts)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess ReturnCode ReturnCode=\"COMPLETED\"\n\
	CheckType ReturnCode ReturnCode=\"COMPLETED\"\n\
	Handle FastCGI \\\n\
		IOObject=\"FastCGIIOObject\" \\\n\
		Variable=\"AUTH_TYPE=$x\" \\\n\
		Variable=\"CONTENT_LENGTH=$Y\" \\\n\
		Variable=\"CONTENT_TYPE=$C\" \\\n\
		Variable=\"HTTPS=$g\" \\\n\
		Variable=\"&if($K,HTTPS_KEYSIZE=$K,)\" \\\n\
		Variable=\"&if($K,HTTPS_SECRETKEYSIZE=$G,)\" \\\n\
		Variable=\"PATH_INFO=$I\" \\\n\
		Variable=\"PATH_TRANSLATED=$Z\" \\\n\
		Variable=\"QUERY_STRING=$q\" \\\n\
		Variable=\"REMOTE_ADDR=$A\" \\\n\
		Variable=\"REMOTE_HOST=$h\" \\\n\
		Variable=\"REMOTE_IDENT=$i\" \\\n\
		Variable=\"REMOTE_USER=$u\" \\\n\
		Variable=\"REQUEST_METHOD=$m\" \\\n\
		Variable=\"SCRIPT_NAME=$z\" \\\n\
		Variable=\"SERVER_NAME=$v\" \\\n\
		Variable=\"SERVER_PORT=$p\" \\\n\
		Variable=\"SERVER_PROTOCOL=$H\" \\\n\
		Variable=\"SERVER_SOFTWARE=$S\"\n\
</Object>\n\
"
	os << SECTION_P17;

#define SECTION_P18 "\n\
<Object>\n\
	Name ISAPI\n\
	Class FlexibleHandlerClass\n\
	Condition \"&cmp(&dblookup(response,string,ObjectMap),ISAPI)\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EFD\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAccess AccessByFile RequirePermissions=\"X\"\n\
	CheckType ReturnCode ReturnCode=COMPLETED\n\
	Handle ISAPIExtensions\n\
</Object>\n\
\n\
<Object>\n\
	Name ISAPIExtensions\n\
	Class ISAPI20Class\n\
	# These are the server variables that are sent to the\n\
	# extension\n\
	Variable \"ALL_HTTP=%a\"\n\
	Variable \"CONTENT_LENGTH=%l\"\n\
	Variable \"AUTH_TYPE=$x\"\n\
	Variable \"CONTENT_TYPE=$C\"\n\
	Variable \"HTTPS=$g\"\n\
	Variable \"GATEWAY_INTERFACE=CGI/1.1\"\n\
	Variable \"PATH_INFO=$I\"\n\
	Variable \"PATH_TRANSLATED=$Z\"\n\
	Variable \"QUERY_STRING=$q\"\n\
	Variable \"REMOTE_ADDR=$A\"\n\
	Variable \"REMOTE_HOST=$h\"\n\
	Variable \"REMOTE_IDENT=$i\"\n\
	Variable \"REMOTE_USER=$u\"\n\
	Variable \"REQUEST_METHOD=$m\"\n\
	Variable \"SCRIPT_NAME=$z\"\n\
	Variable \"SERVER_NAME=$v\"\n\
	Variable \"SERVER_PORT=$p\"\n\
	Variable \"SERVER_PROTOCOL=$H\"\n\
	Variable \"SERVER_SOFTWARE=$S\"\n\
	ExtraHeaders Yes\n\
	ExtraHeadersPrefix \"HTTP_\"\n\
	ExtraHeadersIgnore \"Content-Type Content-Length\"\n\
</Object>\n\
\n"

	os << SECTION_P18;

#define SECTION_P40 "\n\
# ---\n\
#\n\
# Customizable Directory Listing\n\
#\n\
# ---\n\
<Object>\n\
	Name CustomIndex\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY BACKGROUND=\"/icons/\\\n\
&if(&cmpi($o,MainVirtualHostInformation),Pi3Tile.gif,H2Tile.gif)\"/*background depends on vhost*/\\\n\
BGCOLOR=\"#FFFFFF\">\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<CENTER><TABLE CELLSPACING=0 CELLPADDING=0 WIDTH=100'%'>\\\n\
<TR>\\\n\
<TH NOWARP ALIGN=LEFT><BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT><A HREF=\"%p?SortType=A\">Type</A><BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT><A HREF=\"%p?SortName=A\">Name</A><BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT><A HREF=\"%p?SortSize=A\">Size</A><BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT><A HREF=\"%p?SortDate=A\">Last Modified</A><BR><HR SIZE=1 NOSHADE>\\\n\
<TH NOWRAP ALIGN=LEFT><A HREF=\"%p?SortDscr=A\">Description</A><BR><HR SIZE=1 NOSHADE>\\n\"\n\
	FilePattern \"<TR>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>%m</TD><TD NOWRAP ALIGN=LEFT VALIGN=TOP>\\\n\
<A HREF=\"%r\"><IMG SRC=\"%i\" BORDER=NONE ALIGN=MIDDLE ALT=\"\"></A>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP><A HREF=\"%r\">&abbrev(%n,20,...)</A>\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>%s<TD NOWRAP ALIGN=LEFT VALIGN=TOP>%l\\\n\
<TD NOWRAP ALIGN=LEFT VALIGN=TOP>&if(%d,%d,<I>%c</I>)\\n\"\n\
	ListBottom \"</TABLE></CENTER>\\n\"\n\
"
#define SECTION_P41 "\
	MIMEIcon \"text/html /icons/layout.gif\"\n\
	MIMEIcon \"text/x-code /icons/c.gif\"\n\
	MIMEIcon \"text/x-perl /icons/p.gif\"\n\
	MIMEIcon \"text/\* /icons/text.gif\"\n\
	MIMEIcon \"internal/x-directory /icons/folder.gif\"\n\
	MIMEIcon \"audio/\* /icons/sound2.gif\"\n\
	MIMEIcon \"image/\* /icons/image2.gif\"\n\
	MIMEIcon \"application/x-uuencoded /icons/uuencoded.gif\"\n\
	MIMEIcon \"application/x-tar /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-gzip /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-sh /icons/script.gif\"\n\
	MIMEIcon \"application/\* /icons/binary.gif\"\n\
	MIMEIcon \"x-world/x-vrml /icons/burst.gif\"\n\
	MIMEIcon \"\*/\* /icons/unknown.gif\"\n\
	SwapFileName \".. Parent Directory\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	HeaderFile \"HEADER\"\n\
	FooterFile \"README\"\n\
	DescriptionFile \".desc\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
	NewPattern \"<IMG SRC=\"/icons/new.gif\">\"\n\
	NewTreshold 10\n\
	Options \" Name | AbbrevSize | RelPath | Icon | MIMEType | SortType | TagNew\"\n\
# SortDesc | SortCase | SortType | SortName | SortSize | SortDate | SortDscr\n\
</Object>\
\n"
	os << SECTION_P40;
	os << SECTION_P41;

#define SECTION_P45 "\n\
# ---\n\
#\n\
# CGIFileIOObject\n\
# 	\n\
#	IOObject used for communication between web browser and\n\
#	server.\n\
#\n\
# ---\n\
<Object>\n\
	Name CGIFileIOObject\n\
	Class PipeIOClass\n\
	ReadTimeout -1\n\
	WriteTimeout 10\n\
	NoYield \"Read\"\n\
</Object>\n\
\n"
	os << SECTION_P45;

#define SECTION_P20 "\n\
# ---\n\
#\n\
# StandardCGI\n\
#\n\
# Configuration for Standard CGI\n\
#\n\
# ---\n\
<Object>\n\
	Name StandardCGI\n\
	Class CGIClass\n\
	\n\
	# Specify the IO object use to communicate with the CGI program\n\
	FileIOObject CGIFileIOObject\n\
\n\
	# Snoop IO objects can be instantiated in the IO chain with a line\n\
	# like this. They log all IO trafic. This is useful for debugging.\n\
#	FileIOObject SnoopIO IOObject=\"CGIFileIOObject\" OutFile=\"cgi.dbg\" \n\
\n\
	DefaultCommandLine \"%p%q\"\n\
	CommandLineByExt .bat=\"cmd.exe /c %p%q\"\n\
	CommandLineByExt .cmd=\"cmd.exe /c %p%q\"\n\
	CommandLineByExt .pl=\"perl %p%q\"\n\
	CommandLineByExt .cgi=\"perl %p%q\"\n\
	IncludeParentsEnvironment Yes\n\
	EnvironmentSize 8192\n\
	\n\
	# These are the environment variables that are sent to the\n\
	# CGI program\n\
	Variable \"AUTH_TYPE=$x\"\n\
	Variable \"CONTENT_LENGTH=$Y\"\n\
	Variable \"CONTENT_TYPE=$C\"\n\
	Variable \"HTTPS=$g\"\n\
	Variable \"&if($K,HTTPS_KEYSIZE=$K,)\"\n\
	Variable \"&if($K,HTTPS_SECRETKEYSIZE=$G,)\"\n\
	Variable \"GATEWAY_INTERFACE=CGI/1.1\"\n\
	Variable \"PATH_INFO=$I\"\n\
	Variable \"PATH_TRANSLATED=$Z\"\n\
	Variable \"QUERY_STRING=$q\"\n\
	Variable \"REMOTE_ADDR=$A\"\n\
	Variable \"REMOTE_HOST=$h\"\n\
	Variable \"REMOTE_IDENT=$i\"\n\
	Variable \"REMOTE_USER=$u\"\n\
	Variable \"REQUEST_METHOD=$m\"\n\
	Variable \"SCRIPT_NAME=$z\"\n\
	Variable \"SERVER_NAME=$v\"\n\
	Variable \"SERVER_PORT=$p\"\n\
	Variable \"SERVER_PROTOCOL=$H\"\n\
	Variable \"SERVER_SOFTWARE=$S\"\n\
	ExtraHeaders Yes\n\
	ExtraHeadersPrefix \"HTTP_\"\n\
	ExtraHeadersIgnore \"Content-Type Content-Length\"\n\
#	SendCRLF Yes\n\
    #\n\
    # Kill CGI programs after 1 minute if they don't respond\n\
    #\n\
    KillAfter 60\n\
</Object>\n\
\n"
	os << SECTION_P20;

#define SECTION_P21 "\n\
# ---\n\
#\n\
# WindowsCGI\n\
#\n\
# Handle windows CGI\n\
#\n\
# ---\n\
<Object>\n\
	Name WindowsCGI\n\
	Class WinCGIClass\n\
	FileIOObject CGIFileIOObject\n\
	DefaultCommandLine \"%p \"%d\"\"\n\
	ExtraHeaders Yes\n\
	ExtraHeadersPrefix \"HTTP_\"\n\
	ExtraHeadersIgnore \"Content-Type Content-Length\"\n\
\n\
	# Generate windows CGI filenames by preappending the\n\
	# process id to the unique identifier.\n\
	ExternalPath \"cgi-temp/\"\n\
	DataFile \"cgi-temp/$P_%u.ini\"\n\
	StdinFile \"cgi-temp/$P_%u.in\"\n\
	StdoutFile \"cgi-temp/$P_%u.out\"\n\
		DataBlock \"\[CGI]$M\\\n\
Request Protocol=$H$M\\\n\
Request Method=$m$M\\\n\
Executable Path=$f$M\\\n\
Logical Path=$I$M\\\n\
Physical Path=$Z$M\\\n\
Query String=$q$M\\\n\
Referer=&dblookup(request,rfc822,Referer)$M\\\n\
From=$i$M\\\n\
User Agent=&dblookup(request,rfc822,User-Agent)$M\\\n\
Content Type=$C$M\\\n\
Content Length=$Y$M\\\n\
Content File=%i$M\\\n\
Server Name=$v$M\\\n\
Server Admin=$a$M\\\n\
Server Port=$p$M\\\n\
Server Software=$S$M\\\n\
CGI Version=CGI/1.1 (Win)$M\\\n\
Remote Address=$A$M\\\n\
Remote Host=$h$M\\\n\
Authentication Method=$x$M\\\n\
Authenticated Username=$u$M\\\n\
Authenticated Password=&dblookup(response,string,AuthPass)$M\\\n\
Authentication Realm=&dblookup(response,string,AuthType)$M\\\n\
$M\\\n\
[Accept]$M\\\n\
%a$M\\\n\
[System]$M\\\n\
Output File=%o$M\\\n\
Content File=%i$M\\\n\
$M\\\n\
[Extra Headers]$M\\\n\
%h$M\\\n\
[Form Literal]$M\\\n\
%l$M\\\n\
[Form External]$M\\\n\
%e$M\\\n\
[Form File]$M\\\n\
%f$M\\\n\
[Form Huge]$M\\\n\
%g$M\"\n\
</Object>\n\
\n"
	os << SECTION_P21;
	
#define SECTION_P50 "\n\
# ---\n\
#\n\
# CGIMapper\n\
#\n\
#	Instance of PathMapperClass to Map CGI URLs\n\
#\n\
#	CGI Scripts set PATH_INFO. \n\
#\n\
# ---\n\
<Object>\n\
	Name CGIMapper\n\
	Class PathMapperClass\n\
	CaseSensitive \"No\"\n\
	#\n\
	# Subdirectories below the mapping directory should be put in path info\n\
	#\n\
	PathInfo \"Yes\"\n\
	#\n\
	# Set the 'ObjectMap' variable in the response DB if this map is\n\
	# successful\n\
	#\n\
	Action \"&dbreplace(response,string,ObjectMap,Scripts)\"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# WinCGIMapper, FCGIMapper and ISAPIMapper\n\
#\n\
# 	Like CGIMapper, except it leaves a different marker to cause\n\
# 	a different object to handle the request.\n\
#\n\
# ---\n\
<Object>\n\
	Name WinCGIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,WinScripts)\"\n\
</Object>\n\
<Object>\n\
	Name FCGIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,FastCGIScripts)\"\n\
</Object>\n\
<Object>\n\
	Name ISAPIMapper\n\
	Class PathMapperClass\n\
	PathInfo \"Yes\"\n\
	Action \"&dbreplace(response,string,ObjectMap,ISAPI)\"\n\
</Object>\n\
\n"
	os << SECTION_P50;

#define SECTION_P51 "\n\
# ---\n\
#\n\
# FastCGIIOObject\n\
# \n\
# IOObject for communicating with fast CGI application servers. This\n\
# one uses the TCP/IP transport.\n\
#\n\
# ---\n\
<Object>\n\
	Name FastCGIIOObject\n\
	Class TCPIPIOClass\n\
	Type Active\n\
\n\
	# Timeouts. -1 specifies no timeout.\n\
	RecvTimeout 20\n\
	SendTimeout 20\n\
</Object>\n\
\n"
	os << SECTION_P51;

#define SECTION_P52 "\n\
# ---\n\
#\n\
# AccessLogger\n\
#\n\
#	An instance of ExpressionLoggerClass configured to generate an \n\
#	logfile\n\
#\n\
# ---\n\
<Object>\n\
	Name AccessLogger\n\
	Class ExpressionLoggerClass\n\
\n\
	#\n\
	# Append\n\
	#\n\
	OpenMode \"a\"\n\
	#\n\
	# Line to log \n\
	Expression \"$A $h - [$t] \\\"$r\\\" $s $b$M\"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# RefererLogger\n\
#\n\
#	Log bad links only (HTTP error code 404)\n\
#\n\
# ---\n\
<Object>\n\
	Name RefererLogger\n\
	Class ExpressionLoggerClass\n\
\n\
	#\n\
	# Append\n\
	#\n\
	OpenMode \"a\"\n\
	#\n\
	# Line to log \n\
	Expression \"&if(&cmp($s,404),\\\n\
[$t] $h \\\"$r\\\" &dblookup(request,rfc822,Referer)$M)\"\n\
</Object>\n\
\n"
	os << SECTION_P52;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void Internal_writeCustomizedParagraphs( QConfig *pConfig, ostream &os )
{
    int iTmp;

#define L(x,y) (M_LookupValue(pConfig, x, y))

#define SECTION_P1 "\
# ===\n\
#\n\
#	AUTOMATICALLY GENERATED FILE\n\
#\n\
#	This files was automatically generated by the Pi3 enhanced GUI\n\
#	(EnhPi3.exe) for Windows 95 and Window NT 4.0\n\
#\n\
#	Any changes you make to this file will be lost next time the \n\
#	configuration GUI (server admin) is run and changes \n\
#	are saved.\n\
#\n\
# ===\n\
\n\
\n\
\n\
# ===\n\
#\n\
# Top level configuration\n\
#\n\
#	** IMPORTANT **\n\
#	If you do nothing else then do this:\n\
#\n\
\n\
#... ### ### Customize server settings\n\
\n\
# Skip below to the definition of 'MainVirtualHostInformation', change\n\
# the settings for Hostname (*Mandatory* redirects will not work without\n\
# this!) and Administrator (optional).\n\
#\n\
# ===\n\
\n\
# Include  standard IO and server stuff\n\
include ../Conf/IO.cnf\n\
include ../Conf/Server.cnf\n\
include ../Conf/Pi3API.cnf\n\
include ../Conf/HTTP.cnf\n\
include ../Conf/Fcgi.cnf\n\
include ../Conf/Plugins.cnf\n\
include ../Conf/ISAPI.cnf\n\
include ../Conf/PHP4.cnf\n\
include ../Conf/SSL.cnf\n\
include ../Conf/SiteRoot.cnf\n\
\n\
# ---\n\
#\n\
# IO Object for main listening socket.\n\
#\n\
# ---\n\
<Object>\n\
    Name ServerIOObject\n\
    Class TCPIPIOClass\n\
    Type Passive\n\
\n\
# A hostname must be specified for the Pi3Web HTTP server. The hostname\n\
# will be used by the server lookup the IP address that this server will\n\
# listen on. A value of INADDR_ANY let the server listen on any IP address.\n\
\n"
	/*
	** Write network preferences
	*/
	os << SECTION_P1;
	os << "\
	BindHost " << L("General", "Host") << "\n\
\n\
# A TCP port must be specified for this HTTP server. The TCP port\n\
# is the port that the HTTP will listen for requests on. This is\n\
# typically 80 for HTTP servers. \n\
#\n\
# Note that UNIX operating systems normally require that the server\n\
# initially run as user 'root' to bind on ports between 0 and 1024\n\
	BindPort " << L("General", "Port") << "\n\
\n\
# Timeout accepting connections. -1 specifies no timeout.\n\
	AcceptTimeout -1\n\
\n\
# Specify timeout for sending data. -1 specifies no timeout. \n\
	RecvTimeout " << L("General", "RecvTimeout" ) << "\n\
\n\
# Specify timeout for receiving data. -1 specifies no timeout. \n\
	SendTimeout " << L("General", "SendTimeout" ) << "\n" << endl;

#define SECTION_P1_2 "\
	PeerIPVariable STRING:RemoteAddr\n\
	PeerHostNameVariable STRING:RemoteHost\n\
	LocalAddressVariable STRING:LocalAddr\n\
	ServerPortVariable RFC822:ServerPort\n\
\n\
# Flags\n\
#\n\
#   OwnDB - Object adds its own DB, rather than use the parents\n\
#               this is necessary to avoid collision of values placed in\n\
#               the DB between different connection. \n\
#\n\
#   SetPeerAddress - IP address of remote peer with be put\n\
#               into the specified variable.\n\
#\n\
#   DNSReverseLookup - Use reverse DNS to get the hostname of the \n\
#               peer and place it in the specified variable.\n\
#\n\
#   SetLocalAddress - Put local address in the specified variable.\n\
"

	os << SECTION_P1_2;
	os << "\tFlags OwnDB|";
	if ( *M_LookupValue( pConfig, "General", "DNSReverseLookup" ) )
		{ os << "DNSReverseLookup|"; };
	os << "SetPeerAddress|SetLocalAddress" << endl;
	os << "</Object>" << endl << endl;

/*
** Write out information on the ssl object
*/
#define SECTION_P1_30 "\
# ---\n\
#\n\
# SSL IO filter object\n\
#\n\
# ---\n\
<Object>\n\
    Name SSLIOObject\n\
    Class SSLClass\n\
\n\
# Type, passive (server) or active (client)\n\
	Type \"Passive\"\n\
\n\
# SSL version,\n\
#	\"2\"=SSLv2 only\n\
#	\"3\"=SSLv3 only\n\
#	\"23\"=SSLv2 or SSLv3\n\
#	\"10\"=TLS1.0\n\
"
	os << SECTION_P1_30;
	os << "\tVersion ";
	static int __iOptions = atoi(L("SSL", "Options"));
	if ( __iOptions & 1 ) { os << "2"; };
	if ( __iOptions & 2 ) { os << "3"; };
	if ( __iOptions & 4 ) { os << "10"; };
	os << endl << endl;

#define SECTION_P1_31 "\
# Client certificate verification,\n\
#	0=No verification performed\n\
#	1=Certificate is verified, but not mandatory\n\
#	2=Certificate must be present\n\
"
	os << SECTION_P1_31;
	os << "\tVerify "<< L("SSL", "Validation") << endl;
	os << endl;
	os << "# Verify depth to verify client certificates before giving up." << endl;
	os << "\tVerifyDepth " << L("SSL", "Depth") << endl;
	os << endl;
	os << "# insert the CA certificate file path and name here" << endl;
	os << "\tCACertificatePath \"" << L("SSL", "CaCertPath") << "\"" << endl;
	os << "\tCACertificateFile \"" << L("SSL", "CaCertFile") << "\"" << endl;
	os << endl;

#define SECTION_P1_32 "\
# lower level IO object that SSL layer uses for transport\n\
#	IOObject SnoopIO IOObject=\"ServerIOObject\" OutFile=\"./encrypt.dbg\" Flag=\"Verbose\"\n\
	IOObject ServerIOObject\n\
\n"
	os << SECTION_P1_32;
	os << "# insert the name of your key file here" << endl;
	os << "\tPrivateKeyFile \"" << L("SSL", "PKeyFile") << "\"" << endl;
	os << endl;
	os << "# insert the name of your certificate file here" << endl;
	os << "\tCertificateFile \"" << L("SSL", "CertFile") << "\"" << endl;
	os << endl;

#define SECTION_P1_33 "\
# DebugFile output, filename or 'STDERR' or 'STDOUT'\n\
#	DebugFile \"STDERR\"\n\
"
	os << SECTION_P1_33;
	if ( __iOptions & 8 )
		{ os << "\tDebugFile \"../Logs/SSL.txt\"" << endl; }
	else
		{ os << "#\tDebugFile \"../Logs/SSL.txt\"" << endl; }
	os << endl;

	os << "# Flag, miscellanious options, \"Bugs\", \"Hack\", etc." << endl;
	if ( __iOptions & 16 ) { os << "\tFlag \"Bugs\"" << endl; };
	if ( __iOptions & 32 ) { os << "\tFlag \"Hack\"" << endl; };
	os << endl;

#define SECTION_P1_34 "\
# List of acceptable ciphers, disable to allow all valid cipher suites.\n\
#	CipherList \"ALL:!ADH:RC4+RSA:+SSLv2:@STRENGTH\"\n\
#	CipherList \"NULL-MD5\"\n\
"
	os << SECTION_P1_34;
	os << "\tCipherList \"" << L("SSL", "Ciphers") << "\"" << endl;

#define SECTION_P1_35 "\
</Object>\n\
\n"
	os << SECTION_P1_35;

/*
** Write out information on the main virtual host
*/
#define SECTION_P2 "\
# ---\n\
# Main virtual host object.\n\
# ---\n\
<Object>\n\
	Name MainVirtualHostInformation\n\
	Class VirtualHostClass\n"

	os << SECTION_P2;
	/*
	** Main virtual host settings
	*/
	os << "\tAdministrator " << L( "General", "Admin" ) << endl;
	os << "\tHostName " << L( "General", "Host" ) << endl;
	const char *pPort = L( "General", "Port" );
	if ( atoi( pPort )!=80 )
		{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
    os << "</Object>" << endl;

    /*
    ** Now do ip virtual host objects
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sAdministrator( pTmp, i );

        /*
        ** Write out the details
        */
        os << "# --- The IP virtual host '" << sIPAddress << "' ---" << endl;
        os << "<Object>" << endl;
        os << "\tName " << sIPAddress << endl;
        os << "\tClass VirtualHostClass" << endl;
        os << "\tAdministrator " << sAdministrator << endl;
        os << "\tHostName " << sHostname << endl;
		if ( atoi( pPort )!=80 )
			{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
        os << "</Object>" << endl;
        };

    /*
    ** Now do non-ip virtual host objects
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sAdministrator( pTmp, i );

        /*
        ** Write out the details
        */
        os << "# --- The Non-IP virtual host '" << sHostname << "' ---" << endl;
        os << "<Object>" << endl;
        os << "\tName " << sHostname << endl;
        os << "\tClass VirtualHostClass" << endl;
        os << "\tAdministrator " << sAdministrator << endl;
        os << "\tHostName " << sHostname << endl;
		if ( atoi( pPort )!=80 )
			{ os << "\tServerPort " << L( "General", "Port" ) << endl; };
        os << "</Object>" << endl;
        };

#define SECTION_P3 "\
\n\
# ---\n\
#\n\
#    Thread dispatch object\n\
#\n\
#   This is an instance of MultiThreadedIOServerClass with appropriate\n\
#   parameters\n\
#\n\
# ---\n\
<Object>\n\
	Name ThreadPoolDispatcher\n\
	Class MultiThreadedIOServerClass\n\
\n\
# The IOObject is the object that will be used for the listening socket\n\
# it can be a real IOObject (such as TCP/IP), as test IO object which \n\
# feeds stub data into the server or a filter IO object which encrpts\n\
# or logs data passed through it.\n\
#\n\
#    Example test IO object:\n\
# 	IOObject TestIOObject\n\
#\n\
#    Log IO to file:\n\
#	IOObject SnoopIO IOObject=\"ServerIOObject\" OutFile=\"./snoop.dbg\" Flag=\"Verbose\"\n\
#\n\
"
#define SECTION_P3_1 "\
\n\
# Dispatch accepted IO connection to the HTTPLogicObject\n\
	LogicObject HTTPLogicObject\n\
\n\
# Number of threads to use \n\
"
#define SECTION_P3_2 "\n\
\n\
# Number of requests to exit after\n\
"
#define SECTION_P3_3 "\n\
</Object>\n\
\n"

#define SECTION_P4 "\n\
# ---\n\
#\n\
# Main Pi3 object. \n\
#\n\
# This is an instance of the UNIXDaemonClass to fork the server into\n\
# one monitor process and 'NumberOfProcesses' child processes. \n\
#\n\
# The monitor process automatically starts a new child processes whenever\n\
# a child process exits.\n\
#\n\
# ---\n\
<Object>\n\
	Name Main\n\
	Class UNIXDaemonClass\n\
\n\
# Actual class to accept new IO requests and dispatch them\n\
	ServerObject ThreadPoolDispatcher\n\
	PIDFile \"../Logs/pid.log\"\n\
	RLimitCpu Maximum\n\
\n\
# User that child processes will run as. Server must run as root to \n\
# be able to change to the specified user. A user id number can be\n\
# specified with #number. Server which start with root privileges should\n\
# change to a user id with lesser privileges for security reasons.\n\
#   User nobody\n\
\n\
# Group that child processes will run as. Server must run as root to\n\
# be able to change to a different group. A group id number can be\n\
# specified with #number.\n\
#   Group #-1\n\
\n\
# generate a server unique lockfile name and write it into\n\
# Logs/lockfile.txt\n\
	LockFileName \"../Logs/lockfile.txt\"\n\
\n\
# Message to write just before the server is ready to start accepting\n\
# connections.\n\
	Message \"Pi3Web HTTP Server Started\"\n\
\n\
# Specify the number of processes. \n\
"
#define SECTION_P4_1 "\n\
</Object>\n\
"

	os << SECTION_P3;
	os << L("Internal", "ServerIOObjectLine" ) << endl;
	os << SECTION_P3_1;
	os << "\tMaxThreads " << L( "General", "Threads" );
	os << SECTION_P3_2;
	os << "\tExitAfter " << L( "General", "ExitAfter" );
	os << SECTION_P3_3;
	os << SECTION_P4;
	os << "\tNumberOfProcesses " << L( "General", "Processes" );
	os << SECTION_P4_1;

#define SECTION_P10 "\n\
# ---\n\
#\n\
# Start\n\
#\n\
#	Handle initial phases of requests, typically before we have enough\n\
#	information to select different execution paths\n\
#\n\
# ---\n\
<Object>\n\
	Name Start\n\
	Class FlexibleHandlerClass\n\
	Init ReturnCode ReturnCode=COMPLETED\n\
	Headers ReadRequestHeaders\n\
"
    os << SECTION_P10;

    /*
    ** Write out the HOSTMAP phases for IP virtual hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

        os << "\tHostMap VirtualHostByAddress VirtualHostObject=\""
            << sIPAddress << "\" BindAddress=\"" << sIPAddress << "\"" << endl;
        };

    /*
    ** Write out the HOSTMAP phases for Non-IP virtual hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into [IP], hostname, docroot, [administrator]
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

        os << "\tHostMap VirtualHostByName VirtualHostObject=\"" << sHostname
            << "\" HostName=\"" << sHostname << "\"" << endl;
        };

#define SECTION_P10_1 "\n\
	HostMap ReturnCode ReturnCode=COMPLETED\n\
#	Mapping DirectoryMappings\n\
\n\
	#\n\
	# Default mappings\n\
	#\n\
	Mapping MapToErrorDocument \\\n\
		IgnoreStatus=\"0 200\" \\\n\
		DefaultMessage=\"/errors/unknown.htm\" \\\n\
		301=\"/errors/301.htm\" \\\n\
		304=\"/errors/304.htm\" \\\n\
		400=\"/errors/400.htm\" \\\n\
		401=\"/errors/401.htm\" \\\n\
		403=\"/errors/403.htm\" \\\n\
		404=\"/errors/404.htm\" \\\n\
		500=\"/errors/500.htm\" \\\n\
		501=\"/errors/501.htm\"\n\
#	Mapping UserDirectory UserDirectoryRoot=\"/home/\" \\\n\
#		HTMLDirectory=\"/public_html\"\n\
"
#define SECTION_P10_2 "\n\
	HostMap ReturnCode ReturnCode=COMPLETED\n\
#	Mapping DirectoryMappings\n\
\n\
	#\n\
	# Default mappings\n\
	#\n\
	Mapping MapToErrorDocument \\\n\
		IgnoreStatus=\"0 200\" \\\n\
		DefaultMessage=\"/errors/unknown.htm\" \\\n\
		301=\"/errors/301.ssi\" \\\n\
		304=\"/errors/304.ssi\" \\\n\
		400=\"/errors/400.htm\" \\\n\
		401=\"/errors/401.ssi\" \\\n\
		403=\"/errors/403.ssi\" \\\n\
		404=\"/errors/404.ssi\" \\\n\
		500=\"/errors/500.ssi\" \\\n\
		501=\"/errors/501.htm\"\n\
#	Mapping UserDirectory UserDirectoryRoot=\"/home/\" \\\n\
#		HTMLDirectory=\"/public_html\"\n\
"
	if ( strstr( L( "General", "ErrorMessages" ), "Verbose" ))
		{ os << SECTION_P10_2; }
	else
		{ os << SECTION_P10_1; }

    /*
    ** Place holders for default document root
    */
    int iIsDocumentRoot;
    PIString sDocumentRootVirtualPath;
    const char *pDocumentRootType;
    PIString sDocumentRootRealm;

	/*
	** Write in mappings
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Mappings", 
					"Mapping", iTmp );
		if ( !pTmp ) { break; };

        iIsDocumentRoot = 0;
        sDocumentRootVirtualPath = PIString::Empty();
        pDocumentRootType = 0;
        sDocumentRootRealm = PIString::Empty();

		/*
		** Split the mappings into type, from and to
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sType( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sFrom( pTmp, i );

        /*
        ** See if this is the document root
        */
        if ( sFrom=="/" )
            {
            iIsDocumentRoot = 1;
            };

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sTo( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sRealm( pTmp, i );

		if ( sType=="CGI" )
			{ pDocumentRootType = "CGIMapper"; }
		else if ( sType=="Documents" )
			{ pDocumentRootType = "PathMapper"; }
		else if ( sType=="Windows CGI" )
			{ pDocumentRootType = "WinCGIMapper"; }
		else if ( sType=="Fast CGI" )
			{ pDocumentRootType = "FCGIMapper"; }
		else if ( sType=="ISAPI" )
			{ pDocumentRootType = "ISAPIMapper"; }
		else
			{ assert( 0 ); };

        if ( !iIsDocumentRoot )
            {
            os << "\tMapping " << pDocumentRootType << " ";
            pDocumentRootType = 0;
            };

		/*
		** Write in code to generate a realm marker	
		*/
        if ( iIsDocumentRoot )
            {
            sDocumentRootVirtualPath = sTo;
            }
        else
            {
		    os << "From=\"" << sFrom << "\" To=\"" << sTo << "\" ";
            };
		if ( strcmp( sRealm, "(none)" ) )
			{
            if ( !iIsDocumentRoot )
                {
                os << "\\" << endl;
                os << "\t\tAction=\"&dbreplace(response,string,AuthenticationRealm,";
                os << sRealm << ")\"";
                }
            else
                { sDocumentRootRealm = sRealm; };
			};
		os << endl;
		};
   
    /*
    ** Write IP hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"IP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into IP, hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sIPAddress( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);

		PIString sDocroot( pTmp, i );

        os << "\t# Document root for the IP virtual host '"
            << sIPAddress << "'" << endl;
        os << "\tMapping Condition=\"&cmp($l," << sIPAddress << ")\" \\"
            << endl;
        os << "\t\tPathMapper From=\"/\" To=\"" << sDocroot << "\"" << endl;
        };

    /*
    ** Write NonIP hosts
    */
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "VirtualHosts", 
					"NonIP", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the virtual host into hostname, docroot, administrator
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sHostname( pTmp, i );

		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sDocroot( pTmp, i );

        os << "\t# Document root for the Non-IP virtual host '"
            << sHostname << "'" << endl;
        os << "\tMapping Condition=\"&cmpi($v," << sHostname << ")\" \\" <<endl;
        os << "\t\tPathMapper From=\"/\" To=\"" << sDocroot << "\"" << endl;
        };

    /*
    ** Write document root 
    */
    os << "#" << endl;
    os << "# Default document root" << endl;
    os << "#" << endl;
    if ( iIsDocumentRoot )
        {
        os << "\tMapping " << pDocumentRootType << " ";
        os << "From=\"/\" To=\"" << sDocumentRootVirtualPath << "\" ";
        if ( sDocumentRootRealm!=PIString::Empty() )
            {
            os << "\\" << endl;
            os <<"\t\tAction=\"&dbreplace(response,string,AuthenticationRealm,";
            os << sDocumentRootRealm << ")\"";
            };

		os << "PathInfo=\"Yes\"";
		os << endl;
        };

#define SECTION_P11 "\
</Object>\n\
\n"
	os << SECTION_P11;

#define SECTION_P15 "\n\
# ---\n\
#\n\
# Default\n\
#\n\
#	Handle requests that other handlers have passed up\n\
#\n\
# ---\n\
<Object>\n\
	Name Default\n\
	Class FlexibleHandlerClass\n\
	CheckPath DirectoryExists IndexFile=\"index.ssi\" IndexFile=\"index.html\" IndexFile=\"index.htm\" IndexFile=\"default.ssi\" IndexFile=\"default.htm\" IndexFile=\"default.html\"\n\
	CheckPath RefuseFileByMask AllowFileMask=\"EF\" RefuseStatus=404\n\
	CheckPath RefuseFileByMask AllowFileMask=\"F\" RefuseStatus=403\n\
	CheckPath ReturnCode ReturnCode=COMPLETED\n\
	CheckAuth Authenticate\n\
	CheckAccess AccessByFile RequirePermissions=\"R\"\n\
\n\
	CheckType ConditionalGet \n\
	CheckType CheckForDirectory SetType=\"internal/x-directory\"\n\
\n\
	#\n\
	# Otherwise: Set the media type based on the file extension\n\
	#\n\
	CheckType MIMETypeByExtension \n\
\n\
	#\n\
	# Return 'COMPLETED' so an internal error will not be raised\n\
	#\n\
	CheckType ReturnCode ReturnCode=\"COMPLETED\"\n\
\n\
	#\n\
	# Most requests refer to a file, so put that first \n\
	#\n\
	# Handle the request by sending a file if\n\
	#	- The Value of ObjectMap in the response DB is \"\"\n\
	#	AND\n\
	#	- The Media type of the resource does not match internal/* \n\
	#\n\
	Handle Condition=\"&and(&not(&dblookup(response,string,ObjectMap)),\\\n\
&not(&regexp('internal/*',$c)))\" SendFile\n\
	Handle Condition=\"&cmp($c,internal/x-directory)\" \
"
	os << SECTION_P15;
	// allow customized directory index
	if ( atoi(L( "DirIndex", "Layout" )) == 6 )
		{ os << "CustomIndex" << endl; }
	else
		{ os << "DirIndex" << endl; }

#define SECTION_P15_1 "\
	Handle Condition=\"&cmp($c,internal/x-server-parsed-html)\" SendSSI \\\n\
		Exec=Yes ExecFileFixup=\"\\n\
/* exec file=\"\" maps to Standard CGI (Scripts) */\\\n\
&dbreplace(response,string,ObjectMap,Scripts)\\n\
&dbreplace(response,rfc822,Content-Type,internal/x-cgi)\"\n\
	Handle Condition=\"&cmp($c,internal/x-imagemap)\" ImageMap\n\
	Handle Condition=\"&cmp($c,internal/x-server-parsed-php)\" PHP4\n\
	Handle SendFile\n\
"
	os << SECTION_P15_1;

	/*
	** General logfiles
	*/
	/*
	** Write in mappings
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Logfiles", 
					"Logging", iTmp );
		if ( !pTmp ) { break; };

		/*
		** Split the logfiles into type and path
		*/
		int i;
		for( i=0; pTmp[i] && pTmp[i]!='\t'; i++);
		PIString sType( pTmp, i );
		pTmp = &(pTmp[i]);
		for( i=0; pTmp[i] && isspace(pTmp[i]); i++);
		pTmp = &(pTmp[i]);
		PIString sPath( pTmp );

		if ( sType=="Access (CLF)" )
			{ os << "	Log AccessLogger "; }
		else if ( sType=="Referer Logfile" )
			{ os << "	Log RefererLogger "; }

		os << "File=\"" << sPath << "\" \n";
		};
	os << "	Log ReturnCode ReturnCode=COMPLETED\n\
	Destroy DeleteTemporaryFiles\n\
</Object>\n\
\n";

#define SECTION_P30 "\n\
# ---\n\
#\n\
# HTTPLogicObject\n\
#\n\
# 	HTTP Logic object, dispatches requests to all other handlers\n\
#\n\
# ---\n\
<Object>\n\
	Name HTTPLogicObject\n\
	Class HTTPDispatcherClass\n\
"

#define SECTION_P30_05 "\
	ServerRoot ./../\n\
	ServerStamp Pi3Web/2.0.0\n\
	DefaultHost MainVirtualHostInformation\n\
# MIMEFile is not used here because extensions to media mappings are done \n\
# explicitly with AddMIMEEntry \n\
#	MIMEFile \"Fragment/Mime.typ\"\n\
"

#define SECTION_P30_01 "\n\
	# Specifying a file for DebugLogFile effectively turns on debugging\n"

#define SECTION_P30_02 "\n\
	#\n\
	# The following to directives specify expressions to be logged\n\
	# before and after each handler execution respectively.\n\
	# The expression as given logs process/thread and handler context\n\
	# information indented to the nested handler level.\n\
	#\n\
	# Timing information is also given for performance tuning.\n\
	#\n\
	# This logging information is EXTREMELY verbose. Don't even\n\
	# think about using it in a production server\n\
	#\n\
	DebugBeforeHandler \"[$P:$k] &align(,$X)&align(,$X) |--> $T \\\n\
-------- &align($N,12) &align($R,12) $n \\\"$r\\\" $s\"\n\
	DebugAfterHandler \"[$P:$k] &align(,$X)&align(,$X) |<-- $T \\\n\
&align($D,8) &align($N,12) &align($R,12) $n \\\"$r\\\" $s\"\n\
</Object>\n\
"
	os << SECTION_P30;

	/*
	** Write in MIME types
	*/
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "MIMETypes", 
					"Mapping", iTmp );
		if ( !pTmp ) { break; };

		os << "\tAddMIMEEntry \"" << pTmp << "\"" << endl;
		};

	os << "\tKeepOpen \"" << L( "General", "KeepOpen" ) << '"' << endl;
	if (!strcmp( L( "General", "KeepOpen" ), "On" ))
		{
		os << "\tKeepOpenCount " << L( "General", "KeepOpenCount" ) << endl;
		os << "\tKeepOpenTimeout " << M_LookupValue( pConfig, "General",
			"KeepOpenTimeout" ) << endl;
		}

	os << "\tDefaultMIMEType \"" << L( "Internal", "DefaultMIMEType" )
		<< "\"" << endl;
	os << SECTION_P30_05;
	if (*L( "General", "ErrorLog" ))
		{ os << "\tErrorLogFile \"Logs/error.txt\"" << endl; }
	else
		{ os << "#\tErrorLogFile \"Logs/error.txt\"" << endl; }
	os << L( "Internal", "HandlerLine" ) << endl;
	os << SECTION_P30_01;
	if (*L( "General", "Debug" ))
		{ os << "\tDebugLogFile \"Logs/debug.txt\"" << endl; }
	else
		{ os << "#\tDebugLogFile \"Logs/debug.txt\"" << endl; }
	os << SECTION_P30_02;

#define SECTION_P31 "\n\
# ---\n\
#\n\
# TopSiteLogic object, dispatch handlers for this website\n\
#\n\
# ---\n\
<Object>\n\
	Name TopSiteLogic\n\
	Class FlexibleHandlerClass\n\
"DEFAULT_HANDLERS"\n\
</Object>\n\
\n\
# ---\n\
#\n\
# SiteRoot object, root for this website\n\
#\n\
# ---\n\
<Object>\n\
	Name TopSiteRoot\n\
	Class SiteRootClass\n\
	LogicObject \"TopSiteLogic\"\n\
	Realm \"Administration\"\n\
	ConfigFile \""

	if (strstr(M_LookupValue( pConfig, "Internal", "HandlerLine" ), "TopSiteRoot" )) {
		PIString Abs("");
		RelativeToAbsolutePath( "./../", 
			M_LookupValue( pConfig, "Startup", "ConfigName" ), Abs );
		os << SECTION_P31;
		os << (const char *)Abs;
		os << "\"\n\
</Object>\n\
";
	}

	/*
	** Setup authentication 
	*/
#define SECTION_P60 "\n\
# ---\n\
#\n\
# Authenticate\n\
#\n\
#	Handle authentication according to realm\n\
#\n\
# ---\n\
<Object>\n\
	Name Authenticate\n\
	Class FlexibleHandlerClass\n\
	# Pass authentication if there is no realm \n\
	CheckAuth Condition=\"&not(&dblookup(response,string,AuthenticationRealm))\" \\\n\
\t\tReturnCode ReturnCode=COMPLETED\n"

	os << SECTION_P60;
	for(iTmp = 0;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Basic", iTmp );
		if ( !pTmp ) { break; };
		os << "\tCheckAuth BasicAuth_" << pTmp << endl;
		};
	for(iTmp = 0;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Digest", iTmp );
		if ( !pTmp ) { break; };
		os << "\tCheckAuth DigestAuth_" << pTmp << endl;
		};
	for(iTmp = 0;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Ntlm", iTmp );
		if ( !pTmp ) { break; };
		os << "\tCheckAuth NtlmAuth_" << pTmp << endl;
		};
	os << "</Object>" << endl;


	/*
	** Write information on the realms
	*/
#define SECTION_P61 "\n\
# ---\n\
#\n\
# Authentication realm objects\n\
#\n\
# ---\n"
	os << SECTION_P61;
	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", 
					"Basic", iTmp );
		if ( !pTmp ) { break; };

		os << "<Object>" << endl << "\tName BasicAuth_" << pTmp << endl;
		os << "\tClass BasicAuthClass" << endl; 
		/*
		** list all the users for this realm
		*/
		int iTmp2 = 0;
		PIString sRealm( "__" );
		sRealm.Concatenate( pTmp );
		os << "\tRealm \"" << pTmp << "\"" << endl;
		for(;; iTmp2++ )
			{
			const char *pTmp2 = M_LookupValueEx( pConfig, sRealm, 
					"Base64", iTmp2 );
			if ( !pTmp2 ) { break; };
			os << "\tBase64 \"" << pTmp2 << "\"" << endl;
			};

        os << "</Object>" << endl;
		};

	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", 
					"Digest", iTmp );
		if ( !pTmp ) { break; };

		os << "<Object>" << endl << "\tName DigestAuth_" << pTmp << endl;
		os << "\tClass DigestAuthClass" << endl; 
		/*
		** list all the users for this realm
		*/
		int iTmp2 = 0;
		PIString sRealm( "__" );
		sRealm.Concatenate( pTmp );
		os << "\tRealm \"" << pTmp << "\"" << endl;
		for(;; iTmp2++ )
			{
			const char *pTmp2 = M_LookupValueEx( pConfig, sRealm, 
					"Digest", iTmp2 );
			if ( !pTmp2 ) { break; };
			int iLen = strlen( pTmp2 );
			int j;
			for( j=0; j<iLen && pTmp2[j]!='\t'; j++ );
			PIString sUser(pTmp2, j);
			os << "\tUser \"" << (const char *)sUser
				<< ":" << &pTmp2[j+1] << "\"" << endl;
			};

        os << "</Object>" << endl;
		};

	iTmp = 0;
	for(;; iTmp++ )
		{
		const char *pTmp = M_LookupValueEx( pConfig, "Realms", 
					"Ntlm", iTmp );
		if ( !pTmp ) { break; };

		os << "<Object>" << endl << "\tName NtlmAuth_" << pTmp << endl;
		os << "\tClass NtlmAuthClass" << endl; 
		/*
		** no users for a Ntlm realm
		*/
        os << "</Object>" << endl;
		};

	/*
	** Setup Directory Index 
	*/
	iTmp = atoi(L("DirIndex", "Layout"));
	const char *pTmp = L("DirIndex", "SortColumn");

#define SECTION_I00 "\n\
# ---\n\
#\n\
# Directory listing - Layout: \
"
	os << SECTION_I00;
	os << __pLayoutNames[iTmp-1] << endl;

	switch ( iTmp )
		{
		/*
		** HTML table index 
		*/
		case 1:
			{

#define SECTION_I10 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY BACKGROUND=\"/icons/\\\n\
&if(&cmpi($o,MainVirtualHostInformation),Pi3Tile.gif,H2Tile.gif)\"/*background depends on vhost*/\\\n\
BGCOLOR=\"#FFFFFF\">\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<CENTER><TABLE CELLSPACING=0 CELLPADDING=0 WIDTH=100'%'>\\\n\
<TR>\
"
			os << SECTION_I10;

#define SECTION_I11 "\\\n<TH NOWARP ALIGN=LEFT>"
#define SECTION_I12 "<BR><HR SIZE=1 NOSHADE>"
#define LINKHEADER(par, hdr) "<A HREF=\"%p?" << par << "=A\">" << hdr << "</A>"

			// Tag Header (empty)
			if ( atoi(L("DirIndex", "TagNew")) )
				{ os << SECTION_I11 << SECTION_I12; };

			iTmp = atoi(L("DirIndex", "Columns"));

			// Type Header
			if ( iTmp & 1 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortType", "Type" ); }
				else
					{ os << "Type"; }
				os << SECTION_I12;
			};

			// Name Header
			if ( iTmp & 2 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Name" ); }
				else
					{ os << "Name"; }
				os << SECTION_I12;
			};

			// AltName Header
			if ( iTmp & 4 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Short Name" ); }
				else
					{ os << "Short Name"; }
				os << SECTION_I12;
			};

			// Size Header
			if ( iTmp & 8 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortSize", "Size" ); }
				else
					{ os << "Size"; }
				os << SECTION_I12;
			};

			// Date Header
			if ( iTmp & 16 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDate", "Last Modified" ); }
				else
					{ os << "Last Modified"; }
				os << SECTION_I12;
			};

			// Description Header
			if ( iTmp & 32 ) {
				os << SECTION_I11;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDscr", "Description" ); }
				else
					{ os << "Description"; }
				os << SECTION_I12;
			};
			os << "\"" << endl;
			os << "\tFilePattern \"<TR>";

#define SECTION_I13 "\\\n<TD NOWRAP ALIGN=LEFT VALIGN=TOP>"
#define SECTION_I14 "<A HREF=\"%r\"><IMG SRC=\"%i\" BORDER=NONE ALIGN=MIDDLE ALT=\"\"></A>"

			// Tag Data (empty)
			if ( atoi(L("DirIndex", "TagNew")) )
				{ os << SECTION_I13 << "%m" << "</TD>"; }

			// Type Data
			if ( iTmp & 1 )
				{ os << SECTION_I13 <<"\\" << endl << SECTION_I14 << "</TD>"; }

			// Name Data
			if ( iTmp & 2 ) {
				os << SECTION_I13 << "<A HREF=\"%r\">";
				if ( *L("DirIndex", "Truncate") )
					{ os << "&abbrev(%n," << L("DirIndex", "Truncate") << ",...)"; }
				else
					{ os << "%n"; }
				os << "</A></TD>";
			}

			// AltName Data
			if ( iTmp & 4 )
				{ os << SECTION_I13 << "<A HREF=\"%r\">%a</A>" << "</TD>"; }
		
			// Size Data
			if ( iTmp & 8 )
				{ os << SECTION_I13 << "%s" << "</TD>"; }
		
			// Date Data
			if ( iTmp & 16 )
				{ os << SECTION_I13 << "%l" << "</TD>"; }
		
			// Description Data
			if ( iTmp & 32 )
				{ os << SECTION_I13 << "&if(%d,%d,<I>%c</I>)" << "</TD>"; }
		
			os << "\"" << endl;

#define SECTION_I15 "\
	ListBottom \"</TABLE></CENTER>\\n\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	MIMEIcon \"text/html /icons/layout.gif\"\n\
	MIMEIcon \"text/x-code /icons/c.gif\"\n\
	MIMEIcon \"text/x-perl /icons/p.gif\"\n\
	MIMEIcon \"text/\* /icons/text.gif\"\n\
	MIMEIcon \"internal/x-directory /icons/folder.gif\"\n\
	MIMEIcon \"audio/\* /icons/sound2.gif\"\n\
	MIMEIcon \"image/\* /icons/image2.gif\"\n\
	MIMEIcon \"application/x-uuencoded /icons/uuencoded.gif\"\n\
	MIMEIcon \"application/x-tar /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-gzip /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-sh /icons/script.gif\"\n\
	MIMEIcon \"application/\* /icons/binary.gif\"\n\
	MIMEIcon \"x-world/x-vrml /icons/burst.gif\"\n\
	MIMEIcon \"\*/\* /icons/unknown.gif\"\n\
	SwapFileName \".. Parent Directory\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
"
			os << SECTION_I15;
			if ( atoi(L("DirIndex", "TagNew")) ) {
				os << "\tNewPattern \"<IMG SRC=\"/icons/new.gif\">\"" << endl;
				os << "\tNewTreshold " << L("DirIndex", "TagNew") << endl;
				}
			break;
			}

		/*
		** HTML index with fixed font
		*/
		case 2:
			{
#define SECTION_I20 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY BACKGROUND=\"/icons/\\\n\
&if(&cmpi($o,MainVirtualHostInformation),Pi3Tile.gif,H2Tile.gif)\"/*background depends on vhost*/\\\n\
BGCOLOR=\"#FFFFFF\">\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<PRE>\
"
			os << SECTION_I20;

#define SECTION_I21 "\\\n<B>"
#define SECTION_I22 "</B>"
#ifdef LINKHEADER
#	undef LINKHEADER
#endif
#define LINKHEADER(par, hdr, size) "<A HREF=\"%p?" << par << "=A\">" \
<< "&align(" << hdr << "</A>" << "," << size << ")"

			iTmp = atoi(L("DirIndex", "Columns"));

			// Type Header
			os << "\\\n<A HREF=\"%p?SortType=A\">";
			os << "<IMG SRC=\"/icons/blank.gif\" HSPACE=5 BORDER=NONE ALIGN=MIDDLE ALT=\"\">";
			os << "</A>";

			// Name Header
			if ( iTmp & 2 ) {
				os << SECTION_I21;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Name", "27" ); }
				else
					{ os << "&align(Name,23)"; }
				os << SECTION_I22;
			};

			// AltName Header
			if ( iTmp & 4 ) {
				os << SECTION_I21;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Short Name", "17" ); }
				else
					{ os << "&align(Short Name,13)"; }
				os << SECTION_I22;
			};

			// Size Header
			if ( iTmp & 8 ) {
				os << SECTION_I21;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortSize", "Size", "13" ); }
				else
					{ os << "&align(Size,9)"; }
				os << SECTION_I22;
			};

			// Date Header
			if ( iTmp & 16 ) {
				os << SECTION_I21;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDate", "Last Modified", "24" ); }
				else
					{ os << "&align(Last Modified,20)"; }
				os << SECTION_I22;
			};

			// Description Header
			if ( iTmp & 32 ) {
				os << SECTION_I21;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDscr", "Description", "44" ); }
				else
					{ os << "&align(Description,40)"; }
				os << SECTION_I22;
			};
			os << "\\n<HR SIZE=1 NOSHADE>\"" << endl;
			os << "\tFilePattern \"";

#define SECTION_I23 "\\\n"
#define SECTION_I24 "<A HREF=\"%r\"><IMG SRC=\"%i\" HSPACE=5 BORDER=NONE ALIGN=MIDDLE ALT=\"\"></A>"

			// Type Data
			if ( iTmp & 1 )
				{ os << SECTION_I23 << SECTION_I24; }

			// Name Data
			if ( iTmp & 2 ) {
				os << SECTION_I23 << "<A HREF=\"%r\">";
				if ( *L("DirIndex", "Truncate") )
					{ os << "&align(&abbrev(%n," << L("DirIndex", "Truncate") << ",...)</A>,27)"; }
				else
					{ os << "&align(%n</A>,27)"; }
			}

			// AltName Data
			if ( iTmp & 4 )
				{ os << SECTION_I23 << "<A HREF=\"%r\">&align(%a</A>,17)"; }
		
			// Size Data
			if ( iTmp & 8 )
				{ os << SECTION_I23 << "&align(%s,9)"; }
		
			// Date Data
			if ( iTmp & 16 )
				{ os << SECTION_I23 << "&align(%l,20)"; }
		
			// Description Data
			if ( iTmp & 32 )
				{ os << SECTION_I23 << "&align(&if(%d,%d,<I>%c</I>),40)"; }
		
			// Tag Data (empty)
			if ( atoi(L("DirIndex", "TagNew")) )
				{ os << SECTION_I23 << "%m"; }

			os << "\\n\"" << endl;

#define SECTION_I25 "\
	ListBottom \"</PRE>\\n\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	MIMEIcon \"text/html /icons/layout.gif\"\n\
	MIMEIcon \"text/x-code /icons/c.gif\"\n\
	MIMEIcon \"text/x-perl /icons/p.gif\"\n\
	MIMEIcon \"text/\* /icons/text.gif\"\n\
	MIMEIcon \"internal/x-directory /icons/folder.gif\"\n\
	MIMEIcon \"audio/\* /icons/sound2.gif\"\n\
	MIMEIcon \"image/\* /icons/image2.gif\"\n\
	MIMEIcon \"application/x-uuencoded /icons/uuencoded.gif\"\n\
	MIMEIcon \"application/x-tar /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-gzip /icons/compressed.gif\"\n\
	MIMEIcon \"application/x-sh /icons/script.gif\"\n\
	MIMEIcon \"application/\* /icons/binary.gif\"\n\
	MIMEIcon \"x-world/x-vrml /icons/burst.gif\"\n\
	MIMEIcon \"\*/\* /icons/unknown.gif\"\n\
	SwapFileName \".. Parent Directory\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
"
			os << SECTION_I25;
			if ( atoi(L("DirIndex", "TagNew")) ) {
				os << "\tNewPattern \"<IMG SRC=\"/icons/new.gif\">\"" << endl;
				os << "\tNewTreshold " << L("DirIndex", "TagNew") << endl;
				}
			break;
			}

		/*
		** Generic HTML index
		*/
		case 3:
			{
#define SECTION_I30 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY>\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<PRE>\
"
			os << SECTION_I30;

#define SECTION_I31 "\\\n<B>"
#define SECTION_I32 "</B>"
#ifdef LINKHEADER
#	undef LINKHEADER
#endif
#define LINKHEADER(par, hdr, size) "<A HREF=\"%p?" << par << "=A\">" \
<< "&align(" << hdr << "</A>" << "," << size << ")"

			iTmp = atoi(L("DirIndex", "Columns"));

			// Type Header
			os << "\\\n'&nbsp;'";

			// Name Header
			if ( iTmp & 2 ) {
				os << SECTION_I31;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Name", "27" ); }
				else
					{ os << "&align(Name,23)"; }
				os << SECTION_I32;
			};

			// AltName Header
			if ( iTmp & 4 ) {
				os << SECTION_I31;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Short Name", "17" ); }
				else
					{ os << "&align(Short Name,13)"; }
				os << SECTION_I32;
			};

			// Size Header
			if ( iTmp & 8 ) {
				os << SECTION_I31;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortSize", "Size", "13" ); }
				else
					{ os << "&align(Size,9)"; }
				os << SECTION_I32;
			};

			// Date Header
			if ( iTmp & 16 ) {
				os << SECTION_I31;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDate", "Last Modified", "24" ); }
				else
					{ os << "&align(Last Modified,20)"; }
				os << SECTION_I32;
			};

			// Description Header
			if ( iTmp & 32 ) {
				os << SECTION_I31;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDscr", "Description", "44" ); }
				else
					{ os << "&align(Description,40)"; }
				os << SECTION_I32;
			};
			os << "\\n<HR SIZE=1 NOSHADE>\"" << endl;
			os << "\tFilePattern \"";

#define SECTION_I33 "\\\n"
#define SECTION_I34 "<A HREF=\"%r\"><IMG SRC=\"%i\" HSPACE=5 BORDER=NONE ALIGN=MIDDLE ALT=\"\"></A>"

			// Type Data
			if ( iTmp & 1 )
				{ os << SECTION_I33 << "'&nbsp;'"; }

			// Name Data
			if ( iTmp & 2 ) {
				os << SECTION_I33 << "<A HREF=\"%r\">";
				if ( *L("DirIndex", "Truncate") )
					{ os << "&align(&abbrev(%n," << L("DirIndex", "Truncate") << ",...)</A>,27)"; }
				else
					{ os << "&align(%n</A>,27)"; }
			}

			// AltName Data
			if ( iTmp & 4 )
				{ os << SECTION_I33 << "<A HREF=\"%r\">&align(%a</A>,17)"; }
		
			// Size Data
			if ( iTmp & 8 )
				{ os << SECTION_I33 << "&align(%s,9)"; }
		
			// Date Data
			if ( iTmp & 16 )
				{ os << SECTION_I33 << "&align(%l,20)"; }
		
			// Description Data
			if ( iTmp & 32 )
				{ os << SECTION_I33 << "&align(&if(%d,%d,<I>%c</I>),40)"; }
		
			// Tag Data (empty)
			if ( atoi(L("DirIndex", "TagNew")) )
				{ os << SECTION_I33 << "%m"; }

			os << "\\n\"" << endl;

#define SECTION_I35 "\
	ListBottom \"</PRE>\\n\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
"
			os << SECTION_I35;

			if ( atoi(L("DirIndex", "TagNew")) ) {
				os << "\tNewPattern \"\*\* NEW! \*\*\"" << endl;
				os << "\tNewTreshold " << L("DirIndex", "TagNew") << endl;
				}
			break;
			}

		/*
		** DOS style HTML index
		*/
		case 4:
			{
#define SECTION_I40 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class DirectoryIndexClass\n\
	HeaderPattern \"<HTML><TITLE>Index of %p</TITLE>\\\n\
<BODY>\\\n\
<H2>Index of %p</H2>\\n\"\n\
	ListTop \"<PRE>\
"
			os << SECTION_I40;

#define SECTION_I41 "\\\n<B>"
#define SECTION_I42 "</B>"
#ifdef LINKHEADER
#	undef LINKHEADER
#endif
#define LINKHEADER(par, hdr, size) "<A HREF=\"%p?" << par << "=A\">" \
<< "&align(" << hdr << "</A>" << "," << size << ")"

			iTmp = atoi(L("DirIndex", "Columns"));

			// AltName Header
			if ( iTmp & 4 ) {
				os << SECTION_I41;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "DOS Name", "17" ); }
				else
					{ os << "&align(DOS Name,13)"; }
				os << SECTION_I42;
			};

			// Size Header
			if ( iTmp & 8 ) {
				os << SECTION_I41;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{
					os << "     <A HREF=\"%p?SortSize=A\">Size</A>";
					}
				else
					{ os << "&align(Size,9,right)"; }
				os << "  " << SECTION_I42;
			};

			// Date Header
			if ( iTmp & 16 ) {
				os << SECTION_I41;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortDate", "Last Modified", "22" ); }
				else
					{ os << "&align(Last Modified,18)"; }
				os << SECTION_I42;
			};

			// Name Header
			if ( iTmp & 2 ) {
				os << SECTION_I41;
				if ( atoi(L("DirIndex", "SortFlags")) & 1 )
					{ os << LINKHEADER( "SortName", "Name", "27" ); }
				else
					{ os << "&align(Name,23)"; }
				os << SECTION_I42;
			};

			os << "\\n<HR SIZE=1 NOSHADE>\"" << endl;
			os << "\tFilePattern \"";

			// AltName Data
			if ( iTmp & 4 )
				{ os << "\\\n<A HREF=\"%r\">&if(&cmp(%n,..),&align(..</A>,17),&align(%a</A>,17))"; }

			// Size Data
			if ( iTmp & 8 )
				{ os << "\\\n&if(%b,&align('&lt;DIR&gt;',15),&align(%s,9,right))" << "  "; }
		
			// Date Data
			if ( iTmp & 16 )
				{ os << "\\\n&align(%l,18)"; }

			// Name Data
			if ( iTmp & 2 ) {
				os << "\\\n<A HREF=\"%r\">";
				if ( *L("DirIndex", "Truncate") )
					{ os << "&align(&abbrev(%n," << L("DirIndex", "Truncate") << ",...)</A>,27)"; }
				else
					{ os << "&align(%n</A>,27)"; }
			}
			os << "\\n\"" << endl;

#define SECTION_I45 "\
	ListBottom \"</PRE>\\n\"\n\
	FooterPattern \"<HR SIZE=1 NOSHADE>\\\n\
Contact the hostmaster <A HREF=\"mailto:$a\"<I>$a</I></A>.\\\n\
</BODY></HTML>\"\n\
	LastModifiedFormat \"%d/%m/%y %I:%M%p\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
"
			os << SECTION_I45;
			break;
			}

		/*
		** 'Text with Tabs' index
		*/
		case 5:
			{
#define SECTION_I50 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class DirectoryIndexClass\n\
"
			os << SECTION_I50;
			os << "\tFilePattern \"";

			iTmp = atoi(L("DirIndex", "Columns"));

			// Name Data
			if ( iTmp & 2 ) {
				if ( *L("DirIndex", "Truncate") )
					{ os << "&abbrev(%n," << L("DirIndex", "Truncate") << ",...)"; }
				else
					{ os << "%n"; }

			// AltName Data
			if ( iTmp & 4 )
				{ if (iTmp & 2) os << "\\t"; os << "%a"; }

			// Size Data
			if ( iTmp & 8 )
				{ if (iTmp & 6) os << "\\t"; os << "%s"; }
		
			// Date Data
			if ( iTmp & 16 )
				{ if (iTmp & 14) os << "\\t"; os << "%l"; }

			// Description Data
			if ( iTmp & 32 )
				{ if (iTmp & 30) os << "\\t"; os << "&if(%d,%d,%c)"; }
		
			// Tag Data (empty)
			if ( atoi(L("DirIndex", "TagNew")) )
				{ if (iTmp & 62) os << "\\t"; os << "%m"; }

			}
			os << "\\n\"" << endl;

#define SECTION_I55 "\
	HTTPEquiv \"Content-Type: text/plain\"\n\
	LastModifiedFormat \"%d-%b-%y %H:%M\"\n\
	Include \"\*\"\n\
	Exclude \".\"\n\
	Exclude \"~\*\"\n\
	Exclude \".desc\"\n\
	Exclude \"README\"\n\
	Exclude \"HEADER\"\n\
"
			os << SECTION_I55;
			if ( atoi(L("DirIndex", "TagNew")) ) {
				os << "\tNewPattern \"new\"" << endl;
				os << "\tNewTreshold " << L("DirIndex", "TagNew") << endl;
				}
			break;
			}

		/*
		** Custom index, this isn't handled here
		*/
		case 6:
			{
#define SECTION_I60 "\
#\n\
# ---\n\
<Object>\n\
	Name DirIndex\n\
	Class FlexibleHandlerClass\n\
	Handle StatusCode StatusCode=404\n\
	Handle ReturnCode ReturnCode=COMPLETED\n\
"
			os << SECTION_I60;
			break;
			}

		default:
			assert(0);
		};

	if ( iTmp != 6 )
		{
		if ( atoi(L("DirIndex", "FileFlags")) & 1 )
			{ os << "\tHeaderFile \"HEADER\"" << endl; }

		if ( atoi(L("DirIndex", "FileFlags")) & 2 )
			{ os << "\tFooterFile \"README\"" << endl; }

		if ( atoi(L("DirIndex", "FileFlags")) & 4 )
			{ os << "\tDescriptionFile \".desc\"" << endl; }

		os << "\tOptions \"";
		if ( iTmp & 1 ) { os << "Icon | "; }
		if ( iTmp & 2 ) { os << "Name | "; }
		if ( iTmp & 3 ) { os << "AltName | "; }
		os << L("DirIndex", "SizeFormat") << " | ";
		if ( strlen(pTmp) && strcmp(pTmp,"Description"))
			{ os << "Sort" << pTmp << " | "; }
		if ( strlen(pTmp) && !strcmp(pTmp,"Description"))
			{ os << "SortDscr | "; }
		if ( atoi(L("DirIndex", "TagNew")) ) { os << "TagNew | "; }
		if ( atoi(L("DirIndex", "SortFlags")) & 2 ) { os << "SortCase | "; }
		if ( atoi(L("DirIndex", "SortFlags")) & 4 ) { os << "SortDesc | "; }
		os << "RelPath | MIMEType\"" << endl;
		}

	os << "</Object>" << endl;
    os << endl;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Internal_writeDBValues( QConfig *pConfig, ostream &os )
{
    const char *pSectionName;
    for( int i=0; (pSectionName = M_LookupSection( pConfig, i )); i++ )
        {
        /*
        ** Dispatch before write callback, this function returns an
        ** integer value, non-zero for continue, zero for error/ignore.
        **
        ** This value is ignored right now. See below for after write 
        ** callback
        ** 
        */
        M_DispatchWriteCallBacks( pConfig, pSectionName,
            M_CB_BEFORESECTIONWRITE, os );
        
        os << '#' << SECTION_START << pSectionName << endl;
        const char *pVariable;
        const char *pValue;
        for( int j=0;
            ( M_LookupPair( pConfig, pSectionName, j, &pVariable, &pValue )); 
            j++
            )
            {
            os << '#' << pVariable << ':' << pValue << endl;
            };

        os << '#' << SECTION_END << endl;
        /*
        ** Dispatch after write callback
        **
        ** see notes above
        ** 
        */
        M_DispatchWriteCallBacks( pConfig, pSectionName,
            M_CB_AFTERSECTIONWRITE, os );
        };

    return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Pi3File_writeConfig( QConfig *pConfig, ostream &os )
{
	assert( pConfig );

    /*
    ** Set any defaults that have not already been set
    */
// LATER ---- this is temporary 0 until everything is ready.....
Internal_setupDefaults( pConfig );
    const char *pRetained = pConfig->GetRetained();

	/*
	** Write Pi3 language for the server
	*/
	os << "#!../bin/Pi3" << endl << endl << '#' << GENERATED_START << endl;
	Internal_writeCustomizedParagraphs( pConfig, os );
	os << '#' << GENERATED_END << endl;
	os << '#' << NOTMODIFIED_START << endl;

    if ( pRetained && *pRetained )
        {
        os << pRetained;
        }
    else
        {
        Internal_writeDefaultParagraphs( os ); 
        };

	os << '#' << NOTMODIFIED_END << endl;
	
    /*
    ** Write in the QConfig values
    */
    Internal_writeDBValues( pConfig, os );

	return 0;	/* success */
}


/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_newValue( QConfig *pConfig, const char *pSection,
    const char *pPair )
{
    assert( pPair );

    int i;
    for( i=0; pPair[i] && pPair[i]!=':'; i++ );
    PIString sVariable( pPair, i );
    PIString sValue;
    if ( pPair[i] )
        { sValue = &(pPair[i+1]); };
    pConfig->AddValue( pSection, sVariable, sValue );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Pi3File_readConfig( QConfig *pConfig, istream &is )
{
    enum { BUF_SIZE=2047 };
	char szBuf[BUF_SIZE+1];
	
	PIString *pRetained = PI_NEW( PIString );
	
    PIString *pCurrentSection = 0;
	
    enum { NORMAL, SECTION, RETAINED } eState = NORMAL;
	int iSections = 0;
	
	for(; !is.eof();)
		{
		*szBuf = '\0';
		is.getline( szBuf, BUF_SIZE );
		szBuf[BUF_SIZE] = '\0';
	
		if ( eState!=RETAINED && ( *szBuf=='\0' || *szBuf!='#' ) )
			{ continue; };
	
		switch( eState )
			{
			case NORMAL:
				if ( !strncmp( &(szBuf[1]), SECTION_START, 
					sizeof( SECTION_START )-1 ) )
					{
                    PI_DELETE( pCurrentSection );
                    pCurrentSection = 0;
                    pCurrentSection = PI_NEW( PIString( 
                        &(szBuf[sizeof(SECTION_START)]) ) );
                    eState = SECTION;
                    }
				else if ( !strncmp( &(szBuf[1]), NOTMODIFIED_START, 
					sizeof( NOTMODIFIED_START )-1 ) )
					{ eState = RETAINED; };
				break;
	
			case SECTION:
				if ( !strncmp( &(szBuf[1]), SECTION_END, 
					sizeof( SECTION_END )-1 ) )
					{ eState = NORMAL; }
				else
                    {
                    Internal_newValue(pConfig, *pCurrentSection, &szBuf[1]);
					iSections++;
                    };
				break;
					
			case RETAINED:
				if ( !strncmp( &(szBuf[1]), NOTMODIFIED_END, 
					sizeof( NOTMODIFIED_END )-1 ) )
					{ eState = NORMAL; }
				else
					{
					assert( pRetained );
					pRetained->Concatenate( szBuf );
					pRetained->Concatenate( '\n' );
					};
				break;
				
			default:
				assert( 0 );
			};		
		};
	
	/*
	** Done!
	*/
    pConfig->SetRetained( *pRetained );

	/*
	** No sections = Error! (0 = OK)
	*/
	return ( iSections > 0 ) ? 0 : 1;
}
	
