/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/Ssl.cpp,v $
 * $Date: 2002/08/02 06:58:56 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"

#include "QConfig.h"
#include "Ssl.hrc"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define C_RESOURCENAME                    "SslPage"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    SslPage

Description:
    Creates the eponymous property page and is used by the PropSheet object.

Notes:
	The changes you make in this page won't partial immediately affect the behaviour of
	the server but only after restart. This applies to the connection parameters.
	Changes may lead to a configuration, which is inapplicable for remote administration
	or requires corresponding changes in the configuration of the remote interface
	application, particular of the PropSheet object.

Example:
	<PRE>
	&lt;Object&gt;
		Name SslPage
		Class SslPageClass
		PropertyPage SslPage
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class SslPage : public IFaceBase
{
private:
    QConfig *pConfig;
    PROPSHEETPAGE tPsp;
    Interface *pInterface;

    /*
    ** window private values
    */
	HWND hWndPort;
	HWND hWndURL;
	HWND hWndSsl;
	HWND hWndSsl2;
	HWND hWndSsl3;
	HWND hWndTls1;
	HWND hWndDebug;
	HWND hWndPkFile;
	HWND hWndCertFile;
	HWND hWndVal0;
	HWND hWndVal1;
	HWND hWndVal2;
	HWND hWndDepth;
	HWND hWndBug;
	HWND hWndHack;
	HWND hWndCaCertPath;
	HWND hWndCaCertFile;
	HWND hWndCiphers;
	int __iValidation;

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "SslPage: ";

        if ( 0 )
            {
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	SslPage( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pConfig( FindParentQConfig( PIObject_getDB( pObject ) ) ),
        pInterface( 0 ),
		hWndPort( 0 ),
		hWndURL( 0 ),
		hWndSsl( 0 ),
		hWndSsl2( 0 ),
		hWndSsl3( 0 ),
		hWndTls1( 0 ),
		hWndDebug( 0 ),
		hWndPkFile( 0 ),
		hWndCertFile( 0 ),
		hWndVal0( 0 ),
		hWndVal1( 0 ),
		hWndVal2( 0 ),
		hWndDepth( 0 ),
		hWndBug( 0 ),
		hWndHack( 0 ),
		hWndCaCertPath( 0 ),
		hWndCaCertFile( 0 ),
		hWndCiphers( 0 ),
		__iValidation( 0 )
		{
		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        /*
        ** Setup the PROPSHEETPAGE
        */
        memset( &tPsp, 0, sizeof( PROPSHEETPAGE ) );
        tPsp.dwSize = sizeof( PROPSHEETPAGE );
        tPsp.dwFlags = PSP_DEFAULT;
        tPsp.hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );

        /* Status page */
        tPsp.pszTemplate = C_RESOURCENAME;
        tPsp.pfnDlgProc = (DLGPROC)fnDialogProc;
        tPsp.lParam = (LPARAM)this;

        /*
        ** Set the data DB of this object
        */
        PIDB *pDB = PIObject_getDB( Object() );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_WINDOWTYPE,
            (void *)INTRFACE_PROPPAGE, 0 );
        assert(!PIDB_lookup( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE, 0));
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE,
            (void *)&tPsp, 0 );
        
        /*
        ** Done
        */
		};

	~SslPage()
		{
		};

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                pInterface = &tWindow;
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return PIAPI_ABORT;

            case INTRFACE_MSG_LOAD:
                return PIAPI_ABORT;

            case INTRFACE_MSG_DEFAULTS:
				M_AddValue( pConfig, "SSL", "Debug", "" );
				// SSL2 SSL3 TLS1 Debg Bugs Hack
				// 1    2    4    8    16   32
				M_AddValue( pConfig, "SSL", "Options", "51" );
				M_AddValue( pConfig, "SSL", "PKeyFile", "./key.pem" );
				M_AddValue( pConfig, "SSL", "CertFile", "./cert.pem" );
				M_AddValue( pConfig, "SSL", "Validation", "0" );
				M_AddValue( pConfig, "SSL", "Depth", "1" );
				M_AddValue( pConfig, "SSL", "CaCertPath", "./" );
				M_AddValue( pConfig, "SSL", "CaCertFile", "cacert.pem" );
				M_AddValue( pConfig, "SSL", "Ciphers", "!ADH:RC4+RSA:HIGH:MEDIUM:LOW:EXP:+SSLv2:+EXP" );
                break;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

	void Internal_updateSSL( int iSSL )
		{
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		*szBuf = '\0';
		SendMessage( hWndPort, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
	
		if ( iSSL )
			{
			M_ReplaceValue( pConfig, "Internal", "URLProtocol", "https://" );
			M_ReplaceValue( pConfig, "Internal", "DefaultPort", "443" );
			if (!stricmp(szBuf, "80")) {
				SendMessage( hWndPort, WM_SETTEXT, 0, (LPARAM)"443");
				};
			EnableWindow( hWndSsl2, TRUE ); EnableWindow( hWndSsl3, TRUE );
			EnableWindow( hWndTls1, TRUE ); EnableWindow( hWndDebug, TRUE );
			EnableWindow( hWndPkFile, TRUE ); EnableWindow( hWndCertFile, TRUE );
			EnableWindow( hWndVal0, TRUE ); EnableWindow( hWndVal1, TRUE );
			EnableWindow( hWndVal2, TRUE ); EnableWindow( hWndDepth, TRUE );
			EnableWindow( hWndCaCertPath, TRUE ); EnableWindow( hWndCaCertFile, TRUE );
			EnableWindow( hWndBug, TRUE ); EnableWindow( hWndHack, TRUE );
			EnableWindow( hWndCiphers, TRUE );
			}
		else
			{
			M_ReplaceValue( pConfig, "Internal", "URLProtocol", "http://" );
			M_ReplaceValue( pConfig, "Internal", "DefaultPort", "80" );
			if (!stricmp(szBuf, "443")) {
				SendMessage( hWndPort, WM_SETTEXT, 0, (LPARAM)"80");
				};
			EnableWindow( hWndSsl2, FALSE ); EnableWindow( hWndSsl3, FALSE );
			EnableWindow( hWndTls1, FALSE ); EnableWindow( hWndDebug, FALSE );
			EnableWindow( hWndPkFile, FALSE ); EnableWindow( hWndCertFile, FALSE );
			EnableWindow( hWndVal0, FALSE ); EnableWindow( hWndVal1, FALSE );
			EnableWindow( hWndVal2, FALSE ); EnableWindow( hWndDepth, FALSE );
			EnableWindow( hWndCaCertPath, FALSE ); EnableWindow( hWndCaCertFile, FALSE );
			EnableWindow( hWndBug, FALSE ); EnableWindow( hWndHack, FALSE );
			EnableWindow( hWndCiphers, FALSE );
			};
		}

	void Internal_updateURL( HWND hDlg )
		{
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		/* --- Get current value --- */
		assert( M_LookupValue( pConfig, "Internal", "URLProtocol" ) );
		strcpy( szBuf, M_LookupValue( pConfig, "Internal", "URLProtocol" ) );
		int iLen = strlen( szBuf );
		assert( M_LookupValue( pConfig, "General", "Host" ) );
		strcpy( &szBuf[iLen], M_LookupValue( pConfig, "General", "Host" ) );
		iLen = strlen( szBuf );
		szBuf[iLen++] = ':';
		SendMessage( hWndPort, WM_GETTEXT, BUF_SIZE-iLen, (LPARAM)&(szBuf[iLen]) );
		assert( M_LookupValue( pConfig, "Internal", "DefaultPort" ) );
		if ( atoi( &(szBuf[iLen]) )==atoi(M_LookupValue( pConfig, "Internal", 
				"DefaultPort" )) )
			{ szBuf[--iLen] = '\0'; };
	
		/* --- now update the URL --- */
		SendMessage( hWndURL, WM_SETTEXT, 0, (LPARAM)szBuf );
		
		/* --- mark property sheet as modified --- */
		Changed( hDlg );
		}
	

	BOOL DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
	    {
	    enum { BUF_SIZE=1023 };
	    char szBuf[BUF_SIZE+1];

		switch (uMsg)
			{
			case WM_INITDIALOG:
				{
				hWndPort = GetDlgItem( hDlg, IDC_PORT );
				hWndURL = GetDlgItem( hDlg, IDC_URL );
				hWndSsl = GetDlgItem( hDlg, IDC_SSL );
				hWndSsl2 = GetDlgItem( hDlg, IDC_SSL2 );
				hWndSsl3 = GetDlgItem( hDlg, IDC_SSL3 );
				hWndTls1 = GetDlgItem( hDlg, IDC_TLS1 );
				hWndDebug = GetDlgItem( hDlg, IDC_DEBUG );
				hWndPkFile = GetDlgItem( hDlg, IDC_PKEYFILE );
				hWndCertFile = GetDlgItem( hDlg, IDC_CERTFILE );
				hWndVal0 = GetDlgItem( hDlg, IDC_NONE );
				hWndVal1 = GetDlgItem( hDlg, IDC_OPTIONAL );
				hWndVal2 = GetDlgItem( hDlg, IDC_MANDATORY );
				hWndDepth = GetDlgItem( hDlg, IDC_DEPTH );
				hWndBug = GetDlgItem( hDlg, IDC_MSIEBUG );
				hWndHack = GetDlgItem( hDlg, IDC_HACK );
				hWndCaCertPath = GetDlgItem( hDlg, IDC_CAPATH );
				hWndCaCertFile = GetDlgItem( hDlg, IDC_CAFILE );
				hWndCiphers = GetDlgItem( hDlg, IDC_CIPHERS );
	
				assert( hWndPort && hWndSsl && hWndURL );
				assert( hWndSsl2 && hWndSsl3 && hWndTls1 );
				assert( hWndDebug && hWndPkFile && hWndCertFile );
				assert( hWndDepth && hWndVal0 && hWndVal1 && hWndVal2 );
				assert( hWndCaCertPath && hWndCaCertFile && hWndCiphers );
				assert( hWndBug && hWndHack );
	
				::SendMessage( hWndSsl, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndSsl2, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndSsl3, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndTls1, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndDebug, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndBug, BM_SETCHECK, BST_UNCHECKED, 0 );
				::SendMessage( hWndHack, BM_SETCHECK, BST_UNCHECKED, 0 );
	
				/* --- just for sanity --- */
				::SendMessage( hWndPort, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndPkFile, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndCertFile, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndDepth, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndCaCertPath, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndCaCertFile, EM_SETLIMITTEXT, 256, 0 );
				::SendMessage( hWndCiphers, EM_SETLIMITTEXT, 256, 0 );
	
				/* --- SSL, Port and URL --- */
				// are set in PSN_SETACTIVE handler
	
				/* --- SSL version and options --- */
				int i = atoi(M_LookupValue( pConfig, "SSL", "Options" ));
				::SendMessage( hWndSsl2, BM_SETCHECK, (WPARAM)(i & 1), 0 );
				::SendMessage( hWndSsl3, BM_SETCHECK, (WPARAM)(i & 2), 0 );
				::SendMessage( hWndTls1, BM_SETCHECK, (WPARAM)(i & 4), 0 );
				::SendMessage( hWndDebug, BM_SETCHECK, (WPARAM)(i & 8), 0 );
				::SendMessage( hWndBug, BM_SETCHECK, (WPARAM)(i & 16), 0 );
				::SendMessage( hWndHack, BM_SETCHECK, (WPARAM)(i & 32), 0 );
	
				/* server Private Key and Certificate file path --- */
				const char *pVal = M_LookupValue( pConfig, "SSL", "PKeyFile" );
				::SendMessage( hWndPkFile, WM_SETTEXT, 0, (LPARAM)pVal );
				pVal = M_LookupValue( pConfig, "SSL", "CertFile" );
				::SendMessage( hWndCertFile, WM_SETTEXT, 0, (LPARAM)pVal );
	
				/* --- User cert validation --- */
				__iValidation = atoi(M_LookupValue( pConfig, "SSL", "Validation" ));
				::SendMessage( hWndVal0, BM_SETCHECK, (WPARAM)(__iValidation==0), 0 );
				::SendMessage( hWndVal1, BM_SETCHECK, (WPARAM)(__iValidation==1), 0 );
				::SendMessage( hWndVal2, BM_SETCHECK, (WPARAM)(__iValidation==2), 0 );
	
				/* --- validation depth, CA Certificate path and filename --- */
				pVal = M_LookupValue( pConfig, "SSL", "Depth" );
				::SendMessage( hWndDepth, WM_SETTEXT, 0, (LPARAM)pVal );
				pVal = M_LookupValue( pConfig, "SSL", "CaCertPath" );
				::SendMessage( hWndCaCertPath, WM_SETTEXT, 0, (LPARAM)pVal );
				pVal = M_LookupValue( pConfig, "SSL", "CaCertFile" );
				::SendMessage( hWndCaCertFile, WM_SETTEXT, 0, (LPARAM)pVal );
	
				/* --- List of ciphers --- */
				pVal = M_LookupValue( pConfig, "SSL", "Ciphers" );
				::SendMessage( hWndCiphers, WM_SETTEXT, 0, (LPARAM)pVal );
	
				Unchanged( hDlg );
				};
				break;
	
			case WM_COMMAND:
				if ( GET_WM_COMMAND_CMD(wParam, lParam)==BN_CLICKED )
					{
					switch( GET_WM_COMMAND_ID( wParam, lParam) )
						{
						case IDC_SSL:
							Internal_updateSSL( ::SendMessage( hWndSsl, BM_GETCHECK,
								0, 0 ) == BST_CHECKED );
							Internal_updateURL( hDlg );
							break;
						case IDC_SSL2:
						case IDC_SSL3:
							::SendMessage( hWndTls1, BM_SETCHECK, (FALSE), 0 );
							break;
						case IDC_TLS1:
							::SendMessage( hWndSsl2, BM_SETCHECK, (FALSE), 0 );
							::SendMessage( hWndSsl3, BM_SETCHECK, (FALSE), 0 );
							break;
						case IDC_NONE:
							__iValidation = 0; break;
						case IDC_OPTIONAL:
							__iValidation = 1; break;
						case IDC_MANDATORY:
							__iValidation = 2; break;
	
						default:;
						};
	
					/* --- mark property sheet as modified --- */
					Changed( hDlg );
					}
				else if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
					{
					switch( GET_WM_COMMAND_ID( wParam, lParam) )
						{
						case IDC_PORT:
							Internal_updateURL( hDlg );
							break;
						default:;
							/* --- mark property sheet as modified --- */
							Changed( hDlg );
						};
					};
				break;
	
			case WM_DESTROY:
				break;
	
			case WM_NOTIFY:
				switch( ((NMHDR *)lParam)->code )
					{
					case PSN_SETACTIVE:
						{
						int iSaveChanged = IsChanged();
						SendMessage( hWndPort, WM_SETTEXT, 0, (LPARAM)
							M_LookupValue( pConfig, "General", "Port" ) );
	
						const char *pTest = M_LookupValue( pConfig, "Internal", \
							"ServerIOObjectLine" );
						SendMessage( hWndSsl, BM_SETCHECK, (WPARAM)strstr( pTest, "SSLIO" ), 0 );
						Internal_updateSSL( ::SendMessage( hWndSsl, BM_GETCHECK,
								0, 0 ) == BST_CHECKED );
						Internal_updateURL( hDlg );
						if ( !iSaveChanged ) { Unchanged( hDlg ); };
						SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
						return (TRUE);
						}
	
					case PSN_QUERYCANCEL:
						return SHEET->Cancel(pInterface, hDlg);
					
					case PSN_KILLACTIVE:
						if ( IsChanged() )
						{
						*szBuf = 0;
						SendMessage( hWndPort, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						int iPort = atoi( szBuf );
						if ( iPort<0 || iPort>65535 )
							{
							/*
							** Give message and abort change
							*/
							MessageBox( hDlg, "Port must be in the range [0..65536]",
								"Port Field Error", MB_OK );
							SetFocus( hWndPort );
							SendMessage( hWndPort, EM_SETSEL, 0, -1 );
							SetWindowLong( hDlg, DWL_MSGRESULT, PSNRET_INVALID_NOCHANGEPAGE );
							return (TRUE);
							};
						M_ReplaceValue( pConfig, "General", "Port", szBuf ); 
						SendMessage( hWndURL, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "Identity", "URL", szBuf );
	
						if ( ::SendMessage( hWndSsl, BM_GETCHECK, 0, 0 )==BST_CHECKED )
							{
							M_ReplaceValue( pConfig, "Internal", "ServerIOObjectLine", \
								"\tIOObject SSLIOObject" );
							}
						else
							{
							M_ReplaceValue( pConfig, "Internal", "ServerIOObjectLine", \
								"\tIOObject ServerIOObject" );
							}
	
						/* --- SSL version and options --- */
						int iIdx = 0;
						if ( ::SendMessage( hWndSsl2, BM_GETCHECK, 0, 0 )) { iIdx += 1; };
						if ( ::SendMessage( hWndSsl3, BM_GETCHECK, 0, 0 )) { iIdx += 2; };
						if ( ::SendMessage( hWndTls1, BM_GETCHECK, 0, 0 )) { iIdx += 4; };
						if ( ::SendMessage( hWndDebug, BM_GETCHECK, 0, 0 )) { iIdx += 8; };
						if ( ::SendMessage( hWndBug, BM_GETCHECK, 0, 0 )) { iIdx += 16; };
						if ( ::SendMessage( hWndHack, BM_GETCHECK, 0, 0 )) { iIdx += 32; };
						M_ReplaceValue( pConfig, "SSL", "Options", itoa( iIdx,szBuf,10 ));
	
						/* server Private Key and Certificate file path --- */
						*szBuf = 0;
						::SendMessage( hWndPkFile, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "PKeyFile", szBuf );
						*szBuf = 0;
						::SendMessage( hWndCertFile, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "CertFile", szBuf );
	
						/* --- User cert validation --- */
						M_ReplaceValue( pConfig, "SSL", "Validation",
							itoa( __iValidation,szBuf,10 ));
	
						/* --- validation depth, CA Certificate path and filename --- */
						*szBuf = 0;
						::SendMessage( hWndDepth, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "Depth", szBuf );
						*szBuf = 0;
						::SendMessage( hWndCaCertPath, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "CaCertPath", szBuf );
						*szBuf = 0;
						::SendMessage( hWndCaCertFile, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "CaCertFile", szBuf );
	
						/* --- List of ciphers --- */
						*szBuf = 0;
						::SendMessage( hWndCiphers, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						M_ReplaceValue( pConfig, "SSL", "Ciphers", szBuf );
	
						}
						SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
						return (TRUE);
	
					default:
						return (FALSE);
					};
				break;
	
			default:
				return (FALSE);
			};
			return (TRUE);
		}
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, ((LPPROPSHEETPAGE)lParam)->lParam );
        };

    SslPage *pPage = (SslPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->DialogProc( hDlg, uMsg, wParam, lParam ) : FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int SslPage_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( SslPage( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name SslPageClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor SslPage_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name SslPage
		Class SslPageClass
	</Object>

/*___+++CNF_END+++___*/
#endif

