/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/Users.cpp,v $
 * $Date: 2002/08/02 06:58:56 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"
#include "md5.h"
#include "Base64.h"

#include "QConfig.h"
#include "Users.hrc"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define C_RESOURCENAME                    "UsersPage"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    UsersPage

Description:
    Creates the eponymous property page and is used by the PropSheet object.

Example:
	<PRE>
	&lt;Object&gt;
		Name UsersPage
		Class UsersPageClass
		PropertyPage UsersPage
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
static BOOL CALLBACK fnNewRealmDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class UsersPage : public IFaceBase
{
private:
    QConfig *pConfig;
    PROPSHEETPAGE tPsp;
    Interface *pInterface;

    /*
    ** window private values
    */
    HWND hWndRealm;
    HWND hWndList;
    HWND hWndUser;
    HWND hWndPassword;
    HWND hWndConfirm;
    HWND hWndAdd;
    HWND hWndReplace;
    HWND hWndDelete;
    HWND hWndMethod;
    DblList lUsers;
    DblList lPasswords;

	HWND hWndDlgOK;
	HWND hWndDlgRealm;
	HWND hWndDlgBasic;
	HWND hWndDlgDigest;
	HWND hWndDlgNtlm;
	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "UsersPage: ";

        if ( 0 )
            {
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	UsersPage( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pConfig( FindParentQConfig( PIObject_getDB( pObject ) ) ),
        hWndRealm( 0 ),
        hWndList( 0 ),
        hWndUser( 0 ),
        hWndPassword( 0 ),
        hWndConfirm( 0 ),
        hWndAdd( 0 ),
        hWndReplace( 0 ),
        hWndDelete( 0 ),
        hWndMethod( 0 ),
        pInterface( 0 )
		{
        /*
        ** Read input parameters
        */
		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        /*
        ** Setup the PROPSHEETPAGE
        */
        memset( &tPsp, 0, sizeof( PROPSHEETPAGE ) );
        tPsp.dwSize = sizeof( PROPSHEETPAGE );
        tPsp.dwFlags = PSP_DEFAULT;
        tPsp.hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );

        /* Status page */
        tPsp.pszTemplate = C_RESOURCENAME;
        tPsp.pfnDlgProc = (DLGPROC)fnDialogProc;
        tPsp.lParam = (LPARAM)this;

        /*
        ** Set the data DB of this object
        */
        PIDB *pDB = PIObject_getDB( Object() );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_WINDOWTYPE,
            (void *)INTRFACE_PROPPAGE, 0 );
        assert(!PIDB_lookup( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE, 0));
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE,
            (void *)&tPsp, 0 );
        
        /*
        ** Done
        */
		};

	~UsersPage()
		{
		};

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                pInterface = &tWindow;
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return Win32Create( tWindow );

            case INTRFACE_MSG_LOAD:
                return PIAPI_ABORT;

            case INTRFACE_MSG_DEFAULTS:
                // temp. - LATER
                M_AddValue( pConfig, "Realms", "Basic", "Administration" );
                break;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

    int Win32Create( Interface &tWindow )
        {
        /* --- property sheet pages should not be created --- */
        return PIAPI_ABORT;
		};

    void Internal_addLine( const char *pLine )
	    {
		assert( pLine );
		assert( lUsers.Size()==lPasswords.Size() );
	
		int i;
		for( i=0; pLine[i] && !isspace(pLine[i]); i++ );
		
		/* --- scan to tab --- */
		lUsers.Append( (DblList::type)PI_NEW( PIString( pLine, i ) ) );
	
		/* --- scan over whitespace to start of Media type/subtype --- */
		for( ; pLine[i] && isspace(pLine[i]); i++ );
		lPasswords.Append( (DblList::type)PI_NEW( PIString( &(pLine[i]) ) ) );	
	
		assert( lUsers.Size()==lPasswords.Size() );
	    };
	
    void Internal_clearArrays()
	    {
		DblListIterator x( lUsers );
		DblListIterator y( lPasswords );
		for( ; !x.BadIndex(); x++, y++ )
			{
			assert( !x.BadIndex() && !y.BadIndex() );
			PI_DELETE( (PIString *)x.Current() );
			PI_DELETE( (PIString *)y.Current() );
			};
		lUsers.Clear();
		lPasswords.Clear();
	    };
	
    void Internal_reloadArrays()
	    {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		Internal_clearArrays();
	
		/*
		** Get number of items
		*/
		int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );
	
		/* 
		** Insert from listbox into internal arrays
		*/
		int i;
		for( i=0; i<iCount; i++ )
			{
			SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
			Internal_addLine( szBuf );
			};
	    };
	
    void Internal_updateButtons( WPARAM wParam, LPARAM lParam, HWND hWndEdit1, 
		HWND hWndEdit2, const char *pUser )
        {
		if ( GET_WM_COMMAND_CMD(wParam, lParam)!=EN_CHANGE )
			{ return; };
	
		/*
		** If the file extension changed then check if this is an existing
		** file extension
		*/
		int iExistingIndex = -1;
		assert( pUser );
		int iIndex = 0;
		if ( *pUser )
			{
			for( DblListIterator i( lUsers ); !i.BadIndex(); i++, iIndex++ )
				{
				PIString *pString = (PIString *)i.Current();
				assert( pString );
				if ( (*pString)==pUser )
					{
					iExistingIndex = iIndex;
					break;
					};
				};
			};
	
		/*
		** Remove listbox selection or scroll to correct line
		** as appropriate 
		*/
		SendMessage( hWndList, LB_SETCURSEL, iExistingIndex, 0 );
	
		/*
		** Set buttons
		*/
		if (	SendMessage( hWndEdit1, EM_LINELENGTH, 0, 0 )==0 ||
				SendMessage( hWndEdit2, EM_LINELENGTH, 0, 0 )==0 ||
				SendMessage( hWndConfirm, EM_LINELENGTH, 0, 0 )==0 )
			{
			if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
			if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
			}
		else
			{
			if ( iExistingIndex>-1 )	/* --- disable add, enable replace --- */
				{
				if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
				if ( !IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, TRUE );
				}
			else						/* --- disable replace, enable add --- */
				{
				if ( !IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, TRUE );
				if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
				};
			};
	
		/*
		** If there is a selected item in the listbox then
		** enable delete, otherwise disable delete	
		*/
		if ( SendMessage( hWndList, LB_GETCURSEL, 0, 0 )==LB_ERR )
			{ EnableWindow( hWndDelete, FALSE ); }
		else
			{ EnableWindow( hWndDelete, TRUE ); };
	    };
	
    void Internal_loadItems( const char *pRealm )
	    {
		assert( pRealm );
	
		/*
		** Clear any existing items out of listbox
		*/
		SendMessage( hWndList, LB_RESETCONTENT, 0, 0 );
	
		/*
		** Load all users into list box
		*/
		int iTmp = 0;
		for(;; iTmp++ )
			{
			PIString sSection( "__" );
			sSection.Concatenate( pRealm );
			const char *pTmp = M_LookupValueEx( pConfig, sSection, "Base64", iTmp );
			if ( !pTmp )
				{ break; };
	
			PIString sPlain;
			Base64Decode( pTmp, sPlain );
			char *pPlain = (char *)(const char *)sPlain;	
			int iLen = sPlain.Len();
			int i;
			for( i=0; i<iLen && pPlain[i]!=':'; i++);
			if ( pPlain[i]==':' ) { pPlain[i]='\t'; };
	
			SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pPlain );
			};

		for(iTmp = 0;; iTmp++ )
			{
			PIString sSection( "__" );
			sSection.Concatenate( pRealm );
			PIString sPlain = M_LookupValueEx( pConfig, sSection, "Digest", iTmp );
			int iLen = sPlain.Len();
			if ( !iLen )
				{ break; };

			char *pPlain = (char *)(const char *)sPlain;
			SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pPlain );
			};
	
		/*
		** Clear out edit fields
		*/
		::SendMessage( hWndUser, WM_SETTEXT, 0, (LPARAM)"" );
		::SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
		::SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
	
		Internal_reloadArrays();
	    };
	
    int Internal_canDeleteRealm( HWND hDlg, const char *pRealm )
        {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
		int iTmp = 0;
	
		assert( pRealm );
	
		/*
		** Check if realm is used by users
		*/
		iTmp = 0;
		for(;; iTmp++ )
			{
			const char *pTmp = M_LookupValueEx( pConfig, "AccessControl",
				"URLs", iTmp );
	
			if ( !pTmp ) { break; };
	
			int i;
			for( i=0; pTmp[i] && pTmp[i]!='\t'; i++ );
	
			PIString sRealm( pTmp, i );
			
			if ( sRealm==pRealm )
				{
				sprintf( szBuf, "Realm '%s' is being used for access control. \
Remove all Realm-URL Path mappings in 'Access Control' before removing \
this realm.", pRealm );
				MessageBox( hDlg, szBuf, "Remove Realm", MB_OK | MB_ICONEXCLAMATION );
				return 0;
				};
			};
	
		/*
		** Check if realm is used by a mapping
		*/
		iTmp = 0;
		for(;; iTmp++ )
			{
			const char *pTmp = M_LookupValueEx( pConfig, "Users", 
				"Mapping", iTmp );
	
			if ( !pTmp ) { break; };
	
			int i=0;
			for( ; pTmp[i] && isspace(pTmp[i]); i++ );	
			for( ; pTmp[i] && pTmp[i]!='\t'; i++ );
			for( ; pTmp[i] && isspace(pTmp[i]); i++ );
			for( ; pTmp[i] && pTmp[i]!='\t'; i++ );
			for( ; pTmp[i] && isspace(pTmp[i]); i++ );
			for( ; pTmp[i] && pTmp[i]!='\t'; i++ );
			for( ; pTmp[i] && isspace(pTmp[i]); i++ );	
			pTmp = &( pTmp[i] );
			for( ; pTmp[i] && pTmp[i]!='\t'; i++ );	/* read length of realm */
	
			PIString sRealm( pTmp, i );
			
			if ( sRealm==pRealm )
				{
				sprintf( szBuf, "Realm '%s' is being used for directory mapping. \
Remove all Realm-URL Path mappings in 'Mapping' before removing \
this realm.", pRealm );
				MessageBox( hDlg, szBuf, "Remove Realm", MB_OK | MB_ICONEXCLAMATION );
				return 0;
				};
			};
	
		return 1;
        };
	
	int Internal_findMethod( char *pName )
		{
		for(int iTmp = 0;; iTmp++ ) {
			const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Basic", 
				iTmp );
			if ( !pTmp ) { break; };

			if (!strcmp(pTmp,pName)) {
				strcpy(pName, "Basic");
				return 1;
			}
		}
		for(iTmp=0;; iTmp++ ) {
			const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Digest", 
				iTmp );
			if ( !pTmp ) { break; };

			if (!strcmp(pTmp,pName)) {
				strcpy(pName, "Digest");
				return 1;
			}
		}
		for(iTmp=0;; iTmp++ ) {
			const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Ntlm", 
				iTmp );
			if ( !pTmp ) { break; };

			if (!strcmp(pTmp,pName)) {
			strcpy(pName, "Ntlm");
				return 1;
			}
		}
		return 0;
	}

    void Internal_saveItems( const char *pRealm )
	    {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		assert( pRealm );
	
		int i;
		int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

		/*
		** Save all user types
		*/

		/*
		** No users for Ntlm authentication
		*/
		strcpy(szBuf, pRealm);
		if ( Internal_findMethod( szBuf ) && strcmp(szBuf,"Ntlm"))
			{
			/*
			** Basic authentication
			*/
			if (!strcmp(szBuf,"Basic"))
				{
				/* --- remove old ones first --- */
				PIString sSection( "__" );
				sSection.Concatenate( pRealm );
				M_DeleteValue( pConfig, sSection, "Base64" );
				for( i=0; i<iCount; i++ )
					{
					*szBuf = '\0';
					szBuf[BUF_SIZE] = '\0';
					SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );

					int iLen = strlen( szBuf );
					int j;
					for( j=0; j<iLen && szBuf[j]!='\t'; j++ );
					if ( szBuf[j]=='\t' ) { szBuf[j] = ':'; };
					PIString sBase64;
					Base64Encode( szBuf, sBase64 );
		
					M_AddValue( pConfig, sSection, "Base64", sBase64 );
					};
				}
			/*
			** Digest authentication
			*/
			else
				{
				/* --- remove old ones first --- */
				PIString sSection( "__" );
				sSection.Concatenate( pRealm );
				M_DeleteValue( pConfig, sSection, "Digest" );
				for( i=0; i<iCount; i++ )
					{
					*szBuf = '\0';
					szBuf[BUF_SIZE] = '\0';
					SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
					M_AddValue( pConfig, sSection, "Digest", szBuf );
					};
				}
			}
        };
	
    BOOL CALLBACK NewRealmDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
		LPARAM lParam)
	    {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
	
		switch (uMsg)
			{
			case WM_INITDIALOG:
                hWndDlgOK = GetDlgItem( hDlg, IDOK );
                hWndDlgRealm = GetDlgItem( hDlg, IDC_REALM );
				hWndDlgBasic = GetDlgItem( hDlg, IDC_BASIC );
				hWndDlgDigest = GetDlgItem( hDlg, IDC_DIGEST );
				hWndDlgNtlm = GetDlgItem( hDlg, IDC_NTLM );

				assert( hWndDlgOK && hWndDlgRealm && hWndDlgBasic
					&& hWndDlgDigest && hWndDlgNtlm);
				SendMessage( hWndDlgRealm, EM_SETLIMITTEXT, 256, 0 );
				EnableWindow( hWndDlgOK, FALSE );
				SendMessage( hWndDlgBasic, BM_SETCHECK, BST_CHECKED, 0 );

				/*
				** Enable NTLM authentication only for NT systems
				*/
				EnableWindow( hWndDlgNtlm,
					!strcmp(M_LookupValue(pConfig,"General","OS"),"NT" ));
				break;
	
			case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDOK:
						{
						/*
						** Validate content of edit field
						*/
						*szBuf = '\0';
						SendMessage( hWndDlgRealm, WM_GETTEXT, BUF_SIZE, 
							(LPARAM)szBuf );
						PIString sRealm( szBuf );
						int i;
						int iCount = SendMessage( hWndRealm, CB_GETCOUNT, 0, 0 );
						for( i=0; i<iCount; i++ )
							{
							*szBuf = '\0';
							SendMessage( hWndRealm, CB_GETLBTEXT, i,
                                (LPARAM)szBuf );
							if ( sRealm == szBuf )
								{
								sprintf( szBuf, "Realm '%s' already exists.",
                                    (const char *)sRealm );
								MessageBox( hDlg, szBuf, "New Realm",
									MB_OK | MB_ICONEXCLAMATION );
								return (TRUE);
								};
							};
	
						/*
						** Can't use '(none)'
						*/
						if ( !stricmp( sRealm, "(none)" ) )
							{
							sprintf( szBuf, "'%s' is a reserved realm.",
								(const char *)sRealm );
							::MessageBox( hDlg, szBuf,  "Remove Realm",
								MB_OK | MB_ICONEXCLAMATION );
							::SetFocus( hWndDlgRealm );
							::SendMessage( hWndDlgRealm, EM_SETSEL, 0, -1 );
							return (TRUE);
							};
	
						/*
						** Add new realm
						*/
						SendMessage( hWndRealm, CB_ADDSTRING, 0, (LPARAM)(const char *)sRealm );
						*szBuf = '\0';
					
						if (SendMessage( hWndDlgBasic, BM_GETCHECK, 0, 0 ) == BST_CHECKED ) {
							strcpy(szBuf, "Basic");
						};
						if (SendMessage( hWndDlgDigest, BM_GETCHECK, 0, 0 ) == BST_CHECKED ) {
							strcpy(szBuf, "Digest");
						};
						if (SendMessage( hWndDlgNtlm, BM_GETCHECK, 0, 0 ) == BST_CHECKED ) {
							strcpy(szBuf, "Ntlm");
						};

						M_AddValue( pConfig, "Realms", szBuf, (const char *)sRealm );
						SendMessage( hWndMethod, WM_SETTEXT, 0,	(LPARAM)szBuf );

						/*
						** Save old items
						*/
						*szBuf = '\0';
						SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						Internal_saveItems( szBuf );
	
						int iSel = SendMessage( hWndRealm, CB_FINDSTRINGEXACT,
                            (WPARAM)-1, 
							(LPARAM)(const char *)sRealm );
						if ( iSel!=CB_ERR )
							{
							SendMessage( hWndRealm, CB_SETCURSEL, iSel, 0 ); 
							if ( SendMessage( hWndRealm, CB_GETCURSEL, 0, 0 )!=
								CB_ERR )
								{
								EnableWindow( hWndUser, TRUE );
								EnableWindow( hWndPassword, TRUE );
								EnableWindow( hWndConfirm, TRUE );
								};
							Internal_loadItems( sRealm );
							}		
						else
							{ assert( 0 ); };
	
						};
						EndDialog( hDlg, 1 );
						break;
	
					case IDCANCEL:
						EndDialog( hDlg, 0 );
						break;
	
					case IDC_REALM:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
							{
							if ( SendMessage( GET_WM_COMMAND_HWND(wParam,
                                lParam ), EM_LINELENGTH, 0, 0 )>0 )
								{ EnableWindow( hWndDlgOK, TRUE ); }
							else
								{ EnableWindow( hWndDlgOK, FALSE ); }
							}
						else
							return (FALSE);
						break;
	
					default:;
					};
	
			default:
				return (FALSE);
			};
	
		return (TRUE);
	    };
	
    BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
		LPARAM lParam)
        {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
		szBuf[BUF_SIZE]='\0';
		int aTabstop[2];
	
		switch (uMsg)
			{
			case WM_INITDIALOG:
				HourglassOn();
				{
				hWndRealm = GetDlgItem( hDlg, IDC_REALM );
				hWndList = GetDlgItem( hDlg, IDC_LIST );
				hWndUser = GetDlgItem( hDlg, IDC_USER );
				hWndPassword = GetDlgItem( hDlg, IDC_PASSWORD );
				hWndConfirm = GetDlgItem( hDlg, IDC_CONFIRM );
				hWndAdd = GetDlgItem( hDlg, IDC_ADD );
				hWndReplace = GetDlgItem( hDlg, IDC_REPLACE );
				hWndDelete= GetDlgItem( hDlg, IDC_DELETE );
				hWndMethod = GetDlgItem( hDlg, IDC_AUTHMETHOD );
			
				assert( hWndRealm && hWndList && hWndUser && hWndPassword );
				assert( hWndConfirm && hWndAdd && hWndReplace && hWndDelete );
	
				EnableWindow( hWndAdd, FALSE );
				EnableWindow( hWndReplace, FALSE );
				EnableWindow( hWndDelete, FALSE );
	
				/*
				** Limit edit field lengths, really just a sanity thing
				*/
				SendMessage( hWndUser, EM_SETLIMITTEXT, 256, 0 );
				SendMessage( hWndPassword, EM_SETLIMITTEXT, 256, 0 );
				SendMessage( hWndConfirm, EM_SETLIMITTEXT, 256, 0 );
	
				/*
				** Setup tabstops
				*/
				aTabstop[0] = 0;
				aTabstop[1] = 750;
				SendMessage( hWndList, LB_SETTABSTOPS, 2, (LPARAM)&aTabstop );
	
				/*
				** Load all realms into realm box
				*/
				for(int iTmp = 0;; iTmp++ ) {
					const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Basic", iTmp );
					if ( !pTmp ) { break; };
					SendMessage( hWndRealm, CB_ADDSTRING, 0, (LPARAM)pTmp );
				};
				for(iTmp = 0;; iTmp++ ) {
					const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Digest", iTmp );
					if ( !pTmp ) { break; };
					SendMessage( hWndRealm, CB_ADDSTRING, 0, (LPARAM)pTmp );
				};
				for(iTmp = 0;; iTmp++ ) {
					const char *pTmp = M_LookupValueEx( pConfig, "Realms", "Ntlm", iTmp );
					if ( !pTmp ) { break; };
					SendMessage( hWndRealm, CB_ADDSTRING, 0, (LPARAM)pTmp );
				};
	
				/*
				** Load all users into realm box
				*/
				if ( SendMessage( hWndRealm, CB_GETCOUNT, 0, 0 )>0 )
					{
					SendMessage( hWndRealm, CB_SETCURSEL, 0, 0 );
					*szBuf = '\0';
					SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE,
                        (LPARAM)szBuf );
	
					/*
					** Load items into list box
					*/
					Internal_loadItems( szBuf );

					/*
					** update static method text for realm
					*/
					if ( Internal_findMethod( szBuf ))
						{
						SendMessage( hWndMethod, WM_SETTEXT, 0,
							(LPARAM)szBuf );
						if (!strcmp(szBuf,"Ntlm"))
							{
							EnableWindow( hWndUser, FALSE);
							EnableWindow( hWndPassword, FALSE );
							EnableWindow( hWndConfirm, FALSE );
							}
						}
					}
				else
					{
					/* --- no realms, disable edit fields --- */
					EnableWindow( hWndUser, FALSE );
					EnableWindow( hWndPassword, FALSE );
					EnableWindow( hWndConfirm, FALSE );
					};
				}
				HourglassOff();
				break;
	
			case WM_DESTROY:
				{ Internal_clearArrays(); };
				break;
	
			case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDC_REALM:
						switch( GET_WM_COMMAND_CMD(wParam, lParam) )
							{
							case CBN_DROPDOWN:
								/*
								** Save current realm users
								*/
								*szBuf = '\0';
								SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE, 
									(LPARAM)szBuf );
								Internal_saveItems( szBuf );
								break;
	
							case CBN_SELCHANGE:
								/*
								** Load new realm users
								*/
								*szBuf = '\0';
								SendMessage( hWndRealm, WM_GETTEXT, 
									BUF_SIZE, (LPARAM)szBuf );
								if ( SendMessage( hWndRealm, CB_GETCURSEL, 0,
                                    0 )!= CB_ERR )
									{
									EnableWindow( hWndUser, TRUE );
									EnableWindow( hWndPassword, TRUE );
									EnableWindow( hWndConfirm, TRUE );
									};
								Internal_loadItems( szBuf );

								/*
								** update static method text for realm
								*/
								if ( Internal_findMethod( szBuf ))
									{
									SendMessage( hWndMethod, WM_SETTEXT, 0,
										(LPARAM)szBuf );
									if (!strcmp(szBuf,"Ntlm"))
										{
										EnableWindow( hWndUser, FALSE);
										EnableWindow( hWndPassword, FALSE );
										EnableWindow( hWndConfirm, FALSE );
										}
									}
								break;
	
							default:;
							};
						break;
	
					case IDC_NEW:
						if ( DialogBoxParam(
                            tPsp.hInstance,
							MAKEINTRESOURCE( IDD_NEWREALM ),
							hDlg,
							(DLGPROC)fnNewRealmDialogProc,
                            (LPARAM)this
							) )
							{ Changed( hDlg ); };
						break;
	
	
					case IDC_REMOVE:
						{
						int iSel = SendMessage( hWndRealm, CB_GETCURSEL, 0, 0 );
						if ( iSel==CB_ERR )
							{ break; };
	
						SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE,   
                            (LPARAM)szBuf );
			
						if ( !Internal_canDeleteRealm( hDlg, szBuf ) )
							{ break; };
	
						SendMessage( hWndRealm, CB_DELETESTRING, iSel, 0 );
						int iCount = SendMessage( hWndRealm, CB_GETCOUNT, 0, 0);
						if (iSel >= iCount) {
							SendMessage( hWndRealm, CB_SETCURSEL, iCount-1, 0 );
						} else {
							SendMessage( hWndRealm, CB_SETCURSEL, iSel, 0 );
						};

						PIString sSection( "__" );
						sSection.Concatenate( szBuf );
						M_DeleteValue( pConfig, sSection, "Base64" );
						Internal_clearArrays();
						SendMessage( hWndList, LB_RESETCONTENT, 0, 0 );
						SendMessage( hWndUser, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
						Changed( hDlg );

						/*
						** remove realm from configuration
						*/
						PIString sRealm = szBuf;
						PIString sTmp = "";
						*szBuf = '\0';
						SendMessage( hWndMethod, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						int iTmp = 0;
						for(;; iTmp++ ) {
							const char *pTmp = M_LookupValueEx( pConfig, "Realms", szBuf, iTmp );
							if ( !pTmp ) { break; };
							if (!strcmp(pTmp,sRealm)) { continue; };
							if (sTmp.Len()) sTmp.Concatenate(":");
							sTmp.Concatenate(pTmp);
						};
						M_DeleteValue( pConfig, "Realms", szBuf );
						if (sTmp.Len()) {
							StringTokenizer tTokens(sTmp,":");
							for(iTmp=0;iTmp<tTokens.NumTokens();iTmp++) {
								M_AddValue(pConfig,"Realms",szBuf,tTokens.GetToken(iTmp));
							}
						}

						if ( SendMessage( hWndRealm, CB_GETCURSEL, 0, 0 )==CB_ERR )
							{
							SendMessage( hWndRealm, CB_SETCURSEL, (WPARAM)iSel, 0 );
							SendMessage( hWndMethod, WM_SETTEXT, 0, (LPARAM)"" );
							EnableWindow( hWndUser, FALSE );
							EnableWindow( hWndPassword, FALSE );
							EnableWindow( hWndConfirm, FALSE );
							};
						};

						/*
						** update static method text for realm
						*/
						*szBuf = '\0';
						SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						if ( Internal_findMethod( szBuf )) {
							SendMessage( hWndMethod, WM_SETTEXT, 0, (LPARAM)szBuf );
							if (!strcmp(szBuf,"Ntlm")) {
								EnableWindow( hWndUser, FALSE);
								EnableWindow( hWndPassword, FALSE );
								EnableWindow( hWndConfirm, FALSE );
							}
						}
						break;
	
					case IDC_ADD:
						{
						int i;
						*szBuf = '\0';
						SendMessage( hWndUser, WM_GETTEXT, BUF_SIZE,
                            (LPARAM)szBuf );

						/*
						** Username cannot contain ':' character
						*/
						for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!=':';i++);
						if ( szBuf[i]==':' ) 
							{
							MessageBox( hDlg, "A username may not contain the \
colon (:) character.", "Add User", 
								MB_OK | MB_ICONEXCLAMATION );
							SetFocus( hWndUser );
							SendMessage( hWndUser, EM_SETSEL, 0, -1 );
							break;
							};
						for( i=0; i<BUF_SIZE && szBuf[i] &&
                            !isspace( szBuf[i] ); i++ );
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						PIString sUser( szBuf );
						szBuf[i++] = '\t';
						SendMessage( hWndPassword, WM_GETTEXT, BUF_SIZE-i,
                            (LPARAM)&(szBuf[i]) );
						PIString sPassword( &(szBuf[i]) );
						SendMessage( hWndConfirm, WM_GETTEXT, BUF_SIZE-i,
                            (LPARAM)&(szBuf[i]) );
						PIString sConfirm( &(szBuf[i]) );
						if ( !(sPassword==sConfirm) )
							{
							MessageBox( hDlg, "Passwords do not match",
                                "New User",  MB_OK | MB_ICONEXCLAMATION );
							SetFocus( hWndPassword );
							SendMessage( hWndPassword, EM_SETSEL, 0, -1 );
							break;
							};

						/*
						** Compute digest password if appropriate
						*/
						*szBuf = '\0';
						SendMessage( hWndMethod, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						if (!strcmp(szBuf,"Digest"))
							{
							*szBuf = '\0';
							SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
							sConfirm = sUser;
							sConfirm.Concatenate(':');
							sConfirm.Concatenate(szBuf);
							sConfirm.Concatenate(':');
							sConfirm.Concatenate(sPassword);
							sPassword = "";
							MD5Encode((const char *)sConfirm, sPassword);
							}
						strcpy(szBuf,sUser);
						strcat(szBuf,"\t");
						strcpy(&szBuf[i],sPassword);

						lUsers.Append( (DblList::type)PI_NEW( PIString( sUser ) ) );
						lPasswords.Append( (DblList::type)PI_NEW( PIString ( sPassword ) ) );
						SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );
	
						/*
						** Clear edit fields, focus to list
						*/
						SendMessage( hWndUser, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
						Internal_reloadArrays();
						int iIndex = SendMessage( hWndList,
                            LB_FINDSTRINGEXACT, 0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR ) SendMessage( hWndList,
                            LB_SETTOPINDEX, iIndex, 0 );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;
		
					case IDC_REPLACE:
						{
						/*
						** Get selected item index 
						*/
						int iCurSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
	
						/*
						** Get text to add and add it to internal lists only
						*/
						int i;
						*szBuf = '\0';
						SendMessage( hWndUser, WM_GETTEXT, BUF_SIZE,
                            (LPARAM)szBuf );

						/*
						** Username cannot contain ':' character
						*/
						for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!=':';
                            i++ );
						if ( szBuf[i]==':' ) 
							{
							MessageBox( hDlg, "A username may not contain \
the colon (:) character.", "Add User", 
								MB_OK | MB_ICONEXCLAMATION );
							SetFocus( hWndUser );
							SendMessage( hWndUser, EM_SETSEL, 0, -1 );
							break;
							};
						for( i=0; i<BUF_SIZE && szBuf[i] &&
                            !isspace( szBuf[i] ); i++ );
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						PIString sUser( szBuf );
						szBuf[i++] = '\t';
						SendMessage( hWndPassword, WM_GETTEXT, BUF_SIZE-i,
                            (LPARAM)&(szBuf[i]) );
						PIString sPassword( &(szBuf[i]) );
						SendMessage( hWndConfirm, WM_GETTEXT, BUF_SIZE-i,
                            (LPARAM)&(szBuf[i]) );
						PIString sConfirm( &(szBuf[i]) );
						if ( !(sPassword==sConfirm) )
							{
							MessageBox( hDlg, "Passwords do not match",
                                "New User",  MB_OK | MB_ICONEXCLAMATION );
							SetFocus( hWndPassword );
							SendMessage( hWndPassword, EM_SETSEL, 0, -1 );
							break;
							};
					
						/*
						** Compute digest password if appropriate
						*/
						*szBuf = '\0';
						SendMessage( hWndMethod, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						if (!strcmp(szBuf,"Digest"))
							{
							*szBuf = '\0';
							SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
							sConfirm = sUser;
							sConfirm.Concatenate(':');
							sConfirm.Concatenate(szBuf);
							sConfirm.Concatenate(':');
							sConfirm.Concatenate(sPassword);
							MD5Encode((const char *)sConfirm, sPassword);
							}
						strcpy(szBuf,sUser);
						strcat(szBuf,"\t");
						strcpy(&szBuf[i],sPassword);

						lUsers.Append( (DblList::type)PI_NEW(
                            PIString( sUser ) ) );
						lPasswords.Append( (DblList::type)PI_NEW(
                            PIString ( sPassword ) ) );
	
						/*
						** Delete the item from the list and internal lists
						*/
						SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
						PI_DELETE( (PIString *)lUsers.Detach( iCurSel) );
						PI_DELETE( (PIString *)lPasswords.Detach( iCurSel) );
						
						/*
						** Add new item to listbox
						*/
						SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );
	
						/*
						** Clear edit fields, focus to list
						*/
						SendMessage( hWndUser, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
						Internal_reloadArrays();
						int iIndex = SendMessage( hWndList,
                            LB_FINDSTRINGEXACT, 0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR ) SendMessage( hWndList,
                            LB_SETTOPINDEX, iIndex, 0 );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;
	
					case IDC_DELETE:
						{
						int iCurSel = SendMessage( hWndList, LB_GETCURSEL,
                            0, 0 );
						SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
						PI_DELETE( (PIString *)lUsers.Detach( iCurSel) );
						PI_DELETE( (PIString *)lPasswords.Detach( iCurSel) );
				
						/*
						** Clear edit fields, focus to list
						*/
						SendMessage( hWndUser, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;
	
					case IDC_USER:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE ) {
							*szBuf = '\0';
							SendMessage( hWndUser, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf ); 
							Internal_updateButtons( wParam, lParam, hWndUser, 
								hWndPassword, szBuf );

							/*
							** If user exist, fill password field, otherwise delete,
							*/
							*szBuf = '\0';
							SendMessage( hWndMethod, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
							int iSel;
							iSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
							if (iSel != LB_ERR && !strcmp(szBuf,"Basic")) {
								SendMessage( hWndPassword, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)lPasswords[iSel]) );
								SendMessage( hWndConfirm, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)lPasswords[iSel]) );
							} else {
								SendMessage( hWndPassword, WM_SETTEXT, 0, (LPARAM)"" );
								SendMessage( hWndConfirm, WM_SETTEXT, 0, (LPARAM)"" );
							}
						}
						break;
	
					case IDC_PASSWORD:
					case IDC_CONFIRM:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE ) {
							*szBuf = '\0';
							SendMessage( hWndUser, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
							Internal_updateButtons( wParam, lParam, hWndPassword, 
								hWndUser, szBuf );
						}
						break;
	
					case IDC_LIST:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==LBN_SELCHANGE )
							{
							*szBuf = '\0';
							SendMessage( hWndMethod, WM_GETTEXT, BUF_SIZE,(LPARAM)szBuf );
							int iSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
							if (iSel != LB_ERR)
								{
								assert( iSel>=0 && iSel<lUsers.Size() );
								EnableWindow( hWndDelete, TRUE );
								SendMessage( hWndUser, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)lUsers[iSel]) );
								 if (!strcmp(szBuf,"Basic"))
									{
									SendMessage( hWndPassword, WM_SETTEXT, 0,
										(LPARAM)(const char *)*((PIString *)lPasswords[iSel]) );
									SendMessage( hWndConfirm, WM_SETTEXT, 0,
										(LPARAM)(const char *)*((PIString *)lPasswords[iSel]) );
									};
								};
							};
						break;
	
					default:;		
					}
				break;
	
			case WM_NOTIFY:
				switch( ((NMHDR *)lParam)->code )
					{
					case PSN_QUERYCANCEL:
						return SHEET->Cancel(pInterface, hDlg);
	
					case PSN_KILLACTIVE:
						/*
						** Save values
						*/
						*szBuf = '\0';
						SendMessage( hWndRealm, WM_GETTEXT, BUF_SIZE,
                            (LPARAM)szBuf );
						Internal_saveItems( szBuf );
						SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
						break;
	
					case PSN_APPLY:
						return SHEET->SaveConfig(pInterface, hDlg);

					default:
						return (FALSE);
					};
				break;
	
			default:
				return (FALSE);
			};
	
		return (TRUE);
	    };
	
};
	
/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, ((LPPROPSHEETPAGE)lParam)->lParam );
        };

    UsersPage *pPage = (UsersPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->DialogProc( hDlg, uMsg, wParam, lParam ) : FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnNewRealmDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, lParam );
        };

    UsersPage *pPage = (UsersPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->NewRealmDialogProc( hDlg, uMsg, wParam, lParam ) :
        FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UsersPage_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( UsersPage( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name UsersPageClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor UsersPage_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name UsersPage
		Class UsersPageClass
	</Object>

/*___+++CNF_END+++___*/
#endif

