/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Intrface/VHosts.cpp,v $
 * $Date: 2002/08/02 06:58:56 $
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <iostream.h>
#include <stdio.h>
#include <windowsx.h>

#include "IFaceBse.h"
#include "PIStrStr.h"
#include "StrToken.h"

#include "QConfig.h"
#include "VHosts.hrc"

/*
** #define D { cerr << __FILE__ << ": " << __LINE__ << endl; }
*/
#define D

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define C_IPRESOURCENAME                "IPVHostsPage"
#define C_NONIPRESOURCENAME             "NonIPVHostsPage"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define SECTION         "VirtualHosts"
#define IPVARIABLE      "IP"
#define NONIPVARIABLE   "NonIP"
#define KEY_CONF_FLAG   "Flag"
#define FLAG_IP         "IP"
#define FLAG_NONIP      "NonIP"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
    VHostsPage

Description:
    Creates the eponymous property page and is used by the PropSheet object.

Example:
	<PRE>
	&lt;Object&gt;
		Name VHostsPage
		Class VHostsPageClass
		PropertyPage VHostsPage
	&lt;/Object&gt;

	</PRE>

/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class VHostsPage : public IFaceBase
{
private:
    QConfig *pConfig;
    PROPSHEETPAGE tPsp;
    Interface *pInterface;

    /*
    ** window private values
    */
    HWND hWndList;
    HWND hWndIPAddress;
    HWND hWndHostname;
    HWND hWndDocumentRoot;
    HWND hWndAdd;
    HWND hWndReplace;
    HWND hWndDelete;
    HWND hWndAdministrator;
    DblList lIPs;
    DblList lHostnames;
    DblList lDocumentRoots;
    DblList lAdministrators;

    /*
    ** Difference between IP and Non-IP virtual hosts
    */
    int iIsIP;
    PIString sVariable;
    PIString sResourceName;

	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable && pValue );

		PIOStrStream os;

		os << pWhere << "VHostsPage: ";

        if ( !PIUtil_stricmp( KEY_CONF_FLAG, pVariable ) )
            {
            if ( !PIUtil_stricmp( FLAG_IP, pValue ) )
                {
                iIsIP = 1;
                sVariable = IPVARIABLE;
                sResourceName = C_IPRESOURCENAME;
                }
            else if ( !PIUtil_stricmp( FLAG_NONIP, pValue ) )
                {
                iIsIP = 0;
                sVariable = NONIPVARIABLE;
                sResourceName = C_NONIPRESOURCENAME;
                }
            else
                {
			    os << "Unknown directive '" << pVariable <<
				    "'" << ends;
			    CONFIG_ERR( Object(), os.str() );
			    return 0;
                };
            }
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1;
		};

public:
	VHostsPage( PIObject *pObject, int iArgc, const char *ppArgv[] )
	:	IFaceBase( pObject ),
        pConfig( FindParentQConfig( PIObject_getDB( pObject ) ) ),
        hWndList( 0 ),
        hWndIPAddress( 0 ),
        hWndHostname( 0 ),
        hWndDocumentRoot( 0 ),
        hWndAdd( 0 ),
        hWndReplace( 0 ),
        hWndDelete( 0 ),
        hWndAdministrator( 0 ),
        pInterface( 0 ),
        iIsIP( -1 )
		{
        /*
        ** Read input parameters
        */
		ReadParameters( iArgc, ppArgv );

        if ( !IsOK() )
            { return; };

        if ( iIsIP==-1 )
            {
            CONFIG_ERR( Object(), "VHostsPage: One of the values 'IP' or \
'NonIP' must be specified for 'Flag'." );
            SetOK( 0 );
            return;
            };

        /*
        ** Setup the PROPSHEETPAGE
        */
        memset( &tPsp, 0, sizeof( PROPSHEETPAGE ) );
        tPsp.dwSize = sizeof( PROPSHEETPAGE );
        tPsp.dwFlags = PSP_DEFAULT;
        tPsp.hInstance = (HINSTANCE)PIDLL_getSystemHandle( PIClass_getLibrary(
            PIObject_getClass( Object() ) ) );

        /* Status page */
        tPsp.pszTemplate = sResourceName;
        tPsp.pfnDlgProc = (DLGPROC)fnDialogProc;
        tPsp.lParam = (LPARAM)this;

        /*
        ** Set the data DB of this object
        */
        PIDB *pDB = PIObject_getDB( Object() );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_WINDOWTYPE,
            (void *)INTRFACE_PROPPAGE, 0 );
        PIDB_add( pDB, PIDBTYPE_OPAQUE, INTRFACE_PROPSHEETPAGE,
            (void *)&tPsp, 0 );
        
        /*
        ** Done
        */
		};

	~VHostsPage()
		{
		};

    virtual int Handle( Interface &tWindow )
        {
        switch( tWindow.iMessage )
            {
            case INTRFACE_MSG_INIT:
                pInterface = &tWindow;
                break;

            case INTRFACE_MSG_WIN32CREATE:
                return Win32Create( tWindow );

            case INTRFACE_MSG_LOAD:
                return PIAPI_ABORT;

            case INTRFACE_MSG_DEFAULTS:
                break;

            default:
                assert( 0 );
                return PIAPI_ABORT;
            };

        return PIAPI_COMPLETED;
        };

    int Win32Create( Interface &tWindow )
        {
        /* --- property sheet pages should not be created --- */
        return PIAPI_ABORT;
		};

    int Internal_verifyFields( HWND hDlg, char *pszBuf, int iBufSize )
        {
	    int iToLen;
	
	    /*
	    ** Get the 'DocumentRoot' field
	    */
	    SendMessage( hWndDocumentRoot, WM_GETTEXT, iBufSize, (LPARAM)pszBuf );
	
	    /*
	    ** 'DocumentRoot' field must end with '/'
	    */
	    assert( *pszBuf );
	    iToLen = strlen(pszBuf);
	    if ( pszBuf[iToLen-1]!='/' && pszBuf[iToLen-1]!='\\' )
	        {
	        MessageBox( hDlg, "'DocumentRoot' field must end with a \
forward-slash ('/') or a backslash ('\\').",
	            "Error in 'DocumentRoot' Field", MB_OK | MB_ICONEXCLAMATION );
	
	        ::SetFocus( hWndDocumentRoot );
	        ::SendMessage( hWndDocumentRoot, EM_SETSEL, (WPARAM)(INT)0,
                (LPARAM)(INT)-1 );
	
	        goto verify_fail;
	        };
	
	    /*
	    ** Good
	    */
	    return 1;
	
verify_fail:
	    return 0;
        };
	
    void Internal_addLine( const char *pLine )
        {
		assert( pLine );
        if ( iIsIP )
            {
		    assert( lIPs.Size()==lHostnames.Size() );
            };
		assert( lHostnames.Size()==lDocumentRoots.Size() );
		assert( lDocumentRoots.Size()==lAdministrators.Size() );
	
		int i, j;
	    i = 0;
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );
		
        if ( iIsIP )
            {
            lIPs.Append( (DblList::type)PI_NEW( PIString( &(pLine[j]), i-j)));
	
		    /* --- scan over whitespace to start of second field --- */
		    for( ; pLine[i] && isspace(pLine[i]); i++ );
	        j = i;
		    for( ; pLine[i] && !isspace(pLine[i]); i++ );
            };

		lHostnames.Append((DblList::type)PI_NEW( PIString( &(pLine[j]), i-j)));	
	
		/* --- scan over whitespace to start of second field --- */
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );
		lDocumentRoots.Append((DblList::type)PI_NEW( PIString( &(pLine[j]),
            i-j)));	
	
		/* --- scan over whitespace to start of third field --- */
		for( ; pLine[i] && isspace(pLine[i]); i++ );
	    j = i;
		for( ; pLine[i] && !isspace(pLine[i]); i++ );
		lAdministrators.Append((DblList::type)PI_NEW( PIString( &(pLine[j]),
            i-j)));	
	
        if ( iIsIP )
            {
		    assert( lIPs.Size()==lHostnames.Size() );
            };
		assert( lHostnames.Size()==lDocumentRoots.Size() );
		assert( lDocumentRoots.Size()==lAdministrators.Size() );
        };
	
    void Internal_clearArrays()
        {
		DblListIterator w( lIPs );
		DblListIterator x( lHostnames );
		DblListIterator y( lDocumentRoots );
		DblListIterator z( lAdministrators );
		for( ; !x.BadIndex(); )
			{
            if ( iIsIP ) { assert( !w.BadIndex() ); };
			assert( !x.BadIndex() && !y.BadIndex() && !z.BadIndex() );
            if ( iIsIP ) { PI_DELETE( (PIString *)w.Current() ); };
			PI_DELETE( (PIString *)x.Current() );
			PI_DELETE( (PIString *)y.Current() );
			PI_DELETE( (PIString *)z.Current() );

            if ( iIsIP ) { w++; };
            x++;
            y++;
            z++;
			};
		lIPs.Clear();
		lHostnames.Clear();
		lDocumentRoots.Clear();
		lAdministrators.Clear();
        };
	
    void Internal_reloadArrays()
        {
		enum { BUF_SIZE=2047 };
		char szBuf[BUF_SIZE+1];
	
		Internal_clearArrays();
	
		/*
		** Get number of items
		*/
		int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );
	
		/* 
		** Insert from listbox into internal arrays
		*/
		int i;
		for( i=0; i<iCount; i++ )
			{
			SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
			Internal_addLine( szBuf );
			};
        };
	
    void Internal_updateButtons( WPARAM wParam, LPARAM lParam,
	    HWND hWndEdit1, HWND hWndEdit2, const char *pData )
        {
		if ( GET_WM_COMMAND_CMD(wParam, lParam)!=EN_CHANGE )
			{ return; };
	
		int iExistingIndex = -1;
		assert( pData );
		int iChangeToIndex = -1;
		int iIndex = 0;
		if ( *pData )
			{
			for( DblListIterator i( lHostnames );!i.BadIndex(); i++,iIndex++)
				{
				PIString *pString = (PIString *)i.Current();
				assert( pString );
				if ( !PIUtil_strncmpi( (*pString), pData, strlen( pData ) ) )
					{
					iChangeToIndex = iIndex;
					/*
					** If this is a full index, select it
					*/
					if ( !stricmp( (*pString), pData ) )
						{ iExistingIndex = iIndex; };
					break;
					};
				};
			};
	
		/*
		** Remove listbox selection or scroll to correct line
		** as appropriate 
		*/
		SendMessage( hWndList, LB_SETCURSEL, iChangeToIndex, 0 );
	
		/*
		** Set buttons
		*/
		if ( ::SendMessage( hWndEdit1, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndDocumentRoot, EM_LINELENGTH, 0, 0 )==0 ||
		    ::SendMessage( hWndAdministrator, EM_LINELENGTH, 0, 0 )==0 )
			{
			if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
			if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
			}
		else if ( SendMessage( hWndEdit2, EM_LINELENGTH, 0, 0 )!=0 )
			{
			if ( iExistingIndex>-1 )	/* --- disable add, enable replace --- */
				{
				if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
				if ( !IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, TRUE );
				}
			else						/* --- disable replace, enable add --- */
				{
				if ( !IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, TRUE );
				if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
				};
			};
	
		/*
		** If there is a selected item in the listbox then
		** enable delete, otherwise disable delete	
		*/
		if ( SendMessage( hWndList, LB_GETCURSEL, 0, 0 )==LB_ERR )
			{ EnableWindow( hWndDelete, FALSE ); }
		else
	        { EnableWindow( hWndDelete, TRUE ); };
        };
	
    BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
		LPARAM lParam)
	    {
		enum { BUF_SIZE=1023 };
		char szBuf[BUF_SIZE+1];
		int aTabstop[4];
	
		switch (uMsg)
			{
			case WM_INITDIALOG:
				HourglassOn();
				{
				const char *pTmp;
				int iTmp;
				hWndList = GetDlgItem( hDlg, IDC_LIST );
                if ( iIsIP )
                    {
                    hWndIPAddress = GetDlgItem( hDlg, IDC_IPADDRESS );
                    };
				hWndHostname = GetDlgItem( hDlg, IDC_HOSTNAME );
				hWndDocumentRoot= GetDlgItem( hDlg, IDC_DOCUMENTROOT );
				hWndAdd = GetDlgItem( hDlg, IDC_ADD );
				hWndReplace = GetDlgItem( hDlg, IDC_REPLACE );
				hWndDelete= GetDlgItem( hDlg, IDC_DELETE );
				hWndAdministrator= GetDlgItem( hDlg, IDC_ADMINISTRATOR );
	
                if ( iIsIP )
                    {
                    assert( hWndIPAddress );
                    };
				assert( hWndList && hWndHostname && hWndDocumentRoot );
				assert( hWndAdd && hWndReplace && hWndDelete &&
                    hWndAdministrator );
	
				EnableWindow( hWndAdd, FALSE );
				EnableWindow( hWndReplace, FALSE );
				EnableWindow( hWndDelete, FALSE );
	
				/*
				** Limit edit field lengths, really just a sanity thing
				*/
                if ( iIsIP )
                    {
                    SendMessage( hWndIPAddress, EM_SETLIMITTEXT, 256, 0 );
                    };
				SendMessage( hWndHostname, EM_SETLIMITTEXT, 256, 0 );
				SendMessage( hWndDocumentRoot, EM_SETLIMITTEXT, 256, 0 );
				SendMessage( hWndAdministrator, EM_SETLIMITTEXT, 256, 0 );
	
				/*
				** Setup tabstops
				*/
				aTabstop[0] = 0;
				aTabstop[1] = 60;
				aTabstop[2] = 120;
				aTabstop[3] = 170;
				SendMessage( hWndList, LB_SETTABSTOPS, 4, (LPARAM)&aTabstop );
	
				/*
				** Load all data into the listbox
				*/
				iTmp = 0;
				for(;; iTmp++ )
					{
					pTmp = M_LookupValueEx( pConfig, SECTION, 
						sVariable, iTmp );
					if ( !pTmp )
						{ break; };
	
					SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pTmp );
					};
				Internal_reloadArrays();
				}
				HourglassOff();
				break;
	
			case WM_DESTROY:
				Internal_clearArrays();
				break;
	
	        case WM_COMMAND:
				switch (GET_WM_COMMAND_ID(wParam, lParam))
					{
					case IDC_ADD:
	                    {
	                    /*
	                    ** Verify fields are valid
	                    */
	                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
	                        { break; };
	
						int i, j;
						*szBuf = '\0';
                        i = 0;
	
                        if ( iIsIP )
                            {
						    SendMessage( hWndIPAddress, WM_GETTEXT, BUF_SIZE,
	                            (LPARAM)szBuf );
						    for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';
                                i++ );
						    if ( szBuf[i] ) { szBuf[i] = '\0'; };
						    lIPs.Append( (DblList::type)PI_NEW(
                                PIString(szBuf )));
				        	szBuf[i++] = '\t';
                            };
	
						SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lHostnames.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						SendMessage( hWndDocumentRoot, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lDocumentRoots.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						SendMessage( hWndAdministrator, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lAdministrators.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
	
						SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );
	
						/*
						** Clear edit fields, focus to list
						*/
                        if ( iIsIP )
                            {
                            SendMessage( hWndIPAddress, WM_SETTEXT, 0,
                                (LPARAM)"" );
                            };
						SendMessage( hWndHostname, WM_SETTEXT, 0,
                            (LPARAM)"" );
						SendMessage( hWndDocumentRoot, WM_SETTEXT, 0,
                            (LPARAM)"" );
						SendMessage( hWndAdministrator, WM_SETTEXT, 0,
                            (LPARAM)"" );
						Internal_reloadArrays();
						int iIndex = SendMessage( hWndList,
                            LB_FINDSTRINGEXACT, 0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR )
                            SendMessage( hWndList, LB_SETTOPINDEX, iIndex, 0 );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;
		
					case IDC_REPLACE:
	                    {
	                    /*
	                    ** Verify fields are valid
	                    */
	                    if ( !Internal_verifyFields( hDlg, szBuf, BUF_SIZE ) )
	                        { break; };
	
						/*
						** Get selected item index 
						*/
						int iCurSel = SendMessage( hWndList, LB_GETCURSEL,
                            0, 0 );
	
						/*
						** Get text to add and add it to internal lists only
						*/
						int i, j;
						*szBuf = '\0';
                        i = 0;
	
                        if ( iIsIP )
                            {
						    SendMessage( hWndIPAddress, WM_GETTEXT, BUF_SIZE,
	                            (LPARAM)szBuf );
						    for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';
                                i++ );
						    if ( szBuf[i] ) { szBuf[i] = '\0'; };
						    lIPs.Append( (DblList::type)PI_NEW(
                                PIString(szBuf )));
						    szBuf[i++] = '\t';
                            };
	
						SendMessage( hWndHostname, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lHostnames.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						SendMessage( hWndDocumentRoot, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i;i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t';i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lDocumentRoots.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
						szBuf[i++] = '\t';
	
						SendMessage( hWndAdministrator, WM_GETTEXT, BUF_SIZE-i,
	                        (LPARAM)&(szBuf[i]) );
						for(j=i; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++);
						if ( szBuf[i] ) { szBuf[i] = '\0'; };
						lAdministrators.Append( (DblList::type)PI_NEW(
	                        PIString( &(szBuf[j]) ) ) );
	
						/*
						** Delete the item from the list and internal lists
						*/
						SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
                        if ( iIsIP )
                            {
						    PI_DELETE( (PIString *)lIPs.Detach( iCurSel) );
                            };
						PI_DELETE( (PIString *)lHostnames.Detach( iCurSel) );
						PI_DELETE( (PIString *)lDocumentRoots.Detach( iCurSel));
						PI_DELETE( (PIString *)lAdministrators.Detach(iCurSel));
						
						/*
						** Add new item to listbox
						*/
						SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );
	
						/*
						** Clear edit fields, focus to list
						*/
                        if ( iIsIP )
                            {
                            SendMessage( hWndIPAddress, WM_SETTEXT, 0,
                                (LPARAM)"" );
                            };
						SendMessage( hWndHostname, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndDocumentRoot, WM_SETTEXT,0,(LPARAM)"");
						SendMessage( hWndAdministrator,WM_SETTEXT,0,(LPARAM)"");
						Internal_reloadArrays();
						int iIndex = SendMessage( hWndList, LB_FINDSTRINGEXACT,
                            0, (LPARAM)szBuf );
						if ( iIndex!=LB_ERR ) SendMessage( hWndList,
	                        LB_SETTOPINDEX, iIndex, 0 );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;
	
					case IDC_DELETE:
						{
						int iCurSel = SendMessage( hWndList, LB_GETCURSEL,
                            0, 0 );
						SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
                        if ( iIsIP )
                            {
						    PI_DELETE( (PIString *)lIPs.Detach( iCurSel) );
                            };
						PI_DELETE( (PIString *)lHostnames.Detach( iCurSel) );
						PI_DELETE( (PIString *)lDocumentRoots.Detach(iCurSel));
						PI_DELETE( (PIString *)lAdministrators.Detach(iCurSel));
				
						/*
						** Clear edit fields, focus to list
						*/
                        if ( iIsIP )
                            {
                            SendMessage( hWndIPAddress, WM_SETTEXT, 0,
                                (LPARAM)"" );
                            };
						SendMessage( hWndHostname, WM_SETTEXT, 0, (LPARAM)"" );
						SendMessage( hWndDocumentRoot, WM_SETTEXT, 0,
                            (LPARAM)"" );
						SendMessage( hWndAdministrator, WM_SETTEXT, 0,
                            (LPARAM)"" );
						SetFocus( hWndList );
						Changed( hDlg );
						};
						break;

					case IDC_IPADDRESS:
		                if ( iIsIP )
							{
	                        *szBuf = '\0';
		                    SendMessage( hWndIPAddress, WM_GETTEXT, BUF_SIZE,
			                    (LPARAM)szBuf ); 
						    Internal_updateButtons( wParam, lParam, hWndIPAddress, 
							    hWndHostname, szBuf );
						    };
						break;

					case IDC_HOSTNAME:
						*szBuf = '\0';
						SendMessage(
					        iIsIP ? hWndIPAddress : hWndHostname,
						    WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf ); 
						Internal_updateButtons( wParam, lParam, hWndHostname, 
							iIsIP ? hWndIPAddress : hWndDocumentRoot, szBuf );
						break;

					case IDC_DOCUMENTROOT:
						*szBuf = '\0';
						SendMessage(
				            iIsIP ? hWndIPAddress : hWndHostname,
					        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						Internal_updateButtons( wParam, lParam, hWndDocumentRoot, 
							hWndHostname, szBuf );
						break;

					case IDC_ADMINISTRATOR:
						*szBuf = '\0';
						SendMessage(
				            iIsIP ? hWndIPAddress : hWndHostname,
					        WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
						Internal_updateButtons( wParam, lParam, hWndHostname, 
							hWndDocumentRoot, szBuf );
						break;

					case IDC_LIST:
						if ( GET_WM_COMMAND_CMD(wParam, lParam)==LBN_SELCHANGE )
							{
							int iSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
							if ( iSel!=LB_ERR )
								{
								assert( iSel>=0 && iSel<lHostnames.Size() );
								EnableWindow( hWndDelete, TRUE );
		                        if ( iIsIP )
			                        {
								    SendMessage( hWndIPAddress, WM_SETTEXT, 0,
									    (LPARAM)(const char *)*((PIString *)
						                    lIPs[iSel]) );
							        };
								SendMessage( hWndHostname, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)
		                                lHostnames[iSel]) );
								SendMessage( hWndDocumentRoot, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)
					                    lDocumentRoots[iSel]) );
								SendMessage( hWndAdministrator, WM_SETTEXT, 0,
									(LPARAM)(const char *)*((PIString *)
										lAdministrators[iSel]) );
								};
							};
						break;

					default:;		
					}
				break;

			case WM_NOTIFY:
				switch( ((NMHDR *)lParam)->code )
					{
					case PSN_QUERYCANCEL:
						return SHEET->Cancel(pInterface, hDlg);

					case PSN_KILLACTIVE:
						{
						int i;
						int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

						/*
						** Save all date
						*/
						/* --- remove old ones first --- */
						M_DeleteValue( pConfig, SECTION, sVariable );
						for( i=0; i<iCount; i++ )
							{
							*szBuf = '\0';
							szBuf[BUF_SIZE] = '\0';
							SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
							M_AddValue( pConfig, SECTION, sVariable, szBuf );
							};
						};
						SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
						break;

					default:
						return (FALSE);
					};
				break;

			default:
				return (FALSE);
			};	

		return (TRUE);
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK fnDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
    if ( uMsg==WM_INITDIALOG )
        {
        ::SetWindowLong( hDlg, DWL_USER, ((LPPROPSHEETPAGE)lParam)->lParam );
        };

    VHostsPage *pPage = (VHostsPage *)::GetWindowLong( hDlg, DWL_USER );
    return pPage ? pPage->DialogProc( hDlg, uMsg, wParam, lParam ) : FALSE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int VHostsPage_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return IFaceBase_constructor( pObj, PI_NEW( VHostsPage( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name VHostsPageClass
		Type LogicExtension
		Library IntrFace
		OnClassLoad IFaceBase_onClassLoad
		Constructor VHostsPage_constructor
		CopyConstructor IFaceBase_copyConstructor
		Destructor IFaceBase_destructor
		Execute IFaceBase_execute
	</Class>

	<Object>
		Name IPVHostsPage
		Class VHostsPageClass
        Flag "IP"
	</Object>

	<Object>
		Name NonIPVHostsPage
		Class VHostsPageClass
        Flag "NonIP"
	</Object>

/*___+++CNF_END+++___*/
#endif

