/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Parse/MarkFile.h,v $
 * $Date: 2001/04/07 13:30:21 $
 *
 Description:
\*____________________________________________________________________________*/
/* $HeaderTop:$ */

#ifndef MARKFILE_H_
#define MARKFILE_H_

#include <assert.h>
#include <ctype.h>

#include "PICompat.h"
#include "PIString.h"
#include "FileRead.h"

#define PI_EOF (127)

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class MarkupFile
{
private:
	FileReader *pReader;
	PIString sFileName;
	ostream &os;
	long lX, lY;
	inline void NF()	{ lY=1; lX=0; };
	inline void NL()	{ lY++; lX=0; };
	enum {	start_line,
		normal, 
		comment,
		start_markup, 	
		markup_c1,
		markup_c2,
		markup_e1,
		markup_e2,
		markup,
		markdown,
		start_markup_comment,
		markup_comment,
		nl
		} eInputState;
	void emit( bool bSkip );
	int iErrors;
	int iWarnings;

	/* --- input block management --- */
	char *ppPtr;		/* current pointer */
	char *pePtr;		/* end pointer */

	/* --- internal buffer --- */
	enum { BUF_SIZE=4095 };
	char szBuf[BUF_SIZE+1];
	int iLen;
	inline
	bool PutCh(char c);
	inline
	int GetCh2();
	inline
	int GetCh1();
	inline
	int GetCh0();
	inline
	int GetChWithBS();
	inline
	int GetCh();
	bool Init();
	bool InternalLex( bool bHTML );

	/* --- push back buffer --- */
	unsigned char cLastCharacter;

protected:
	inline bool LexConfFile()	{ return InternalLex( false ); };
	inline bool LexHTMLFile()	{ return InternalLex( true ); };
	bool LexBlocksFile();

	/* --- response fn's --- */
	virtual void Comment()	{};
	virtual void Text()	{};
	virtual void CommentMarkup()	{};
	virtual void Markup()	{};
	virtual void Markdown()	{};
	virtual void Abort()=0;	// cannot continue
	virtual bool Overflow()=0;

	/* --- data on current position and file --- */
	inline const char *GetPtr()	{ return szBuf; };
	inline long GetX()		{ return lX; };
	inline long GetY()		{ return lY; };
	inline int GetLen()		{ assert(iLen); return iLen-1; };
	const PIString &GetFileName()		{ return sFileName; };

public:
	MarkupFile( const char *pFileName, ostream &os );
	virtual ~MarkupFile();
	
	/* --- misc --- */
	ostream &Error();
	ostream &Warning();
	inline int GetNumErrors()	{ return iErrors; };
	inline int GetNumWarnings()	{ return iWarnings; };
	inline ostream &GetOs()		{ return os; };
};


#endif	/* MARKFILE_H_ */

