/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Pi3API/_Pi3Expr.h,v $
 * $Date: 2001/04/07 13:30:22 $
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#ifndef _PI3EXPR_H_
#define _PI3EXPR_H_

#if defined(PI3_INTERNAL)	/* --- Internal use only --- */

#include <assert.h>

#include "PIHTTP.h"
#include "PIString.h"
#include "DblList.h"
#include "Pi3Expr.h"

/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Base class for components. Components are parts of an expression that
	known how to render themselves.
\*____________________________________________________________________________*/
class Context;
class ComponentBase
{
public:
	virtual ~ComponentBase()	{};
	virtual void WriteDebugTree( int iIndent ) const = 0;
	virtual int Write( Context &tContext, char *pszBuffer, int iLength )
		const = 0;
	virtual int IsOK() const = 0;
	virtual int IsStatic() const = 0;	/* component may be pre-evaluated */
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class TokenSource;
class _Pi3Expression
{
private:
	ComponentBase *pExpr;
	PIString sExpression;
	int iOK;

	/* --- internal methods --- */
	_Pi3Expression( const _Pi3Expression & )
		{
		/* --- forbid copy constructor --- */
		assert( 0 );
		};

public:

	/* --- empty expression --- */
	_Pi3Expression();

	/* --- internal use --- */
	_Pi3Expression( TokenSource &tSource, FnPi3Write *pParameterFunctions,
		PIString &sError );

	/* --- general use --- */
	_Pi3Expression( const char *pExpr, FnPi3Write *pParameterFunctions,
		PIString &sError );
	/*
	** NOTE: This is a workaround for a linkage bug
	*/
	static void *Compiler_Bug_Alloc( void *p1, void *p2, void *p3 );

	~_Pi3Expression();
	inline void WriteDebugTree( int iIndent ) const
		{
		if ( pExpr )
			{ pExpr->WriteDebugTree( iIndent ); }
		};

	inline int Write( Context &tContext, char *pszBuffer, int iMaxLen ) const
		{
		return pExpr ? pExpr->Write( tContext, pszBuffer, iMaxLen ) : 0;
		};

	int Write( PIHTTP &tPIHTTP, void *pData, char *pszBuffer, int iMaxLen )
		const;

	inline int IsOK() const		{ return iOK; };
	inline int IsStatic() const	{ return !pExpr || pExpr->IsStatic(); };
};

#endif /* defined(PI3_INTERNAL) */

#endif /* _PI3EXPR_H_ */

