/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Platform/POSIXOSE.h,v $
 * $Date: 2001/04/07 13:30:22 $
 *
 Description:
\*____________________________________________________________________________*/
//$HeaderTop:$

#ifndef POSIXOSE_H_
#define POSIXOSE_H_

#include "OSConfig.h"

#if defined(CONFIG_OS_POSIX)	/* --- this is POSIX code only --- */

#include <assert.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/time.h>

#if	defined(CONFIG_DLL)
#	if	defined(CONFIG_DLL_DL)
#		include <dlfcn.h>
#	elif defined(CONFIG_DLL_GNU_DLD)
#		include <dld.h>
#	endif
#endif	/* CONFIG_DLL */

#include "PICompat.h"
#include "OSEntity.h"
#include "FSEntity.h"
#include "Allocate.h"
#include "FSType.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
class PISockAddr : public sockaddr_in {};
class PIHostEnt : public hostent {};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class POSIXFileSysType : public FileSysType
{
public:
	POSIXFileSysType()		{};
	virtual const char *GetName() const	{ return "UNIX File System"; };
	virtual const char *GetExtension( const char *pName ) const;
	virtual char DirectorySeperatorCharacter() const	{ return '/'; };
	virtual bool IsDirectorySeperatorCharacter( char c ) const
		{ return c=='/'; };
	virtual bool CaseSensitive() const		{ return true; };
	virtual bool NormaliseFileName( const PIString &sBefore,
        PIString &sAfter ) const	{ sAfter=sBefore; return true; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
    Derived from abstract class DLL, this class encapsulates a 
	unix shared library.
\*____________________________________________________________________________*/
class UnixDLL : public DLL
{
private:
	void *pDlHandle;
	PIString sPath;
	PIString sLastErrorMsg;
#if defined(CONFIG_DLL_GNU_DLD)
	static bool bHaveInitializedDld;
#endif
	
public:
	UnixDLL( const char * );
	~UnixDLL();
	virtual void *GetAddress( const char *pSymbolName );
	virtual const char *GetLastErrorDescription() const
					{ return sLastErrorMsg; };
	virtual bool IsOK() const	{ return pDlHandle!=0; };
    virtual void *GetSystemHandle() const { return (void *)pDlHandle; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class POSIXMappedFile : public MappedFile
{
private:
	int iFileDes;
	size_t tLength;
	caddr_t tAddress;
	void Setup( const char *pFileName, size_t tTheLength );

public:
	POSIXMappedFile( FileSystemEntity &tFSE );
	~POSIXMappedFile();
	virtual void *Lock( int *piLength )
		{
		if ( piLength )
			{ *piLength = tLength; };
		return tAddress;
		};
	virtual bool UnLock()	{ return true; };
	virtual bool IsOK()		{ return iFileDes!=-1 && tAddress!=0; };
};

/*____________________________________________________________________________*\
 *
 Class:
 Description:
	Derived from Timer, this abstract class implements a microsecond
	timer on unix operating systems.
\*____________________________________________________________________________*/
class UnixTimer : public Timer
{
private:
	struct timeval tv;
	bool bOK;
	
public:
	UnixTimer();
	virtual bool Reset();
	virtual bool Elapsed( long *plSeconds, long *plMicroSeconds );
	virtual bool IsOK()						{ return bOK; };
};

#endif	/* defined(CONFIG_OS_POSIX) */

#endif	/* POSIXOSE_H_ */

