/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/HandBase.cpp,v $
 * $Date: 2001/07/28 17:21:36 $
 *
 Description:
	Base class for all C++ handlers, provides a more comfortable interface
	above the raw Pi3 HTTP interface.

\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <ctype.h>

#include "HandBase.h"
#include "PIString.h"
#include "DeQuote.h"
#include "PICompat.h"
#include "HTTPCore.h"
#include "PIHTTP.h"

PIObject *HandlerBaseServlet::pContainerObj = 0;
int HandlerBaseServlet::ServletRefCount = 0;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
HandlerBaseServlet::HandlerBaseServlet( PIObject *pTheObject, const char *aLogName )
:
	pObject( pTheObject ),
	iOK( 1 ),
	pLogName( aLogName )
{
	/* ---
	**** Load up the ServletContainer ****
	(there should be a better place to do this, because	ServletContainer
	is both a singleton and a static member of Servlet)
	--- */
	if ( !pContainerObj ) {
		const char *pObjLine = PIConfig_lookupValue( PIObject_getConfigurationDB( pObject ),
			KEY_CONF_SERVLETCONTEXT, 0, 0 );

		if ( !pObjLine ) {
			PIOStrStream os;
			os << "Servlet: Missing directive '" << KEY_CONF_SERVLETCONTEXT << "'" << ends;
			CONFIG_ERR( pObject, os.str() );
			iOK = 0;
		} else {
			pContainerObj =  PIObject_loadFromLine( PIObject_getDB( pObject ),
				PIObject_getConfigurationDB( pObject ), pObjLine );

			if (!pContainerObj) {
				PIOStrStream os;
				os << "Servlet: Can't load ServletContext '" << KEY_CONF_SERVLETCONTEXT << "'" << ends;
				CONFIG_ERR( pObject, os.str() );
				iOK = 0;
			};
		};
	};
	ServletRefCount++;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void HandlerBaseServlet::ReadParameters( int iArgc, const char *ppArgv[] )
{
	if ( iOK ) {
		iOK = PIObject_readParameters( pObject, iArgc, ppArgv,
			ParameterFn, this );
	};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
HandlerBaseServlet::~HandlerBaseServlet()
{

	/* ---
	**** Destroy the ServletContainer ****
	(there should be a better place to do this, because	ServletContainer
	is both a singleton and a static member of Servlet)
	--- */
	ServletRefCount--;

	if ( !ServletRefCount && pContainerObj ) {
		PIObject_delete( pContainerObj, 0, 0 );
		pContainerObj = 0;
	};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int HandlerBaseServlet::Execute( int iArgc, const char *ppArgv[] )
{
	PI_MARK;

	if ( iArgc!=1 )
		{ return PIAPI_ERROR; };	

	PIHTTP *pPIHTTP = (PIHTTP *)ppArgv[0];	
	if ( !pPIHTTP )
		{ return PIAPI_ERROR; };
	int iPhase = (int)pPIHTTP->ciPhase;

	int iRet = Handle(
		iPhase,
		*pPIHTTP,
		*( pPIHTTP->pBuffer )
		);
	PI_MARK;
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseServlet_execute( PIObject *pObj, int iArgc,
		const char *ppArgv[] )
{
	if ( !pObj ) return PIAPI_ERROR;
	return ((HandlerBaseServlet *)PIObject_getUserData(pObj))->
		Execute( iArgc, ppArgv );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseServlet_onClassLoad( PIClass_LoadAction eLoad,
	PIDB *pDB )
{
	switch( eLoad )
		{
	case STARTUP:
/*
// Don't work because we are in the class configuration, not the object !!!
// Destroy ServletContainer on SHUTDOWN also won't work because the container's
// class is already destroyed !!!
			{
			PIConfig *pConf = PIConfig_findConfiguration( pDB );
			const char *pObjectName = PIConfig_lookupValue( pConf,
				KEY_CONF_CONTAINER, 0, 0 );
			PIObject *pObj = PIObject_load( pDB, pConf, pObjectName, 0, 0 );
			if ( pObj )
				{
				HandlerBaseServlet::setContainerObj( pObj );
				};
			break;
			};
*/
		default:;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int HandlerBaseServlet_constructor( PIObject *pObj, HandlerBaseServlet *pHandler )
{
	if ( !( pHandler && pHandler->IsOK() ) )
		{
		delete pHandler;
		return PIAPI_ERROR;
		};

	if ( PIObject_setUserData( pObj, pHandler ) )
		{
		delete pHandler;
		CONFIG_ERR( pObj, "HandlerBaseServlet_constructor: PIObject_setUserData() failed" );
		return PIAPI_ERROR;
		};

	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseServlet_copyConstructor( PIObject *, int, const char *[] )
{
	/* --- forbid copy constructor --- */
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HandlerBaseServlet_destructor( PIObject *pObj, int,
	const char *[] )
{
	delete (HandlerBaseServlet *)PIObject_getUserData( pObj );
	return PIAPI_COMPLETED;
}

