/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/org/pi3/jsp/core/Pi3HttpJspPage.java,v $
 * $Date: 2001/08/19 18:40:44 $
 *
 Description:
	Abstract extension of jsp API HttpJspPage interface, parent of all Pi3Web Jsp's.

\*____________________________________________________________________________*/
 
package org.pi3.jsp.core;

import java.lang.String;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.HttpJspPage;
import java.io.IOException;

/**
 * This is the interface that a JSP processor-generated class for the
 * HTTP protocol must satisfy.
 */

abstract public class Pi3HttpJspPage implements HttpJspPage {

    private ServletConfig config;

    final public void init(ServletConfig config) throws ServletException {
        this.config = config;
        jspInit();
    }

    final public ServletConfig getServletConfig() {
        return config;
    }

    /* This one is not final so it can be overridden by a more precise method */
    public String getServletInfo() {
        return "Pi3Web superclass for an HTTP JSP";
    }

    final public void destroy() {
        jspDestroy();
    }

    public void jspInit() {}
    public void jspDestroy() {}

    /**
    * The entry point into service, not to be used but abstract method
	* has to be implemented.
    */
    final public void service(ServletRequest req, ServletResponse res)
      throws ServletException, IOException {
        ServletException e = new ServletException(
"Invalid attempt to call method service(ServletRequest req, ServletResponse res).");
        throw e;
    }

    /**
    * The entry point into service, used by Pi3Web.
    */
    final public void service(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {
        _jspService(req, res);
	}

    /**
    * abstract method to be provided by the JSP processor in the subclass
    * Must be defined in subclass.
    */
    abstract public void _jspService(HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException;
}
