/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in
	the documentation and/or other materials provided with the
	distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
	mentioning features must contain the following
	acknowledgment:
	"This product includes software developed by John Roy
	and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Servlet/org/pi3/servlet/util/Pi3BufferedOutputStream.java,v $
 * $Date: 2001/04/07 13:30:23 $
 *
 Description:
	Enhanced BufferedOutputStream class.

\*____________________________________________________________________________*/

package org.pi3.servlet.util;

import java.io.IOException;
import java.io.OutputStream;
import java.io.BufferedOutputStream;


/**
 *
 * Extends the {@link java.io.BufferedOutputStream} class
 * to implement buffer size configuration. 
 *
 *
 * @author 	Holger Zimmermann [zimpel@pi3.org]
 * @version	$Revision: 1.3 $
 *
 *
 */

final public class Pi3BufferedOutputStream extends BufferedOutputStream {


    private boolean flushed = false;


	/** An overriden constructor of the base class */
    public Pi3BufferedOutputStream(OutputStream out) {
        super(out);
    }




    /** An overriden constructor of the base class */
    public Pi3BufferedOutputStream(OutputStream out, int size) throws IOException {
        super(out, size);
        if ( size != buf.length ) {
            try {
                setBufferSize(size);
            }
            catch (IOException e) {
                throw e;
            };
        };
    }




    /**
     *
     * Returns the size of the currently allocated buffer for this
	 * stream.
     *
     * @return          the buffer size of this buffered stream
     *
     *
     * @see             #resetBuffer
	 *
     */

    public int getBufferSize() {
	    return buf.length;
	}




    /**
     *
     * This function resets the content in the buffer of the stream.
	 * The buffer is empty after calling the method. However, any
	 * flushed content cannot be reset.
     *
     *
     * @see             #getCount
     *
     */

	public void resetBuffer() {
	    count = 0;
	}



    /**
     *
     * This function returns the number of data in the buffer.
     *
     * @return          the number of data in the buffer of this stream
     *
     *
     * @see             #resetBuffer
     *
     */

	public int getCount() {
	    return count;
	}


    /**
     *
     * This function allocates a new buffer of <code>size</code> for the
	 * stream. Any data in the old buffer will be flushed before the new
	 * buffer is allocated.
     *
     * @param size      the size of the new buffer 
	 *
     *
     * @see             #getBufferSize
     *
     */

    public void setBufferSize(int size) throws IOException {

        if ( size == buf.length ) return;

        if ( count > 0 ) {
            try {
                flush();
            }
            catch (IOException e) {
                throw e;
            };
        };
        buf = new byte[size];
		count = 0;
    }

	public void flush() throws IOException {
		super.flush();
	    flushed = true;
	}

	public boolean isFlushed() { return flushed; }
}
